Package: optconerrf
Title: Optimal Monotone Conditional Error Functions
Version: 1.0.3
Authors@R: 
  c(person(given = "Morten", family = "Dreher", email = "morten.dreher@outlook.de", role = c("aut", "cre")),
    person(given = "Werner", family = "Brannath", role = c("aut", "cph"), comment = c(ORCID = "0000-0002-8622-3904")),
    person(given = "Cornelia Ursula", family = "Kunz", role = "ctb", comment = c(ORCID = "0000-0002-8900-9401")),
    person(given = "Johanna", family = "zur Verth", role = "aut"))
Description: Design and analysis of confirmatory adaptive clinical trials using the optimal conditional error framework according to Brannath and Bauer (2004) <doi:10.1111/j.0006-341X.2004.00221.x>. An extension to the optimal conditional error function using interim estimates as described in Brannath and Dreher (2024) <doi:10.48550/arXiv.2402.00814> and functions to ensure that the resulting conditional error function is non-increasing are also available.
Imports: ggplot2, methods
License: GPL (>= 3)
Encoding: UTF-8
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
URL: https://github.com/morten-dreher/optconerrf
BugReports: https://github.com/morten-dreher/optconerrf/issues
Config/roxygen2/version: 8.1.0
NeedsCompilation: no
Packaged: 2026-09-03 19:45:39 UTC; Morten
Author: Morten Dreher [aut, cre],
  Werner Brannath [aut, cph] (ORCID:
    <https://orcid.org/0000-0002-8622-3904>),
  Cornelia Ursula Kunz [ctb] (ORCID:
    <https://orcid.org/0000-0002-8900-9401>),
  Johanna zur Verth [aut]
Maintainer: Morten Dreher <morten.dreher@outlook.de>
Repository: CRAN
Date/Publication: 2026-09-03 20:10:03 UTC

# Generated by roxygen2: do not edit by hand

S3method(plot,TrialDesignOptimalConditionalError)
S3method(print,PowerResultsOptimalConditionalError)
S3method(print,SimulationResultsOptimalConditionalError)
S3method(print,TrialDesignOptimalConditionalError)
S3method(summary,TrialDesignOptimalConditionalError)
export(getDesignOptimalConditionalErrorFunction)
export(getExpectedSecondStageInformation)
export(getLikelihoodRatio)
export(getNu)
export(getNuPrime)
export(getOptimalConditionalError)
export(getOverallPower)
export(getPsi)
export(getQ)
export(getSecondStageInformation)
export(getSimulationResults)
importFrom(methods,new)
importFrom(stats,
  pnorm,
  qnorm,
  rnorm
)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# optconerrf

<!-- badges: start -->

[![R-CMD-check](https://github.com/morten-dreher/optconerrf/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/morten-dreher/optconerrf/actions/workflows/R-CMD-check.yaml)
[![CRAN RStudio mirror
downloads](https://cranlogs.r-pkg.org/badges/grand-total/optconerrf?color=blue)](https://r-pkg.org/pkg/optconerrf)
[![CRAN
checks](https://badges.cranchecks.info/summary/optconerrf.svg)](https://cran.r-project.org/web/checks/check_results_optconerrf.html)
<!-- badges: end -->

The goal of `optconerrf` is to enable the use of adaptive two-stage
clinical trial designs with the optimal conditional error function
approach proposed by Brannath & Bauer (2004). The optimal conditional
error function minimises the expected second-stage sample
size/information under the assumption that the true effect size is
known. As an extension to the original methodology, `optconerrf` allows
the effect size used to plan the second stage to depend on the interim
data. In `optconerrf`, tested hypotheses for a treatment effect $\Delta$
are always one-sided of the form $H_0: \Delta \leq 0$ and
$H_1: \Delta > 0$.

## Installation

You can install the development version of optconerrf from
[GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("morten-dreher/optconerrf")
```

Installing from CRAN can be done via:

``` r
install.packages("optconerrf")
```

## Example

A trial design object with the following properties:

- overall type I error rate 0.025 (`alpha = 0.025`)
- early efficacy boundary 0.0001 (`alpha1 = 0.0001`, p-value scale)
- binding futility boundary 0.5 (`alpha0 = 0.5`, p-value scale)
- conditional power 0.9 (`conditionalPower = 0.9`)
- assumed treatment effect of 0.25 (`delta1 = 0.25`)
- no use of interim estimate (i.e., always using `delta1` to plan the
  secon d stage, `useInterimEstimate = FALSE`)
- first-stage information 100 (`firstStageInformation = 100`)
- fixed likelihood ratio distribution
  (`likelihoodRatioDistribution = "fixed"`) with effect 0.25
  (`deltaLR = 0.25`) (i.e., minimising the expected second-stage
  information under the assumption that the true effect size is 0.25)

can be generated by the code below.

``` r
library(optconerrf)
trialDesign <- 
  getDesignOptimalConditionalErrorFunction(
    alpha = 0.025,
    alpha1 = 0.0001,
    alpha0 = 0.5,
    conditionalPower = 0.9,
    delta1 = 0.25,
    useInterimEstimate = FALSE,
    firstStageInformation = 100,
    likelihoodRatioDistribution = "fixed",
    deltaLR = 0.25
  )
```

The design object can then be passed to other central package functions,
such as `getExpectedSecondStageInformation()`:

``` r
getExpectedSecondStageInformation(
  trialDesign,
  likelihoodRatioDistribution = "fixed",
  deltaLR = 0)
#> [1] 99.84901
```

The above code calculates the expected second-stage information of the
design for a scenario under the null hypothesis (`deltaLR = 0`).

`optconerrf` also implements `plot()` and `print()` generics for the
trial design object:

``` r
plot(trialDesign)
```

<img src="man/figures/README-unnamed-chunk-3-1.png" width="100%" />

``` r
print(trialDesign)
#> Optimal Conditional Error Function Design: 
#>  
#> General design parameters: 
#>   Overall significance level: 0.025 
#>   First-stage efficacy boundary (p-value scale): 1e-04 
#>   Binding first-stage futility boundary (p-value scale): 0.5 
#> 
#> Conditional power specification: 
#>   Target conditional power: 0.9 
#>   Alternative: 0.25 
#>   First-stage non-centrality parameter: 2.5 
#>   First-stage information: 100 
#> 
#> Likelihood ratio specification: 
#>   Fixed parameter(s) in likelihood ratio:  0.25 
#>   Parameter weights:  1 
#> 
#> Level constant: 
#>   Constant: 7.079328 
#>   Searched on interval: [0, 10]
```

<!-- README.md is generated from README.Rmd. Please edit that file -->

# optconerrf

<!-- badges: start -->

[![R-CMD-check](https://github.com/morten-dreher/optconerrf/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/morten-dreher/optconerrf/actions/workflows/R-CMD-check.yaml)
[![CRAN RStudio mirror
downloads](https://cranlogs.r-pkg.org/badges/grand-total/optconerrf?color=blue)](https://r-pkg.org/pkg/optconerrf)
[![CRAN
checks](https://badges.cranchecks.info/summary/optconerrf.svg)](https://cran.r-project.org/web/checks/check_results_optconerrf.html)
<!-- badges: end -->

The goal of `optconerrf` is to enable the use of adaptive two-stage
clinical trial designs with the optimal conditional error function
approach proposed by Brannath & Bauer (2004). The optimal conditional
error function minimises the expected second-stage sample
size/information under the assumption that the true effect size is
known. As an extension to the original methodology, `optconerrf` allows
the effect size used to plan the second stage to depend on the interim
data.

## Installation

You can install the development version of optconerrf from
[GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("morten-dreher/optconerrf")
```

Installing from CRAN can be done via:

``` r
install.packages("optconerrf")
```

## Example

A design object with the following properties:

- overall type I error rate 0.025 (`alpha = 0.025`)
- early efficacy boundary 0.0001 (`alpha1 = 0.0001`, p-value scale)
- binding futility boundary 0.5 (`alpha0 = 0.5`, p-value scale)
- conditional power 0.9 (`conditionalPower = 0.9`)
- assumed treatment effect of 0.25 (`delta1 = 0.25`)
- no use of interim estimate (i.e., always using `delta1` to plan the
  secon d stage, `useInterimEstimate = FALSE`)
- first-stage information 100 (`firstStageInformation = 100`)
- fixed likelihood ratio distribution
  (`likelihoodRatioDistribution = "fixed"`) with effect 0.25
  (`deltaLR = 0.25`) (i.e., minimising the expected second-stage
  information under the assumption that the true effect size is 0.25)

can be generated by the code below.

``` r
library(optconerrf)
trialDesign <- 
  getDesignOptimalConditionalErrorFunction(
    alpha = 0.025,
    alpha1 = 0.0001,
    alpha0 = 0.5,
    conditionalPower = 0.9,
    delta1 = 0.25,
    useInterimEstimate = FALSE,
    firstStageInformation = 100,
    likelihoodRatioDistribution = "fixed",
    deltaLR = 0.25
  )
```

The design object can then be passed to other central package functions,
such as `getExpectedSecondStageInformation()`:

``` r
getExpectedSecondStageInformation(
  trialDesign,
  likelihoodRatioDistribution = "fixed",
  deltaLR = 0)
#> [1] 99.84901
```

The above code calculates the expected second-stage information of the
design for a scenario under the null hypothesis (`deltaLR = 0`).

`optconerrf` also implements `plot()` and `print()` generics:

``` r
plot(trialDesign)
```

<img src="man/figures/README-unnamed-chunk-3-1.png" width="100%" />

``` r
print(trialDesign)
#> Optimal Conditional Error Function Design: 
#>  
#> General design parameters: 
#>   Overall significance level: 0.025 
#>   First-stage efficacy boundary (p-value scale): 1e-04 
#>   Binding first-stage futility boundary (p-value scale): 0.5 
#> 
#> Conditional power specification: 
#>   Target conditional power: 0.9 
#>   Alternative: 0.25 
#>   First-stage non-centrality parameter: 2.5 
#>   First-stage information: 100 
#> 
#> Likelihood ratio specification: 
#>   Fixed parameter(s) in likelihood ratio:  0.25 
#>   Parameter weights:  1 
#> 
#> Level constant: 
#>   Constant: 7.079328 
#>   Searched on interval: [0, 10]
```

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.dim = c(6,4)
)

## ----setup--------------------------------------------------------------------
library(optconerrf)

## -----------------------------------------------------------------------------
trialDesignConstraints <- getDesignOptimalConditionalErrorFunction(
  alpha = 0.025,
  alpha1 = 0.000158,
  alpha0 = 0.5,
  conditionalPower = 0.9,
  delta1 = 0.25,
  useInterimEstimate = FALSE,
  likelihoodRatioDistribution = "maxlr",
  firstStageInformation = 85,
  minimumSecondStageInformation = 85/2,
  maximumSecondStageInformation = 85*2
)

## ----fig.dim = c(6,4)---------------------------------------------------------
# type = 2 for second-stage information
plot(trialDesignConstraints, type = 2) +
  ggplot2::geom_hline(yintercept = c(85/2, 85*2), linetype = "dashed", colour = "gray")

## -----------------------------------------------------------------------------
trialDesignConstraintsConditionalError <- getDesignOptimalConditionalErrorFunction(
  alpha = 0.025,
  alpha1 = 0.000158,
  alpha0 = 0.5,
  conditionalPower = 0.9,
  delta1 = 0.25,
  useInterimEstimate = FALSE,
  likelihoodRatioDistribution = "maxlr",
  firstStageInformation = 85,
  minimumConditionalError =  0.02396,
  maximumConditionalError = 0.36382
)

## -----------------------------------------------------------------------------
getExpectedSecondStageInformation(
  trialDesignConstraints, 
  likelihoodRatioDistribution = "fixed", 
  deltaLR = 0.25
)
getExpectedSecondStageInformation(
  trialDesignConstraintsConditionalError, 
  likelihoodRatioDistribution = "fixed",
  deltaLR = 0.25
)

## -----------------------------------------------------------------------------
trialDesignInterimEstimate <- 
  getDesignOptimalConditionalErrorFunction(
    alpha = 0.025,
    alpha1 = 0.000158,
    alpha0 = 0.5,
    conditionalPower = 0.9,
    useInterimEstimate = TRUE,
    delta1Min = 0.15,
    delta1Max = 0.35,
    firstStageInformation = 85,
    likelihoodRatioDistribution = "maxlr"
  )

## -----------------------------------------------------------------------------
plot(trialDesignInterimEstimate, type = 1)
plot(trialDesignInterimEstimate, type = 2)

## -----------------------------------------------------------------------------
myConditionalPowerFunction <- function(firstStagePValue) {
  return(pnorm(1 - firstStagePValue))
}

## -----------------------------------------------------------------------------
p1 <- seq(0, 1, 0.01)
plot(p1, myConditionalPowerFunction(p1), type = "l")
abline(h = myConditionalPowerFunction(c(0, 1)), col = "blue")

## -----------------------------------------------------------------------------
trialDesignConditionalPowerFunction <- 
  getDesignOptimalConditionalErrorFunction(
    alpha = 0.025,
    alpha1 = 0.000158,
    alpha0 = 0.5,
    conditionalPowerFunction = myConditionalPowerFunction,
    delta1 = 0.25,
    useInterimEstimate = FALSE,
    firstStageInformation = 100,
    likelihoodRatioDistribution = "maxlr"
  )

## -----------------------------------------------------------------------------
plot(trialDesignConditionalPowerFunction, type = 1)

## -----------------------------------------------------------------------------
plot(trialDesignConditionalPowerFunction, type = 2)

## -----------------------------------------------------------------------------
combinedDesign <- 
  getDesignOptimalConditionalErrorFunction(
    alpha = 0.025,
    alpha1 = 0.000158,
    alpha0 = 0.5,
    conditionalPowerFunction = myConditionalPowerFunction,
    delta1Min = 0.25,
    useInterimEstimate = TRUE,
    firstStageInformation = 100,
    likelihoodRatioDistribution = "maxlr",
    minimumSecondStageInformation = 50
  )

## -----------------------------------------------------------------------------
plot(combinedDesign)

#' Internal variation of \code{getPsi()} used by \code{findLevelConstant()}
#' @name getInnerPsi
#'
#' @description Calculate psi for the given scenario, incl. constant
#' @details Internal function called by \code{getIntegral()} that calculates a point value of \code{getPsi()}.
#'
#' @template param_firstStagePValue_integrate
#' @template param_constant_integrate
#' @template param_design
#'
#' @return Point value of \code{getPsi()}.
#' @keywords internal

getInnerPsi <- function(firstStagePValue, constant, design) {

  # If monotonisation constants provided, perform non-increasing transformation
  if(design$enforceMonotonicity) {
    Q <- getMonotoneFunction(
      x = firstStagePValue, fun = getQ, argument = "firstStagePValue",
      design = design)
  } else {
    Q <- getQ(firstStagePValue = firstStagePValue, design = design)
  }

  # Calculate the value to be supplied to getPsi
  inner <- -exp(constant)/Q

  # Identify conditional power to be supplied to getPsi and to be used for (information) constraints
  # Check if conditional power function should be used
  if(!is.null(suppressWarnings(body(design$conditionalPowerFunction)))) {
    conditionalPower <- design$conditionalPowerFunction(firstStagePValue)
  } else {
    conditionalPower <- design$conditionalPower
  }

  #Calculate the constraints
  C_max <- getConstraintC_max(firstStagePValue = firstStagePValue, design = design)
  C_min <- getConstraintC_min(firstStagePValue = firstStagePValue, design = design)

  return(pmin(pmax(getPsi(nuPrime = inner, conditionalPower = conditionalPower, design = design, firstStagePValue = firstStagePValue),
                   C_min), C_max))
}

#' Calculate Nu
#' @name getNu
#'
#' @description Calculate the factor which relates \eqn{\alpha_2} to the second-stage information for given conditional power.
#'
#' @details Note that this function uses factor 1 instead of factor 2 (Brannath & Bauer 2004). This has no impact on the optimal conditional error function, as constant factors are absorbed by the level constant \eqn{c_0}. \cr
#' The calculation is:
#' \deqn{\nu(\alpha_2(p_1)) = (\Phi^{-1}(1-\alpha_2(p_1)) + \Phi^{-1}(CP))^2.}
#' @template param_alpha_cerr
#' @template param_conditionalPower
#'
#' @return Factor linking information and \eqn{\alpha_2}.
#' @export
#'
#' @examples
#' getNu(alpha = 0.05, conditionalPower = 0.9)
#'
#' # Returns 0 if alpha exceeds conditionalPower
#' getNu(alpha = 0.8, conditionalPower = 0.7)
#'
#' @template reference_optimal

getNu <- function(alpha, conditionalPower) {
  nu <- 0
  if(any(alpha > conditionalPower)) {
    warning("alpha/conditional error should not exceed conditionalPower. Information is otherwise 0")
  }
  else{
    nu <- (qnorm(1-alpha)+qnorm(conditionalPower))^2
  }
  return(nu)
}

getNu <- Vectorize(FUN = getNu, vectorize.args = c("alpha", "conditionalPower"))

#' Create a design object for the optimal conditional error function.
#' @name getDesignOptimalConditionalErrorFunction
#'
#' @description This function returns a design object which contains all important parameters for the specification of the optimal conditional error function.
#' The returned object is of class \code{TrialDesignOptimalConditionalError} and can be passed to other package functions.
#'
#'
#' @details
#' The design object contains the information required to determine the specific setting of the optimal conditional error function and can be passed to other package functions.
#' From the given user specifications, the constant to achieve level condition for control of the overall type I error rate as well as the constants to ensure a non-increasing optimal CEF (if required) are automatically calculated.
#'
#' @section Likelihood ratio distribution:
#' To calculate the optimal conditional error function, an assumption about the true parameter under which the second-stage information is to be minimised is required.
#' Various options are available and can be specified via the argument \code{likelihoodRatioDistribution}:
#' \itemize{
#'    \item \code{likelihoodRatioDistribution="fixed"}: calculates the likelihood ratio for a fixed \eqn{\Delta}. The non-centrality parameter of the likelihood ratio \eqn{\vartheta} is then computed as \code{deltaLR}*\code{sqrt(firstStageInformation)} and the likelihood ratio is calculated as:
#'          \deqn{l(p_1) = e^{\Phi^{-1}(1-p_1)\vartheta - \vartheta^2/2}.} \code{deltaLR} may also contain multiple elements, in which case a weighted likelihood ratio is calculated for the given values. Unless positive weights that sum to 1 are provided by the argument \code{weightsDeltaLR}, equal weights are assumed.
#'    \item \code{likelihoodRatioDistribution="normal"}: calculates the likelihood ratio for a normally distributed prior of \eqn{\vartheta} with mean \code{deltaLR}*\code{sqrt(firstStageInformation)} (\eqn{\mu}) and standard deviation \code{tauLR}*\code{sqrt(firstStageInformation)} (\eqn{\sigma}). The parameters \code{deltaLR} and \code{tauLR} must be specified on the mean difference scale.
#'          \deqn{l(p_1) = (1+\sigma^2)^{-\frac{1}{2}}\cdot e^{-(\mu/\sigma)^2/2 + (\sigma\Phi^{-1}(1-p_1) + \mu/\sigma)^2 / (2\cdot (1+\sigma^2))}}
#'    \item \code{likelihoodRatioDistribution="exp"}: calculates the likelihood ratio for an exponentially distributed prior of \eqn{\vartheta} with mean \code{kappaLR}*\code{sqrt(firstStageInformation)} (\eqn{\eta}). The likelihood ratio is then calculated as:
#'          \deqn{l(p_1) = \kappa \cdot \sqrt{2\pi} \cdot e^{(\Phi^{-1}(1-p_1)-\eta)^2/2} \cdot \Phi(\Phi^{-1}(1-p_1)-\eta)}
#'    \item \code{likelihoodRatioDistribution="unif"}: calculates the likelihood ratio for a uniformly distributed prior of \eqn{\vartheta} on the support \eqn{[0, \Delta\cdot\sqrt{I_1}]}, where \eqn{\Delta} is specified as \code{deltaMaxLR} and \eqn{I_1} is the \code{firstStageInformation}.
#'          \deqn{l(p_1) = \frac{\sqrt{2\pi}}{\Delta\cdot\sqrt{I_1}} \cdot e^{\Phi^{-1}(1-p_1)^2/2} \cdot (\Phi(\Delta\cdot\sqrt{I_1} - \Phi^{-1}(1-p_1))-p_1)}
#'    \item \code{likelihoodRatioDistribution="maxlr"}: the non-centrality parameter \eqn{\vartheta} is estimated from the data and no additional parameters must be specified. The likelihood ratio is estimated from the data as:
#'          \deqn{l(p_1) = e^{max(0, \Phi^{-1}(1-p_1))^2/2}}
#'          The maximum likelihood ratio is always restricted to effect sizes \eqn{\vartheta \geq 0} (corresponding to \eqn{p_1 \leq 0.5}).
#' }
#'
#' @section Effect for conditional power:
#' For the treatment effect at which the target conditional power should be achieved, either a fixed effect or an interim estimate can be used.
#' The usage of a fixed effect is indicated by setting \code{useInterimEstimate=FALSE}, in which case the fixed effect is provided by \code{delta1} on the mean difference scale or by \code{ncp1} on the non-centrality parameter scale (i.e., \code{delta1*sqrt(firstStageInformation)}).
#' For an interim estimate, specified by \code{useInterimEstimate=TRUE}, a lower cut-off for the interim estimate must be provided, either by \code{delta1Min} on the mean difference scale, or \code{ncp1Min} on the non-centrality parameter scale.
#' In addition, an upper limit of the estimate may be analogously provided by \code{delta1Max} or \code{ncp1Max}.
#'
#' @section Sample size and information:
#' The first-stage information of the trial design must be specified to allow for calculations between the mean difference and non-centrality parameter scale.
#' It is provided to the design object via \code{firstStageInformation}. \cr
#' Listed below are some examples for the calculation between information (\eqn{I_1}) and sample size:
#' \itemize{
#'  \item One-sample z-test with \eqn{n} total patients: \eqn{I_1 = \frac{n}{\sigma^2}}, where \eqn{\sigma^2} is the variance of an individual observation
#'
#' \item Balanced two-sample z-test with \eqn{n_1} patients per group: \eqn{I_1 = \frac{1}{2}\cdot\frac{n_1}{\sigma^2}}, where \eqn{\sigma^2} is the common variance
#'
#' \item General two-sample z-test with \eqn{n_1}, \eqn{n_2} patients per group: \eqn{I_1 = 1/(\frac{\sigma_1^2}{n_1}+\frac{\sigma_2^2}{n_2})}, where \eqn{\sigma_1^2}, \eqn{\sigma_2^2} are the group-wise variances
#' }
#'
#' @section Monotonicity:
#' By default, the optimal conditional error function returned by \code{getDesignOptimalConditionalErrorFunction()} is transformed to be non-increasing in the first-stage p-value \eqn{p_1} if found to be increasing on any interval.
#' The necessary intervals and constants for the transformation are calculated by \code{getMonotonisationConstants()}.
#' Although not recommended for the operating characteristics of the design, the transformation may be omitted by setting \code{enforceMonotonicity=FALSE}.
#'
#' @section Constraints:
#' In some applications, it may be feasible to restrict the optimal conditional error function by a lower and/or upper limit.
#' These constraints can be directly implemented on the function by using the arguments \code{minimumConditionalError} and \code{maximumConditionalError}.
#' By default, \code{minimumConditionalError=0} and \code{maximumConditionalError=1}, i.e., no constraints are applied.
#' The constraints may also be specified on the second-stage information via \code{minimumSecondStageInformation} and \code{maximumSecondStageInformation}.
#' If both \code{minimumConditionalError} and \code{maximumSecondStageInformation} respectively \code{maximumConditionalError} and \code{minimumSecondStageInformation}
#' are provided, both constraints will be applied.
#'
#' @section Level constant:
#' The level constant is determined by the helper function \code{getLevelConstant()}. It is identified using the \code{uniroot()} function and by default, the interval between 0 and 10 is searched for the level constant.
#' In specific settings, the level constant may lie outside of this interval. In such cases, the search interval can be changed by altering the parameters \code{levelConstantMinimum} and \code{levelConstantMaximum}. \cr
#' If inappropriate constraints to the optimal conditional error function are provided via \code{minimumConditionalError} and \code{maximumConditionalError}
#' or \code{minimumSecondStageInformation} and \code{maximumSecondStageInformation}, it may be impossible to find a level constant which exhausts the full alpha level.
#'
#' @section Generic functions:
#' The \code{print()} and \code{plot()} functions are available for objects of class \code{TrialDesignOptimalConditionalError}.
#' For details, see \code{?print.TrialDesignOptimalConditionalError} and \code{?plot.TrialDesignOptimalConditionalError}.
#'
#' @template param_alpha
#' @template param_alpha1
#' @template param_alpha0
#' @template param_conditionalPower
#' @template param_conditionalPowerFunction
#' @template param_ncp1
#' @template param_ncp1Min
#' @template param_ncp1Max
#' @template param_delta1
#' @template param_delta1Min
#' @template param_delta1Max
#' @template param_likelihoodRatioDistribution
#' @template param_firstStageInformation
#' @template param_useInterimEstimate
#' @template param_minimumSecondStageInformation
#' @template param_maximumSecondStageInformation
#' @template param_minimumConditionalError
#' @template param_maximumConditionalError
#' @template param_levelConstantMinimum
#' @template param_levelConstantMaximum
#' @template param_enforceMonotonicity
#' @param ... Additional arguments required for the specification of the likelihood ratio.
#'
#' @importFrom methods new
#'
#' @return An object of class \code{TrialDesignOptimalConditionalError}, which can be passed to other package functions.
#'
#'
#' @examples
#' \donttest{
#' # Create a single-arm design with fixed parameter for the likelihood ratio
#' # and a fixed effect for conditional power. 80 patients are observed in the
#' # first-stage (firstStageInformation = 80 in the one-sample test, variance 1).
#' # The second-stage information is restricted to be between 40 and 160.
#' getDesignOptimalConditionalErrorFunction(
#'   alpha = 0.025, alpha1 = 0.001, alpha0 = 0.5, conditionalPower = 0.9,
#'   delta1 = 0.25, likelihoodRatioDistribution = "fixed", deltaLR = 0.25,
#'   firstStageInformation = 80, useInterimEstimate = FALSE,
#'   minimumSecondStageInformation = 40, maximumSecondStageInformation = 160
#' )
#'
#' # Create a design comparing two groups using the maximum likelihood ratio
#' # and an interim estimate for the effect for conditional power.
#' # 160 patients per arm are observed in the first stage
#' # (firstStageInformation = 80 in the balanced two-sample test, variance 1).
#' getDesignOptimalConditionalErrorFunction(
#'   alpha = 0.025, alpha1 = 0.001, alpha0 = 0.5, conditionalPower = 0.9,
#'   delta1Min = 0.25, likelihoodRatioDistribution = "maxlr",
#'   firstStageInformation = 80, useInterimEstimate = TRUE
#' )
#' }
#'
#' @export
#'
#' @template reference_optimal
#' @template reference_monotone
#'
getDesignOptimalConditionalErrorFunction <- function(
  alpha,
  alpha1,
  alpha0,
  conditionalPower = NA_real_,
  delta1 = NA_real_,
  delta1Min = NA_real_,
  delta1Max = Inf,
  ncp1 = NA_real_,
  ncp1Min = NA_real_,
  ncp1Max = Inf,
  useInterimEstimate = TRUE,
  firstStageInformation,
  likelihoodRatioDistribution,
  minimumSecondStageInformation = 0,
  maximumSecondStageInformation = Inf,
  minimumConditionalError = 0,
  maximumConditionalError = 1,
  conditionalPowerFunction = NA,
  levelConstantMinimum = 0,
  levelConstantMaximum = 10,
  enforceMonotonicity = TRUE,
  ...
) {
  design <- new(
    "TrialDesignOptimalConditionalError",
    alpha = alpha,
    alpha1 = alpha1,
    alpha0 = alpha0,
    conditionalPower = conditionalPower,
    delta1 = delta1,
    delta1Min = delta1Min,
    delta1Max = delta1Max,
    ncp1 = ncp1,
    ncp1Min = ncp1Min,
    ncp1Max = ncp1Max,
    firstStageInformation = firstStageInformation,
    useInterimEstimate = useInterimEstimate,
    likelihoodRatioDistribution = likelihoodRatioDistribution,
    minimumSecondStageInformation = minimumSecondStageInformation,
    maximumSecondStageInformation = maximumSecondStageInformation,
    minimumConditionalError = minimumConditionalError,
    maximumConditionalError = maximumConditionalError,
    conditionalPowerFunction = conditionalPowerFunction,
    levelConstantMinimum = levelConstantMinimum,
    levelConstantMaximum = levelConstantMaximum,
    enforceMonotonicity = enforceMonotonicity,
    ... = ...
  )

  return(design)
}

test_that("SecondStageInformation correctly calculated", {
  # Without using the interim estimate

  Info_compare <- c(101.7700, 141.9709, 168.1198, 188.4785, 205.6950, 220.9849, 235.0241, 248.2418, 260.9416, 273.3614)

  design_fixed_delta <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025, alpha1 = 0.000158, alpha0 = 0.5, conditionalPower = 0.9,
    likelihoodRatioDistribution = "fixed", ncp1 = sqrt(170 / 2) * 0.25, deltaLR = 0.25,
    firstStageInformation = 170 / 2, useInterimEstimate = FALSE
  )

  Info_fixed_delta <- getSecondStageInformation(firstStagePValue = seq(0.05, 0.5, 0.05), design = design_fixed_delta)

  expect_equal(Info_compare, Info_fixed_delta, tolerance = 1e-4)

  # Skip remaining tests on cran
  skip_on_cran()

  # With using the interim estimate

  Info_compare <- c(2.327416, 3.834037, 5.861998, 8.889864, 12.836328)

  design_maxlr <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025, alpha1 = 0.000158, alpha0 = 0.5, conditionalPower = 0.8,
    likelihoodRatioDistribution = "maxlr", delta1Min = 0.7003963,
    firstStageInformation = 1, useInterimEstimate = TRUE
  )

  Info_maxlr <- getSecondStageInformation(firstStagePValue = seq(0.05, 0.25, 0.05), design = design_maxlr)

  expect_equal(Info_compare, Info_maxlr, tolerance = 1e-4)
})

test_that("Simple level constant correct", {
  # 1)---
  # Design without interim estimate and with early stopping rules
  design <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025,
    alpha1 = 0.001,
    alpha0 = 0.5,
    conditionalPower = 0.9,
    delta1 = 0.5,
    useInterimEstimate = FALSE,
    firstStageInformation = 4,
    likelihoodRatioDistribution = "maxlr",
    enforceMonotonicity = TRUE
  )

  # This is the type I error at the second stage
  secondStageIntegral <- integrate(
    f = getOptimalConditionalError,
    lower = 0.001,
    upper = 0.5,
    design
  )$value

  # The type I error at the second stage must be alpha-alpha1
  expect_equal(secondStageIntegral, 0.024, tolerance = 1e-10)

  # Skip remaining tests on CRAN
  skip_on_cran()

  # 2)---
  # Design without interim estimate and without early stopping rules
  design <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.05,
    alpha1 = 0,
    alpha0 = 1,
    conditionalPower = 0.9,
    delta1 = 0.5,
    useInterimEstimate = FALSE,
    firstStageInformation = 4,
    likelihoodRatioDistribution = "maxlr",
    enforceMonotonicity = TRUE
  )

  # This is the type I error at the second stage
  secondStageIntegral <- integrate(
    f = getOptimalConditionalError,
    lower = 0,
    upper = 1,
    design
  )$value

  # The type I error at the second stage must be alpha-alpha1
  expect_equal(secondStageIntegral, 0.05, tolerance = 1e-10)

  # 3)---
  # A design WITH interim estimate, but without monotonicity (uses normal integration routine)
  # This is expected to cause a warning that is not informative in this case and thus suppressed
  design <- suppressWarnings(getDesignOptimalConditionalErrorFunction(
    alpha = 0.025,
    alpha1 = 0.001,
    alpha0 = 0.5,
    conditionalPower = 0.9,
    useInterimEstimate = TRUE,
    delta1Min = 0.2,
    delta1Max = 0.4,
    firstStageInformation = 4,
    likelihoodRatioDistribution = "maxlr",
    enforceMonotonicity = FALSE
  ))

  # This is the type I error at the second stage
  secondStageIntegral <- integrate(
    f = getOptimalConditionalError,
    lower = 0.001,
    upper = 0.5,
    design
  )$value

  # The type I error at the second stage must be alpha-alpha1
  expect_equal(secondStageIntegral, 0.024, tolerance = 1e-10)
})


#This test is deprecated because the root finding of the level constant fails because
#of a too strict lower constraint!

# testthat::test_that(desc = "Error for root finding works", code = {
#   testthat::expect_error(
#     getDesignOptimalConditionalErrorFunction(
#       alpha = 0.025,
#       alpha1 = 0.001,
#       alpha0 = 0.5,
#       conditionalPower = 0.9,
#       delta1 = 0.25,
#       useInterimEstimate = FALSE,
#       firstStageInformation = 40,
#       likelihoodRatioDistribution = "maxlr",
#       minimumConditionalError = 0.05,
#       maximumConditionalError = 0.1
#     ),
#     regexp = "Root finding for level constant failed"
#   )
# })

testthat::test_that(desc = "Error for incorrect specification of alpha, CP, alpha1, alpha0 works", code = {
  testthat::expect_error(
    getDesignOptimalConditionalErrorFunction(
      alpha = 0.01,
      alpha1 = 0,
      alpha0 = 0.015,
      conditionalPower = 0.6,
      delta1 = 0.25,
      useInterimEstimate = FALSE,
      firstStageInformation = 40,
      likelihoodRatioDistribution = "maxlr"
    ),
    regexp = "must exceed alpha"
  )
})

testthat::test_that(desc = "Error for incorrect specification of CP fun, alpha1, alpha0 works", code = {
  testthat::expect_error(
    getDesignOptimalConditionalErrorFunction(
      alpha = 0.025,
      alpha1 = 0.0001,
      alpha0 = 0.2,
      conditionalPowerFunction = function(x) {
        (0.5 - x) * 0.2
      },
      delta1 = 0.25,
      useInterimEstimate = FALSE,
      firstStageInformation = 40,
      likelihoodRatioDistribution = "maxlr"
    ),
    regexp = "Integral over conditional power function"
  )
})

test_that("Class objects correctly created", {
  # Create a design object using interim estimate
  design_interim <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025, alpha1 = 0.001, alpha0 = 0.5,
    conditionalPower = 0.9, useInterimEstimate = TRUE,
    delta1Min = 0.2, delta1Max = 1,
    firstStageInformation = 4, likelihoodRatioDistribution = "maxlr",
    enforceMonotonicity = TRUE
  )
  # Class correct?
  expect_s4_class(design_interim, "TrialDesignOptimalConditionalError")

  # Fields correct?
  expect_equal(design_interim$alpha, 0.025)
  expect_equal(design_interim$alpha1, 0.001)
  expect_equal(design_interim$alpha0, 0.5)
  expect_equal(design_interim$conditionalPower, 0.9)
  expect_equal(design_interim$useInterimEstimate, TRUE)
  expect_equal(design_interim$firstStageInformation, 4)
  expect_equal(design_interim$likelihoodRatioDistribution, "maxlr")
  expect_equal(design_interim$enforceMonotonicity, TRUE)
  expect_equal(design_interim$delta1Min, 0.2)
  expect_equal(design_interim$delta1Max, 1)
  expect_equal(design_interim$ncp1Min, 0.2 * sqrt(4), tolerance = 1e-8)
  expect_equal(design_interim$ncp1Max, 1 * sqrt(4), tolerance = 1e-8)

  # Skip remaining tests on CRAN
  skip_on_cran()

  # Create a design object using a fixed effect for CP
  design_fixed <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025, alpha1 = 0.001, alpha0 = 0.5,
    conditionalPower = 0.9, delta1 = 0.5, useInterimEstimate = FALSE,
    firstStageInformation = 4, likelihoodRatioDistribution = "fixed",
    deltaLR = 1, enforceMonotonicity = FALSE
  )
  # Class correct?
  expect_s4_class(design_fixed, "TrialDesignOptimalConditionalError")

  # Fields correct?
  expect_equal(design_fixed$alpha, 0.025)
  expect_equal(design_fixed$alpha1, 0.001)
  expect_equal(design_fixed$alpha0, 0.5)
  expect_equal(design_fixed$conditionalPower, 0.9)
  expect_equal(design_fixed$delta1, 0.5)
  expect_equal(design_fixed$useInterimEstimate, FALSE)
  expect_equal(design_fixed$firstStageInformation, 4)
  expect_equal(design_fixed$likelihoodRatioDistribution, "fixed")
  expect_equal(design_fixed$deltaLR, 1)
  expect_equal(design_fixed$enforceMonotonicity, FALSE)
  expect_equal(design_fixed$ncp1, 0.5 * sqrt(4), tolerance = 1e-8)
})

test_that("Correct results for expected second stage information", {
  design_fixed_delta <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025, alpha1 = 0.000158, alpha0 = 0.5, conditionalPower = 0.9,
    likelihoodRatioDistribution = "fixed", ncp1 = sqrt(170 / 2) * 0.25, deltaLR = 0.25,
    firstStageInformation = 170 / 2, useInterimEstimate = FALSE
  )

  res0 <- getExpectedSecondStageInformation(design = design_fixed_delta, likelihoodRatioDistribution = "fixed", deltaLR = 0)
  res1_15 <- getExpectedSecondStageInformation(design = design_fixed_delta, likelihoodRatioDistribution = "fixed", deltaLR = 0.125)
  res2_3 <- getExpectedSecondStageInformation(design = design_fixed_delta, likelihoodRatioDistribution = "fixed", deltaLR = 0.25)

  # Skip remaining tests on CRAN
  skip_on_cran()

  expect_equal(res0, 95.89078, tolerance = 1e-4)
  expect_equal(res1_15, 115.9472, tolerance = 1e-4)
  expect_equal(res2_3, 55.40431, tolerance = 1e-4)

  design_fixed_delta0 <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025, alpha1 = 0.000158, alpha0 = 0.5, conditionalPower = 0.9,
    likelihoodRatioDistribution = "fixed", ncp1 = sqrt(170 / 2) * 0.25, deltaLR = 0,
    firstStageInformation = 170 / 2, useInterimEstimate = FALSE
  )

  res0 <- getExpectedSecondStageInformation(design = design_fixed_delta0, likelihoodRatioDistribution = "fixed", deltaLR = 0)
  res1_15 <- getExpectedSecondStageInformation(design = design_fixed_delta0, likelihoodRatioDistribution = "fixed", deltaLR = 0.125)
  res2_3 <- getExpectedSecondStageInformation(design = design_fixed_delta0, likelihoodRatioDistribution = "fixed", deltaLR = 0.25)

  expect_equal(res0, 68.62581, tolerance = 1e-4)
  expect_equal(res1_15, 119.2098, tolerance = 1e-4)
  expect_equal(res2_3, 122.479, tolerance = 1e-4)

  design_uniform <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025, alpha1 = 0.000158, alpha0 = 0.5, conditionalPower = 0.9,
    likelihoodRatioDistribution = "unif",
    ncp1 = sqrt(170 / 2) * 0.25, deltaMaxLR = 2 * sqrt(170 / 2) * 0.25,
    firstStageInformation = 170 / 2, useInterimEstimate = FALSE
  )

  res0 <- getExpectedSecondStageInformation(design = design_uniform, likelihoodRatioDistribution = "fixed", deltaLR = 0)
  res1_15 <- getExpectedSecondStageInformation(design = design_uniform, likelihoodRatioDistribution = "fixed", deltaLR = 0.125)
  res2_3 <- getExpectedSecondStageInformation(design = design_uniform, likelihoodRatioDistribution = "fixed", deltaLR = 0.25)

  expect_equal(res0, 81.44857, tolerance = 1e-4)
  expect_equal(res1_15, 107.1427, tolerance = 1e-4)
  expect_equal(res2_3, 58.24021, tolerance = 1e-4)

  design_normal <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025, alpha1 = 0.000158, alpha0 = 0.5, conditionalPower = 0.9,
    likelihoodRatioDistribution = "normal", deltaLR = 0.25, tauLR = 1.4 / sqrt(170 / 2), # tauLR standard deviation
    ncp1 = sqrt(170 / 2) * 0.25,
    firstStageInformation = 170 / 2, useInterimEstimate = FALSE
  )

  res0 <- getExpectedSecondStageInformation(design = design_normal, likelihoodRatioDistribution = "fixed", deltaLR = 0)
  res1_15 <- getExpectedSecondStageInformation(design = design_normal, likelihoodRatioDistribution = "fixed", deltaLR = 0.125)
  res2_3 <- getExpectedSecondStageInformation(design = design_normal, likelihoodRatioDistribution = "fixed", deltaLR = 0.25)

  expect_equal(res0, 83.1371, tolerance = 1e-4)
  expect_equal(res1_15, 107.9471, tolerance = 1e-4)
  expect_equal(res2_3, 57.40248, tolerance = 1e-4)

  design_exponential <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025, alpha1 = 0.000158, alpha0 = 0.5, conditionalPower = 0.9,
    likelihoodRatioDistribution = "exp", kappaLR = (1 / 2.3) / sqrt(170 / 2),
    ncp1 = sqrt(170 / 2) * 0.25,
    firstStageInformation = 170 / 2, useInterimEstimate = FALSE
  )

  res0 <- getExpectedSecondStageInformation(design = design_exponential, likelihoodRatioDistribution = "fixed", deltaLR = 0)
  res1_15 <- getExpectedSecondStageInformation(design = design_exponential, likelihoodRatioDistribution = "fixed", deltaLR = 0.125)
  res2_3 <- getExpectedSecondStageInformation(design = design_exponential, likelihoodRatioDistribution = "fixed", deltaLR = 0.25)

  expect_equal(res0, 77.32785, tolerance = 1e-4)
  expect_equal(res1_15, 105.2796, tolerance = 1e-4)
  expect_equal(res2_3, 61.74755, tolerance = 1e-4)

  design_maxlr <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025, alpha1 = 0.000158, alpha0 = 0.5, conditionalPower = 0.9,
    likelihoodRatioDistribution = "maxlr", deltaLR = sqrt(170 / 2) * 0.25,
    ncp1 = sqrt(170 / 2) * 0.25,
    firstStageInformation = 170 / 2, useInterimEstimate = FALSE
  )

  res0 <- getExpectedSecondStageInformation(design = design_maxlr, likelihoodRatioDistribution = "fixed", deltaLR = 0)
  res1_15 <- getExpectedSecondStageInformation(design = design_maxlr, likelihoodRatioDistribution = "fixed", deltaLR = 0.125)
  res2_3 <- getExpectedSecondStageInformation(design = design_maxlr, likelihoodRatioDistribution = "fixed", deltaLR = 0.25)

  expect_equal(res0, 78.01145, tolerance = 1e-4)
  expect_equal(res1_15, 106.3943, tolerance = 1e-4)
  expect_equal(res2_3, 60.6867, tolerance = 1e-4)
})

test_that("Correct result for optimal conditional error function", {
  setting_1 <- read.csv(test_path("testdata", "ocef_setting1.csv"))

  # Comparison conditional error functions - fixed delta
  design_fixed_delta_1 <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025,
    alpha1 = 0.000158,
    alpha0 = 0.5,
    conditionalPower = 0.8,
    likelihoodRatioDistribution = "fixed",
    ncp1 = 1,
    deltaLR = 1 / sqrt(170 / 2),
    firstStageInformation = 170 / 2,
    useInterimEstimate = FALSE
  )
  cond_error <- getOptimalConditionalError(
    setting_1$p1[-1],
    design_fixed_delta_1
  )
  expect_equal(cond_error, setting_1$fixed[-1], tolerance = 1e-4)
  testthat::expect_equal(
    object = getOptimalConditionalError(
      firstStagePValue = c(0, 1),
      design = design_fixed_delta_1
    ),
    expected = c(1, 0)
  )

  # Skip remaining tests on CRAN
  skip_on_cran()

  # Comparison conditional error functions - maxlr

  design_maxlr_1 <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025,
    alpha1 = 0.000158,
    alpha0 = 0.5,
    conditionalPower = 0.8,
    likelihoodRatioDistribution = "maxlr",
    ncp1 = 1,
    deltaLR = 1 / sqrt(170 / 2),
    firstStageInformation = 170 / 2,
    useInterimEstimate = FALSE
  )
  cond_error <- getOptimalConditionalError(setting_1$p1[-1], design_maxlr_1)
  expect_equal(cond_error, setting_1$maxlr[-1], tolerance = 1e-4)

  # Comparison conditional error functions - normal

  design_normal_1 <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025,
    alpha1 = 0.000158,
    alpha0 = 0.5,
    conditionalPower = 0.8,
    likelihoodRatioDistribution = "normal",
    ncp1 = 1,
    deltaLR = 1 / sqrt(170 / 2),
    tauLR = 0.6 / sqrt(170 / 2),
    firstStageInformation = 170 / 2,
    useInterimEstimate = FALSE
  )
  cond_error <- getOptimalConditionalError(setting_1$p1[-1], design_normal_1)
  expect_equal(cond_error, setting_1$normal[-1], tolerance = 1e-4)

  # Comparison conditional error functions - exponential

  design_exp_1 <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025,
    alpha1 = 0.000158,
    alpha0 = 0.5,
    conditionalPower = 0.8,
    likelihoodRatioDistribution = "exp",
    ncp1 = 1,
    deltaLR = 1 / sqrt(170 / 2),
    kappaLR = 1 / sqrt(170 / 2),
    firstStageInformation = 170 / 2,
    useInterimEstimate = FALSE
  )
  cond_error <- getOptimalConditionalError(setting_1$p1[-1], design_exp_1)
  expect_equal(cond_error, setting_1$exp[-1], tolerance = 1e-4)

  # Comparison conditional error functions - uniform

  design_unif_1 <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025,
    alpha1 = 0.000158,
    alpha0 = 0.5,
    conditionalPower = 0.8,
    likelihoodRatioDistribution = "unif",
    ncp1 = 1,
    deltaLR = 1 / sqrt(170 / 2),
    deltaMaxLR = 2 * 1 / sqrt(170 / 2),
    firstStageInformation = 170 / 2,
    useInterimEstimate = FALSE
  )
  cond_error <- getOptimalConditionalError(setting_1$p1[-1], design_unif_1)
  expect_equal(cond_error, setting_1$unif[-1], tolerance = 1e-4)

  # Comparison conditional error functions - maxlr with interim estimate

  design_maxlr_interim_1 <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025,
    alpha1 = 0.000158,
    alpha0 = 0.5,
    conditionalPower = 0.8,
    likelihoodRatioDistribution = "maxlr",
    firstStageInformation = 1,
    delta1Min = (qnorm(0.8) - qnorm(0.025)) / 4,
    useInterimEstimate = TRUE
  )
  cond_error <- getOptimalConditionalError(
    setting_1$p1[-1],
    design_maxlr_interim_1
  )
  expect_equal(cond_error, setting_1$maxlr_interim[-1], tolerance = 1e-4)

  # Comparison conditional error functions - maxlr with constraints

  C_min <- pnorm(qnorm(0.9) - sqrt(2) * 2.3)
  C_max <- pnorm(qnorm(0.9) - sqrt(0.5) * 2.3)

  design_maxlr_constr_1 <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025,
    alpha1 = 0.000158,
    alpha0 = 0.5,
    conditionalPower = 0.9,
    likelihoodRatioDistribution = "maxlr",
    ncp1 = 1,
    firstStageInformation = 1,
    useInterimEstimate = FALSE,
    minimumConditionalError = C_min,
    maximumConditionalError = C_max
  )
  cond_error <- getOptimalConditionalError(
    setting_1$p1[-1],
    design_maxlr_constr_1
  )
  expect_equal(cond_error, setting_1$maxlr_constr[-1], tolerance = 1e-4)

  # Comparison conditional error functions - uniform with constraints

  design_unif_constr_1 <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025,
    alpha1 = 0.000158,
    alpha0 = 0.5,
    conditionalPower = 0.9,
    likelihoodRatioDistribution = "unif",
    ncp1 = 1,
    deltaMaxLR = 2 / sqrt(170 / 2),
    firstStageInformation = 170 / 2,
    useInterimEstimate = FALSE,
    minimumConditionalError = C_min,
    maximumConditionalError = C_max
  )
  cond_error <- getOptimalConditionalError(
    setting_1$p1[-1],
    design_unif_constr_1
  )
  expect_equal(cond_error, setting_1$unif_constr[-1], tolerance = 1e-4)

  # Additional regression testing

  # Design with conditional power function and interim estimate

  design_cpfun_interim <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.05,
    alpha1 = 0.0001,
    alpha0 = 0.6,
    conditionalPowerFunction = function(x) {
      1 - x
    },
    useInterimEstimate = TRUE,
    delta1Min = 0.2,
    likelihoodRatioDistribution = "maxlr",
    firstStageInformation = 45
  )

  expect_equal(
    getOptimalConditionalError(
      firstStagePValue = c(0.01, 0.025, 0.05, 0.1, 0.2),
      design = design_cpfun_interim
    ),
    c(0.54225673, 0.23556328, 0.16033201, 0.12023572, 0.06174099),
    tolerance = 1e-4
  )

  # Design with conditional power function and fixed effect
  design_cpfun_fixed <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.05,
    alpha1 = 0.0001,
    alpha0 = 0.6,
    conditionalPowerFunction = function(x) {
      1 - x
    },
    useInterimEstimate = FALSE,
    delta1 = 0.25,
    likelihoodRatioDistribution = "maxlr",
    firstStageInformation = 45
  )

  expect_equal(
    getOptimalConditionalError(
      c(0.01, 0.025, 0.05, 0.1, 0.2),
      design = design_cpfun_fixed
    ),
    c(0.987465221, 0.391909962, 0.170156530, 0.082921382, 0.043223507),
    tolerance = 1e-4
  )

  # Design with conditional power function, fixed effect, information constraints
  design_cpfun_fixed_constr <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.05,
    alpha1 = 0.0001,
    alpha0 = 0.6,
    conditionalPowerFunction = function(x) {
      1 - x
    },
    useInterimEstimate = FALSE,
    delta1 = 0.25,
    likelihoodRatioDistribution = "maxlr",
    firstStageInformation = 110,
    minimumSecondStageInformation = 1,
    maximumSecondStageInformation = 300
  )

  expect_equal(
    getOptimalConditionalError(
      c(0.01, 0.025, 0.05, 0.1, 0.2),
      design = design_cpfun_fixed_constr
    ),
    c(0.981069106, 0.396877529, 0.171916330, 0.083727926, 0.043628517),
    tolerance = 1e-4
  )

  # Design with fixed CP and information constraints
  design_inf_constraints <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025,
    alpha1 = 0.000158,
    alpha0 = 0.5,
    conditionalPower = 0.9,
    delta1 = 0.25,
    useInterimEstimate = FALSE,
    firstStageInformation = 50,
    likelihoodRatioDistribution = "maxlr",
    minimumSecondStageInformation = 30,
    maximumSecondStageInformation = 200
  )

  expect_equal(
    getOptimalConditionalError(
      c(0.01, 0.025, 0.05, 0.1, 0.2),
      design = design_inf_constraints
    ),
    c(0.362933432, 0.156691207, 0.086154255, 0.049504082, 0.030525989),
    tolerance = 1e-4
  )
})

# Fix design for all tests
design <- getDesignOptimalConditionalErrorFunction(
  alpha = 0.025,
  alpha1 = 0.001,
  alpha0 = 0.5,
  conditionalPower = 0.9,
  delta1 = 0.25,
  likelihoodRatioDistribution = "fixed",
  deltaLR = 0.25,
  firstStageInformation = 80,
  useInterimEstimate = FALSE,
)

# More elaborate design
design2 <-
  suppressWarnings(
    getDesignOptimalConditionalErrorFunction(
      alpha = 0.025,
      alpha1 = 0.001,
      alpha0 = 0.5,
      delta1Min = 0.2,
      useInterimEstimate = TRUE,
      firstStageInformation = 40,
      likelihoodRatioDistribution = "maxlr",
      conditionalPowerFunction = function(x) {
        1 - pnorm(x)
      },
      minimumConditionalError = 0.01,
      maximumConditionalError = 0.9,
      minimumSecondStageInformation = 5,
      maximumSecondStageInformation = 400,
      levelConstantMinimum = 0.5,
      levelConstantMaximum = 10.5
    )
  )


testthat::test_that(desc = "print.TrialDesignOptimalConditionalError works", code = {
  print_output <- capture.output(print(design))

  # Sporadic tests of output
  testthat::expect_true(
    object = all(
      c(
        grepl(
          pattern = "Optimal Conditional Error Function Design:",
          x = print_output[1]
        ),
        grepl(
          pattern = "General design parameters:",
          x = print_output[3]
        ),
        grepl(
          pattern = "Overall significance level: 0.025",
          x = print_output[4]
        ),
        grepl(
          pattern = "0.001",
          x = print_output[5]
        ),
        grepl(
          pattern = "0.5",
          x = print_output[6]
        ),
        grepl(
          pattern = "Alternative: 0.25",
          x = print_output[10]
        ),
        grepl(
          pattern = "First-stage information: 80",
          x = print_output[12]
        )
      )
    )
  )
  print_output <- capture.output(print(design2))
  testthat::expect_true(
    all(c(
      grepl(pattern = "Conditional power specification", x = print_output[10]),
      grepl(
        pattern = "Alternative: interim estimate restricted",
        x = print_output[12]
      ),
      grepl(pattern = "Monotonisation constants", x = print_output[23]),
      grepl(pattern = "Constraints on optimal", x = print_output[7]),
      grepl(pattern = "Constraints on second-stage", x = print_output[8])
    ))
  )
})

testthat::test_that(desc = "print.SimulationResultsOptimalConditionalError works", code = {
  sim_results <- getSimulationResults(
    design = design,
    alternative = c(0, 0.5),
    seed = 12345,
    maxNumberOfIterations = 10
  )

  print_output <- capture.output(print(sim_results))

  # Sporadic tests of output
  testthat::expect_true(
    object = all(
      c(
        grepl(
          pattern = "Simulation results",
          x = print_output[1]
        ),
        grepl(
          pattern = "Alternative:",
          x = print_output[3]
        ),
        grepl(
          pattern = "First-stage futility:",
          x = print_output[4]
        ),
        grepl(
          pattern = "First-stage efficacy:",
          x = print_output[5]
        ),
        grepl(
          pattern = "Overall power:",
          x = print_output[6]
        )
      )
    )
  )
})

testthat::test_that(desc = "print.PowerResultsOptimalConditionalError operational", code = {
  power_results <- getOverallPower(design, alternative = 0.25)

  print_output <- capture.output(print(power_results))

  testthat::expect_true(
    all(
      c(
        grepl(pattern = "Power calculation results", x = print_output[1]),
        grepl(pattern = "Alternative", x = print_output[3]),
        grepl(pattern = "First-stage futility", x = print_output[4]),
        grepl(pattern = "First-stage efficacy", x = print_output[5]),
        grepl(pattern = "Overall power", x = print_output[6])
      )
    )
  )
})

testthat::test_that(desc = "plot.TrialDesignOptimalConditionalError with type = 1 operational", code = {
  plot_output <- plot(
    design,
    type = 1
  )
  testthat::expect_s3_class(
    object = plot_output,
    class = "ggplot"
  )

  plot_output2 <- plot(
    design2,
    type = 1,
    plotNonMonotoneFunction = TRUE
  )
  testthat::expect_s3_class(
    object = plot_output2,
    class = "ggplot"
  )
})

testthat::test_that(desc = "plot.TrialDesignOptimalConditionalError with type = 2 operational", code = {
  plot_output <- plot(
    design,
    type = 2
  )
  testthat::expect_s3_class(
    object = plot_output,
    class = "ggplot"
  )

  plot_output2 <- plot(
    design2,
    type = 2,
    plotNonMonotoneFunction = TRUE
  )
  testthat::expect_s3_class(
    object = plot_output2,
    class = "ggplot"
  )
})

testthat::test_that(desc = "plot.TrialDesignOptimalConditionalError with type = 3 operational", code = {
  plot_output <- plot(
    design,
    type = 3
  )
  testthat::expect_s3_class(
    object = plot_output,
    class = "ggplot"
  )

  plot_output2 <- plot(
    design2,
    type = 3,
    plotNonMonotoneFunction = TRUE
  )
  testthat::expect_s3_class(
    object = plot_output2,
    class = "ggplot"
  )
})

testthat::test_that(desc = "plot.TrialDesignOptimalConditionalError with type = 4 operational", code = {
  plot_output <- plot(
    design,
    type = 4
  )
  testthat::expect_s3_class(
    object = plot_output,
    class = "ggplot"
  )

  plot_output2 <- plot(
    design2,
    type = 4,
    plotNonMonotoneFunction = TRUE
  )
  testthat::expect_s3_class(
    object = plot_output2,
    class = "ggplot"
  )
})

testthat::test_that(desc = "summary.TrialDesignOptimalConditionalError operational", code = {
  summary_output <- capture.output(summary(design))

  testthat::expect_true(
    all(c(
      grepl(
        pattern = "Summary of the Optimal Conditional Error Function Design",
        x = summary_output[1]
      ),
      grepl(
        pattern = "Overall significance level: 0.025",
        x = summary_output[4]
      ),
      grepl(pattern = "First-stage efficacy", x = summary_output[5]),
      grepl(pattern = "Binding first-stage futility", x = summary_output[6]),
      grepl(
        pattern = "Maximum second-stage information",
        x = summary_output[13]
      ),
      grepl(pattern = "Futility stopping probability", x = summary_output[19])
    ))
  )

  summary_output2 <- capture.output(summary(design2))

  testthat::expect_true(
    all(c(
      grepl(
        pattern = "Summary of the Optimal Conditional Error Function Design",
        x = summary_output2[1]
      ),
      grepl(
        pattern = "Overall significance level: 0.025",
        x = summary_output2[4]
      ),
      grepl(pattern = "First-stage efficacy", x = summary_output2[5]),
      grepl(pattern = "Binding first-stage futility", x = summary_output2[6]),
      grepl(pattern = "Second-stage information", x = summary_output2[8]),
      grepl(
        pattern = "Power and stopping probabilities",
        x = summary_output2[14]
      )
    ))
  )
})

testthat::test_that(desc = ".rangeCheck operational", code = {
  testvar <- c(0, 1, 2)
  testthat::expect_true(
    .rangeCheck(variable = testvar, range = c(0, 2), allowedEqual = TRUE)
  )
  testthat::expect_error(
    .rangeCheck(variable = testvar, range = c(0, 1), allowedEqual = FALSE)
  )
  testthat::expect_error(
    .rangeCheck(variable = testvar, range = c(1, 2), allowedEqual = FALSE)
  )
  testthat::expect_error(
    .rangeCheck(variable = testvar, range = c(3, 4), allowedEqual = TRUE)
  )
})

testthat::test_that("Tests for getSimulationResults() work", {
  # Tests are always skipped on CRAN (as they are time intensive)
  testthat::skip_on_cran()

  # Create a design
  design <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025,
    alpha1 = 0.001,
    alpha0 = 0.5,
    conditionalPower = 0.9,
    delta1 = 0.25,
    likelihoodRatioDistribution = "fixed",
    deltaLR = 0.25,
    firstStageInformation = 80,
    useInterimEstimate = FALSE,
  )

  # Get simulation results under H0 (alternative = 0) and under H1 (alternative = 0.25)
  simulationResults <- getSimulationResults(
    design = design,
    alternative = c(0, 0.25),
    seed = 1234
  )

  # Test class
  testthat::expect_s4_class(
    object = simulationResults,
    class = "SimulationResultsOptimalConditionalError"
  )

  # Test alternative
  testthat::expect_equal(
    object = simulationResults$alternative,
    expected = c(0, 0.25)
  )

  # Test number of iterations
  testthat::expect_equal(
    object = simulationResults$maxNumberOfIterations,
    expected = 10000
  )

  # Test first-stage futility
  testthat::expect_equal(
    object = simulationResults$firstStageFutility,
    expected = c(0.4984, 0.0119),
    tolerance = 1e-2
  )

  # Test first-stage efficacy
  testthat::expect_equal(
    object = simulationResults$firstStageEfficacy,
    expected = c(0.0011, 0.1952),
    tolerance = 1e-2
  )

  # Test overall power
  testthat::expect_equal(
    object = simulationResults$overallPower,
    expected = c(0.0252, 0.9079),
    tolerance = 1e-2
  )
})

testthat::test_that(desc = "getPsi works correctly", code = {

  #The design and the value of the first stage p-value,
  #only need to be provided, if the cp is not within the bounds
  #pnorm(2) and pnorm(-2).

  testthat::expect_equal(
    object = getPsi(
      nuPrime = getNuPrime(alpha = 0.05, conditionalPower = 0.9),
      conditionalPower = 0.9,
      design = NULL,
      firstStagePValue = NULL
    ),
    expected = 0.05
  )

  testthat::expect_equal(
    object = getPsi(
      nuPrime = getNuPrime(alpha = 0.01, conditionalPower = 0.8),
      conditionalPower = 0.8,
      design = NULL,
      firstStagePValue = NULL
    ),
    expected = 0.01
  )

  testthat::expect_equal(
    object = getPsi(
      nuPrime = c(-10, -11, -12, -13, -14),
      conditionalPower = 0.9,
      design = NULL,
      firstStagePValue = NULL
    ),
    expected = c(
      0.312539192762,
      0.282106799080,
      0.256949542063,
      0.235827887892,
      0.217855383703
    ),
    tolerance = 1e-8
  )

  #Create an example design
  design_example1 <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025, alpha1 = 0.001, alpha0 = 0.5,
    conditionalPower = 0.995, delta1 = 0.5, useInterimEstimate = FALSE,
    firstStageInformation = 4, likelihoodRatioDistribution = "fixed",
    deltaLR = 1, enforceMonotonicity = FALSE
  )

  testthat::expect_equal(
    object = getPsi(
      nuPrime = c(-10, -11, -12, -13, -14),
      conditionalPower = 0.995,
      design = design_example1,
      firstStagePValue = NULL
    ),
    expected = c(
      0.993518878813,
      0.993293975574,
      0.993044787376,
      0.992766201371,
      0.430717398170
    ),
    tolerance = 1e-8
  )

  design_example2 <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.0005, alpha1 = 0, alpha0 = 1,
    conditionalPower = 0.021, delta1 = 0.5, useInterimEstimate = FALSE,
    firstStageInformation = 4, likelihoodRatioDistribution = "fixed",
    deltaLR = 1, enforceMonotonicity = FALSE
  )

  testthat::expect_equal(
    object = getPsi(
      nuPrime = c(-10, -11, -12, -13, -14),
      conditionalPower = 0.021,
      design = design_example2,
      firstStagePValue = NULL
    ),
    expected = c(
      0.0136273452348,
      0.0132047207500,
      0.0128121326993,
      0.0124462618859,
      0.0121042796519
    ),
    tolerance = 1e-8
  )
})

testthat::test_that(desc = "integrateExpectedInformation works correctly", code = {
  # Get a design
  design <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025,
    alpha1 = 0.001,
    alpha0 = 0.5,
    conditionalPower = 0.9,
    delta1 = 0.25,
    likelihoodRatioDistribution = "fixed",
    deltaLR = 0.25,
    firstStageInformation = 80,
    useInterimEstimate = FALSE,
  )

  testthat::expect_equal(
    object = integrateExpectedInformation(
      firstStagePValue = c(0.1, 0.25, 0.3),
      design = design,
      likelihoodRatioDistribution = "fixed",
      deltaLR = 0.25
    ),
    expected = c(203.0228586001, 75.2106316639, 57.7096125375),
    tolerance = 1e-8
  )

  testthat::expect_equal(
    object = integrateExpectedInformation(
      firstStagePValue = c(0.1, 0.25, 0.3),
      design = design,
      likelihoodRatioDistribution = "normal",
      deltaLR = 0.25,
      tauLR = 0.1
    ),
    expected = c(185.2844593244, 96.3795515369, 82.4835033051),
    tolerance = 1e-8
  )

  testthat::expect_equal(
    object = integrateExpectedInformation(
      firstStagePValue = c(0.1, 0.25, 0.3),
      design = design,
      likelihoodRatioDistribution = "exp",
      kappaLR = 0.25
    ),
    expected = c(211.539307161, 227.708914650, 229.501879160),
    tolerance = 1e-8
  )

  testthat::expect_equal(
    object = integrateExpectedInformation(
      firstStagePValue = c(0.1, 0.25, 0.3),
      design = design,
      likelihoodRatioDistribution = "unif",
      deltaMaxLR = 0.5
    ),
    expected = c(161.3848893178, 107.0021378628, 97.9706170662),
    tolerance = 1e-8
  )

  testthat::expect_equal(
    object = integrateExpectedInformation(
      firstStagePValue = c(0.1, 0.25, 0.3),
      design = design,
      likelihoodRatioDistribution = "maxlr"
    ),
    expected = c(320.175526905, 254.564916843, 249.716558773),
    tolerance = 1e-8
  )

  # Get design with interim estimate
  designInterim <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025,
    alpha1 = 0.001,
    alpha0 = 0.5,
    conditionalPower = 0.9,
    likelihoodRatioDistribution = "fixed",
    deltaLR = 0.25,
    firstStageInformation = 80,
    useInterimEstimate = TRUE,
    delta1Min = 0.15,
  )

  testthat::expect_equal(
    object = integrateExpectedInformation(
      firstStagePValue = c(0.1, 0.25, 0.3),
      design = designInterim,
      likelihoodRatioDistribution = NULL
    ),
    expected = c(466.484282391, 185.267412616, 143.575823351),
    tolerance = 1e-8
  )
})

test_that("Overall power correctly calculated", {

# Test if the calculated overall power matches with two special cases (fix cp, fixed LR, without use of Interim Estimate)
design_fix <- getDesignOptimalConditionalErrorFunction(
    alpha = 0.025, alpha1 = 0.001, alpha0 = 0.5, conditionalPower = 0.9,
    delta1 = 0.5, likelihoodRatioDistribution = "fixed", deltaLR = 1,
    firstStageInformation = 2, useInterimEstimate = FALSE)

ncp <- design_fix$delta1 * sqrt(design_fix$firstStageInformation)
power <- 1 - pnorm(qnorm(1 - design_fix$alpha1) - ncp) +
  design_fix$conditionalPower * (pnorm(qnorm(1 - design_fix$alpha1) - ncp) - pnorm(qnorm(1 - design_fix$alpha0) - ncp))

expect_equal(getOverallPower(design_fix, design_fix$delta1)$overallPower, power, tolerance = 1e-4)

# Skip remaining tests on CRAN
skip_on_cran()

expect_equal(getOverallPower(design_fix, 0)$overallPower, design_fix$alpha, tolerance = 1e-4)

# Test if the calculated overall power matches with two special cases (cp function, fixed LR, without use of Interim Estimate)
design_function <- getDesignOptimalConditionalErrorFunction(
  alpha = 0.05, alpha1 = 0.001, alpha0 = 0.5, conditionalPowerFunction = function(x) pnorm(1-x),
  delta1 = 0.7, likelihoodRatioDistribution = "fixed", deltaLR = 1,
  firstStageInformation = 10, useInterimEstimate = FALSE
)

ncp <- design_function$delta1 * sqrt(design_function$firstStageInformation)
secondStageRejection <- function(firstStagePValue) {
  design_function$conditionalPowerFunction(firstStagePValue) * exp(qnorm(1 - firstStagePValue) * ncp - ncp^2 / 2)
}
integral <- stats::integrate(f = secondStageRejection, lower = design_function$alpha1, upper = design_function$alpha0)$value
power <- 1 - pnorm(qnorm(1 - design_function$alpha1) - ncp) + integral

expect_equal(getOverallPower(design_function, design_function$delta1)$overallPower, power, tolerance = 1e-4)
expect_equal(getOverallPower(design_function, 0)$overallPower, design_function$alpha, tolerance = 1e-4)

})

#' Internal calculation of the integral over psi
#' @name getIntegral
#'
#' @description Helper function that integrates over the optimal conditional error function. Used to find appropriate level constant.
#'
#' @details Internal function called by \code{findLevelConstant()} that should be solved (i.e., root should be found).
#'
#' @template param_constant_integrate
#' @template param_design
#'
#' @return Distance of integral over psi to (alpha-alpha1).
#' @keywords internal

getIntegral <- function(constant, design) {
  enforceBasicIntegration <- isTRUE(as.logical(getOption(
    "optconerrf.enforce.basic.integration",
    FALSE
  )))

  designHasConstraints <- (design$maximumConditionalError < 1) ||
    (design$minimumConditionalError > 0) ||
    (design$minimumSecondStageInformation > 0) ||
    (design$maximumSecondStageInformation < Inf)

  # In the following cases, use basic integration:
  # - enforced through option (see above)
  # - design has constraints
  # - monotonicity of design is not enforced
  # - design has no monotonisation constants
  # - design uses a conditional power function

  if (
    enforceBasicIntegration ||
      designHasConstraints ||
      !design$enforceMonotonicity ||
      is.null(unlist(design$monotonisationConstants)) ||
      !is.null(suppressWarnings(body(design$conditionalPowerFunction)))
  ) {
    integral <- stats::integrate(
      f = getInnerPsi,
      lower = design$alpha1,
      upper = design$alpha0,
      constant = constant,
      design = design
    )$value
  } else {
    # If monotonisation constants exist, use alternative integration routine better adapted to constant functions
    integral <- getIntegralWithConstants(
      constant = constant,
      design = design
    )
  }
  return(integral - (design$alpha - design$alpha1))
}

#' Calculate the Constants for Monotonisation
#' @name getMonotonisationConstants
#'
#' @description Computes the constants required to make a function non-increasing on the specified interval. The output of this function is necessary to calculate the monotone optimal conditional error function.
#' The output object is a list that contains the intervals on which constant values are required, specified by the minimum \code{dls} and maximum \code{dus} of the interval and the respective constants, \code{qs}.
#'
#' @template param_fun_mono
#' @template param_lower_mono
#' @template param_upper_mono
#' @template param_argument_mono
#' @template param_nSteps_mono
#' @template param_epsilon_mono
#' @template param_numberOfIterationsQ
#' @template param_design
#'
#' @return A list containing the monotonisation constants (element \code{$qs}) and the intervals on which they must be applied, specified via minimum (element \code{qls}) and maximum (element \code{qus}).
#'
#' @template reference_monotone

getMonotonisationConstants <- function(
  fun,
  lower = 0,
  upper = 1,
  argument,
  nSteps = 10^4,
  epsilon = 10^(-5),
  numberOfIterationsQ = 10^4,
  design
) {
  # Sequence of argument values
  argumentValues <- max(0, lower - 1 / (nSteps + 1)) +
    (upper - lower) * (1:nSteps) / (nSteps + 1)

  # Create a list of arguments that fun requires
  argumentList <- list(argumentValues, design)
  names(argumentList) <- c(argument, "design")

  # Call fun with the specified arguments
  functionValues <- do.call(what = fun, args = argumentList)

  # Get min and max function value
  minFunctionValue <- min(functionValues)
  maxFunctionValue <- max(functionValues)

  # Append max and min values
  functionValues <- c(maxFunctionValue, functionValues, minFunctionValue)

  # Helper variable that saves the number of function values
  m <- length(functionValues)

  # Calculate initial "integral"
  initialIntegral <- (sum(functionValues) -
    0.5 * (minFunctionValue + maxFunctionValue)) /
    m

  # Changes in function values
  derivativeFunctionValues <- functionValues[-1] - functionValues[-m]

  output <- NULL

  # Check if function is increasing anywhere
  if (max(derivativeFunctionValues) > 0) {
    # Vector for the constants
    qs <- NULL

    for (i in 1:m) {
      # Recalculate min and max value
      minFunctionValue <- min(functionValues)
      maxFunctionValue <- max(functionValues)

      # Determine derivatives of function values
      derivativeFunctionValues <- c(0, functionValues[-1] - functionValues[-m])

      # Additional check to potentially save runtime: function already non-increasing?
      if (max(derivativeFunctionValues) <= 0) {
        break
      }

      # Index of lower boundary of first interval
      dl1Position <- min(c(sum(cummin(derivativeFunctionValues <= 0)) + 1, m))

      # Index of upper boundary of first interval
      if (dl1Position < length(functionValues)) {
        duPosition <- dl1Position +
          sum(cummin(derivativeFunctionValues[dl1Position:(m - 2)] > 0))
      } else {
        duPosition <- dl1Position
      }

      # Index of lower boundary of second interval
      if (duPosition < length(functionValues)) {
        dl2Position <- duPosition +
          sum(cummin(derivativeFunctionValues[duPosition:(m - 1)] <= 0))
      } else {
        dl2Position <- duPosition
      }

      # Repeat until integrals are similar enough or maximum number of iterations reached
      for (j in 1:numberOfIterationsQ) {
        # Initial guess for q
        q <- (minFunctionValue + maxFunctionValue) / 2

        # Temporal modification of functionValues with the current guess for q
        modFunctionValues <- pmax(q, functionValues[1:dl1Position])
        modFunctionValues <- c(
          modFunctionValues,
          rep(q, duPosition + 1 - dl1Position)
        )

        if (duPosition < (m - 1)) {
          modFunctionValues <- c(
            modFunctionValues,
            pmin(q, functionValues[(duPosition + 2):dl2Position])
          )
        }
        if (dl2Position < m) {
          modFunctionValues <- c(
            modFunctionValues,
            functionValues[(dl2Position + 1):m]
          )
        }

        newIntegral <- (sum(modFunctionValues) -
          0.5 * (max(modFunctionValues) + min(modFunctionValues))) /
          m

        # If difference between integrals is small enough, stop current iteration
        if (abs(newIntegral - initialIntegral) < epsilon) {
          break
        }

        # If difference between integrals is not small enough yet, update the maximal or minimal values
        if (newIntegral > initialIntegral) {
          maxFunctionValue <- q
        } else {
          minFunctionValue <- q
        }
      }

      # No constants q so far: add
      if (is.null(qs)) {
        qs <- c(qs, q)
      } else {
        # If some constants already exist: if the new constant is larger than the last constant, it must replace the last constant.
        # This ensures that the constants are also non-increasing
        if (q >= qs[length(qs)]) {
          qs[length(qs)] <- q
        } else {
          qs <- c(qs, q)
        }
      }
      # No more increasing interval -> Finished
      if (dl2Position == m - 1) {
        break
      } else {
        functionValues <- modFunctionValues
      }
    }
    dls <- NULL
    dus <- NULL
    argumentValues <- c(lower, argumentValues, upper)

    # Find min and max of the intervals for each q
    for (q in qs) {
      dls <- c(
        dls,
        argumentValues[sum(cummin(modFunctionValues[2:(m - 1)] > q)) + 1]
      )
      dus <- c(
        dus,
        argumentValues[sum(cummin(modFunctionValues[2:(m - 1)] >= q)) + 1]
      )
    }

    output <- list(qs = qs, dls = dls, dus = dus, integral = newIntegral)
  }

  # Special case: function is increasing at the last element -> set last du to upper
  if (length(output$dus) > 0) {
    if (output$dus[length(output$dus)] >= argumentValues[m - 1]) {
      output$dus[length(output$dus)] <- upper
    }
  }

  # If there are no entries, no monotonisation is required
  if (is.null(unlist(output))) {
    output <- list()
  } else {
    if (!design$enforceMonotonicity) {
      warning(
        "Monotonisation is required. Set enforceMonotonicity to TRUE in design object for strict type I error control."
      )
    }
  }

  return(output)
}

#' Calculate the Second-stage Information
#' @name getSecondStageInformation
#'
#' @description Calculate second-stage information for given first-stage p-value and design.
#'
#' @details The second-stage information \eqn{I_{2}} is calculated given a first-stage p-value \eqn{p_1} as:
#' \deqn{I_{2}(p_1) = \frac{(\Phi^{-1}(1-\alpha_2(p_1)) + \Phi^{-1}(CP))^2}{\Delta_1^2} = \frac{\nu(\alpha_2(p_1))}{\Delta_1^2},}
#' where
#' \itemize{
#'    \item \eqn{\alpha_2(p_1)} is the conditional error function
#'    \item \eqn{CP} is the target conditional power
#'    \item \eqn{\Delta_1} is the assumed treatment effect (expressed as a mean difference).
#' }
#' The conditional error is calculated according to the specification provided in the \code{design} argument.
#' For p-values smaller or equal to the first-stage efficacy boundary as well as p-values greater than the first-stage futility boundary,
#' the returned information is 0 (since the trial is ended early in both cases).
#'
#' @template param_firstStagePValue
#' @template param_design
#'
#' @return The second-stage information.
#' @export
#'
#' @seealso [getDesignOptimalConditionalErrorFunction()], [getExpectedSecondStageInformation()], [getOptimalConditionalError()]
#'
#' @examples
#' design <- getDesignOptimalConditionalErrorFunction(
#'   alpha = 0.025, alpha1 = 0.001, alpha0 = 0.5,
#'   conditionalPower = 0.9, delta1 = 0.25, useInterimEstimate = FALSE,
#'   firstStageInformation = 40, likelihoodRatioDistribution = "maxlr"
#' )
#'
#' getSecondStageInformation(
#'   firstStagePValue = c(0.05, 0.1, 0.2), design = design
#' )
#'
#' @template reference_optimal
getSecondStageInformation <- function(firstStagePValue, design) {
  # For p-values outside of the continuation region, return information 0
  if (
    (firstStagePValue <= design$alpha1 && design$alpha1 > 0) ||
      firstStagePValue > design$alpha0
  ) {
    information <- 0
  } else {
    # For design with interim estimate, apply effect restrictions
    if (design$useInterimEstimate) {
      effect <- min(
        max(
          qnorm(1 - firstStagePValue) / sqrt(design$firstStageInformation),
          design$delta1Min
        ),
        design$delta1Max
      )
    } else {
      # For design without interim estimate, use fixed effect
      effect <- design$delta1
    }

    # Calculate conditional error
    conditionalError <- getOptimalConditionalError(
      firstStagePValue = firstStagePValue,
      design = design
    )

    # Check if conditional power function should be used
    if (!is.null(suppressWarnings(body(design$conditionalPowerFunction)))) {
      conditionalPower <- design$conditionalPowerFunction(firstStagePValue)
    } else {
      conditionalPower <- design$conditionalPower
    }

    information <- (getNu(
      alpha = conditionalError,
      conditionalPower = conditionalPower
    )) /
      (effect^2)
  }
  return(information)
}

getSecondStageInformation <- Vectorize(
  FUN = getSecondStageInformation,
  vectorize.args = c("firstStagePValue")
)

#' Calculate Expected Second-stage Information
#' @name getExpectedSecondStageInformation
#'
#' @description Calculate the expected second-stage information using the optimal conditional error function with specific assumptions.
#'
#' @details {The expected second-stage information is calculated as:
#'  \deqn{\mathbb{E}(I_{2})=\int_{\alpha_1}^{\alpha_0}\frac{\nu(\alpha_2(p_1)) \cdot l(p_1)}{\Delta_1^2} dp_1,}
#'    where
#'    \itemize{
#'        \item \eqn{\alpha_1, \alpha_0} are the first-stage efficacy and futility boundaries
#'        \item \eqn{\alpha_2(p_1)} is the optimal conditional error calculated for \eqn{p_1}
#'        \item \eqn{l(p_1)} is the "true" likelihood ratio under which to calculate the expected sample size. This can be different from the likelihood ratio used to calibrate the optimal conditional error function.
#'        \item \eqn{\Delta_1} is the assumed treatment effect to power for, expressed as a mean difference. It may depend on the interim data (i.e., \eqn{p_1}) in case \code{useInterimEstimate = TRUE} was specified for the design object.
#'        \item \eqn{\nu(\alpha_2(p_1)) = (\Phi^{-1}(1-\alpha_2(p_1))+\Phi^{-1}(CP))^2} is a factor calculated for the specific assumptions about the optimal conditional error function and the target conditional power \eqn{CP}.
#' }}
#'
#' @template param_design
#' @template param_likelihoodRatioDistribution_expected
#' @param ... {Additional parameters required for the specification of \code{likelihoodRatioDistribution}}.
#'
#' @return Expected second-stage information.
#'
#' @examples
#' # Get a design
#' design <- getDesignOptimalConditionalErrorFunction(
#' alpha = 0.025, alpha1 = 0.001, alpha0 = 0.5, conditionalPower = 0.9,
#' delta1 = 0.25, likelihoodRatioDistribution = "fixed", deltaLR = 0.25,
#' firstStageInformation = 80, useInterimEstimate = FALSE,
#' )
#' # Calculate expected information under correct specification
#' getExpectedSecondStageInformation(design)
#'
#' # Calculate expected information under the null hypothesis
#' getExpectedSecondStageInformation(
#'  design = design, likelihoodRatioDistribution = "fixed", deltaLR = 0
#' )
#'
#' @export
#' @seealso [getDesignOptimalConditionalErrorFunction()], [getSecondStageInformation()]
#' @template reference_optimal

getExpectedSecondStageInformation <- function(
  design,
  likelihoodRatioDistribution = NULL,
  ...
) {
  # Integrate over a helper function from alpha1 to alpha0
  return(
    stats::integrate(
      f = integrateExpectedInformation,
      lower = design$alpha1,
      upper = design$alpha0,
      design = design,
      likelihoodRatioDistribution = likelihoodRatioDistribution,
      ... = ...
    )$value
  )
}

#' Calculates the lower constraint.
#' @name getConstraintC_min
#'
#' @details Internal function that calculates the lower constraint.
#'
#' @template param_firstStagePValue
#' @template param_design
#'
#' @return Value of the lower constraint
#'
#'

getConstraintC_min <- function(firstStagePValue, design){

  C_min_Info <- NULL

  # Constraints on conditional error scale
  C_min_cond_error <- design$minimumConditionalError

  # Identify conditional power to be supplied to getPsi and to be used for (information) constraints
  # Check if conditional power function should be used
  if(!is.null(suppressWarnings(body(design$conditionalPowerFunction)))) {
    conditionalPower <- design$conditionalPowerFunction(firstStagePValue)

    #Calculate possibly data-dependent constraints
    #Identify effect for constraints
    if(design$useInterimEstimate) {
      delta1 <- pmin(pmax(qnorm(1-firstStagePValue)/sqrt(design$firstStageInformation), design$delta1Min), design$delta1Max)
    } else {
      delta1 <- design$delta1
    }

    #If maximumSecondStageInformation is given, use this
    if(design$maximumSecondStageInformation < Inf){
      C_min_Info <- 1 - pnorm(delta1* sqrt(design$maximumSecondStageInformation)-qnorm(conditionalPower))
    }
  } else {
    conditionalPower <- design$conditionalPower

    #Check if interim estimate is used
    if(design$useInterimEstimate) {
      delta_C_min <- max(qnorm(1-design$alpha0)/sqrt(design$firstStageInformation),design$delta1Min)
    }
    # Otherwise use fixed effect
    else {
      delta_C_min <- design$delta1
    }

    #If maximumSecondStageInformation is given, use this
    if(design$maximumSecondStageInformation < Inf){
      C_min_Info <- 1 - pnorm(delta_C_min* sqrt(design$maximumSecondStageInformation)-qnorm(conditionalPower))
    }
  }

  #Use the constraint that is the stronger restriction
  C_min <- max(C_min_Info, C_min_cond_error)
  return(C_min)
}

#' Get Level Constant for Optimal Conditional Error Function
#' @name getLevelConstant
#'
#' @description Find the constant required such that the conditional error function meets the overall level condition.
#'
#' @details The level condition is defined as:
#' \deqn{\alpha = \alpha_1 + \int_{\alpha_1}^{\alpha_0} \alpha_2(p_1)dp_1.}
#' The constant \eqn{c_0} of the optimal conditional error function is calibrated such that it meets the level condition.
#' For a valid design, the additional following condition must be met to be able to exhaust the level \eqn{\alpha}:
#' \deqn{\alpha_1 + CP(\alpha_0-\alpha_1)>\alpha.}
#' This condition is checked by \code{getLevelConstant()} and the execution is terminated if it is not met. \cr
#' In case a conditional power function is used, the condition is instead:
#' \deqn{\alpha_1 + \int_{\alpha_1}^{\alpha_0} CP(p_1)dp_1>\alpha.}
#'
#' @template param_design
#'
#' @return A list that contains the constant (element \code{$root}) and other components provided by \code{uniroot()}.
#' The level constant is calculated corresponding to the mean difference scale.
#'
#' @template reference_optimal
#' @template reference_monotone

getLevelConstant <- function(design) {
  # Check basic condition for decision rules
  # Fixed conditional power
  if (!is.na(design$conditionalPower)) {
    if (
      design$alpha1 +
        design$conditionalPower * (design$alpha0 - design$alpha1) <=
        design$alpha
    ) {
      stop(
        "(alpha1 + conditionalPower*(alpha0-alpha1)) must exceed alpha, otherwise no level constant fully exhausting alpha can be found."
      )
    }
  } else if (
    !is.null(suppressWarnings(body(design$conditionalPowerFunction)))
  ) {
    # Conditional power function
    if (
      stats::integrate(
        f = design$conditionalPowerFunction,
        lower = design$alpha1,
        upper = design$alpha0
      )$value <=
        design$alpha - design$alpha1
    ) {
      stop(
        "Integral over conditional power function from alpha1 to alpha0 must exceed (alpha-alpha1), otherwise no level constant fully exhausting alpha can be found."
      )
    }
  } else {
    # Unexpected issue
    stop(
      "Unexpected error: both conditionalPower and conditionalPowerFunction are specified inappropriately."
    )
  }

  # Find the level constant.
  # Expects an error if specified non-centrality parameter is very large or very small
  tryCatch(
    expr = {
      stats::uniroot(
        f = getIntegral,
        lower = design$levelConstantMinimum,
        upper = design$levelConstantMaximum,
        design = design,
        tol = 1e-16
      )
    },
    error = function(e) {
      # This specific error may occur if the given non-centrality parameter is too small or too large or if the
      # provided constraints are not suitable and is handled separately
      if (e$message == "f() values at end points not of opposite sign") {
        stop(
          "Root finding for level constant failed. Try changing the search interval via arguments levelConstantMinimum and levelConstantMaximum. \n Alternatively, the constraints on the optimal conditional error function or second-stage information may not be appropriate."
        )
      } else {
        # Print all other errors directly
        stop(e)
      }
    }
  )
}

#' Calculate the overall power
#' @name getOverallPower
#'
#' @description Calculate the overall power and other operating characteristics of a design.
#'
#' @details
#' This function is used to evaluate the overall performance of a design.
#' The probabilities for first-stage futility, first-stage efficacy and overall efficacy (i.e., overall power) are saved in an object of class \code{PowerResultsOptimalConditionalError}.
#'
#' @template param_design
#' @template param_alternative
#'
#' @seealso [getDesignOptimalConditionalErrorFunction()], [getSimulationResults()]
#'
#' @examples
#' # Get a design
#' design <- getDesignOptimalConditionalErrorFunction(
#' alpha = 0.025, alpha1 = 0.001, alpha0 = 0.5, conditionalPower = 0.9,
#' delta1 = 0.25, likelihoodRatioDistribution = "fixed", deltaLR = 0.25,
#' firstStageInformation = 80, useInterimEstimate = FALSE,
#' )
#'
#' getOverallPower(design, alternative = 0.25)
#'
#' @return The overall power of the design at the provided effect size.
#' @export

getOverallPower <- function(design, alternative) {
  ncp1 <- alternative * sqrt(design$firstStageInformation)

  firstStageFutility <- numeric(length(alternative))
  firstStageEfficacy <- numeric(length(alternative))
  overallPower <- numeric(length(alternative))

  for (i in 1:length(alternative)) {
    # Early decision probabilities
    firstStageFutility[i] <- stats::pnorm(
      stats::qnorm(1 - design$alpha0) - ncp1[i]
    )
    firstStageEfficacy[i] <- 1 -
      stats::pnorm(stats::qnorm(1 - design$alpha1) - ncp1[i])

    # Calculate probability to reject at the second stage for given delta
    secondStageRejection <- function(firstStagePValue) {
      (1 -
        stats::pnorm(
          stats::qnorm(
            1 - getOptimalConditionalError(firstStagePValue, design = design)
          ) -
            sqrt(getSecondStageInformation(firstStagePValue, design = design)) *
              alternative[i]
        )) *
        exp(qnorm(1 - firstStagePValue) * ncp1[i] - ncp1[i]^2 / 2)
    }

    integral <- stats::integrate(
      f = secondStageRejection,
      lower = design$alpha1,
      upper = design$alpha0
    )$value

    overallPower[i] <- firstStageEfficacy[i] + integral
  }

  powerResults <- new(
    "PowerResultsOptimalConditionalError",
    alternative = alternative,
    firstStageFutility = firstStageFutility,
    firstStageEfficacy = firstStageEfficacy,
    overallPower = overallPower
  )

  return(powerResults)
}

#' Return Monotone Function Values
#'
#' @name getMonotoneFunction
#'
#' @description Applies the provided monotonisation constants to a specified, possibly non-monotone function. The returned function values are non-increasing.
#'
#' @details The exact monotonisation process is outlined in Brannath et al. (2024), but specified in terms of the first-stage test statistic \eqn{z_1} rather than the first-stage p-value \eqn{p_1}. \cr
#' The algorithm can easily be translated to the use of p-values by switching the maximum and minimum functions, i.e., replacing \eqn{\min\{q, Q(z_1)\}} by \eqn{\max\{q, Q(p_1)\}} and \eqn{\min\{q, Q(z_1)\}} by \eqn{\max\{q, Q(p_1\}}.
#'
#' @param x Argument values.
#' @template param_fun_mono
#' @template param_lower_mono
#' @template param_upper_mono
#' @template param_argument_mono
#' @template param_nSteps_mono
#' @template param_epsilon_mono
#' @template param_numberOfIterationsQ
#' @template param_design
#'
#' @return Monotone function values.
#'
#'
#' @template reference_monotone

getMonotoneFunction <- function(
  x,
  fun,
  lower = NULL,
  upper = NULL,
  argument = NULL,
  nSteps = 10^4,
  epsilon = 10^(-5),
  numberOfIterationsQ = 10^4,
  design
) {
  # If monotonisation is enforced, do it
  if (design$enforceMonotonicity) {
    out <- design$monotonisationConstants
  } else {
    out <- list()
  }

  # If the length of object out is 0, the function is already non-increasing
  if (length(out) == 0) {
    modFunctionValues <- fun(x, design = design)
  } else {
    # If the length of out is larger than 0, a transformation is necessary
    pos_lower <- apply(outer(x, out$dls, ">="), 1, sum)
    pos_upper <- apply(outer(x, out$dus, ">"), 1, sum) + 1
    modFunctionValues <- ifelse(
      pos_lower == pos_upper,
      yes = out$qs[pmax(1, pos_lower)],
      fun(x, design = design)
    )
  }
  return(modFunctionValues)
}

#' Print optimal conditional error trial design
#'
#' @description
#' Print an overview of the specified design parameters.
#'
#' @param x Design object of class \code{TrialDesignOptimalConditionalError}
#' @param ... Additional arguments required for generic compatibility
#'
#' @return No return value, prints the design.
#'
#' @export
print.TrialDesignOptimalConditionalError <- function(x, ...) {
  cat("Optimal Conditional Error Function Design: \n \n")
  cat("General design parameters: \n")
  cat("  Overall significance level:", x$alpha, "\n")
  cat("  First-stage efficacy boundary (p-value scale):", x$alpha1, "\n")
  cat(
    "  Binding first-stage futility boundary (p-value scale):",
    x$alpha0,
    "\n"
  )
  if (x$minimumConditionalError > 0 || x$maximumConditionalError < 1) {
    cat(
      "  Constraints on optimal conditional error:",
      paste(
        "[",
        x$minimumConditionalError,
        ", ",
        x$maximumConditionalError,
        "]",
        sep = ""
      ),
      "\n"
    )
  }
  if (
    x$minimumSecondStageInformation > 0 || x$maximumSecondStageInformation < Inf
  ) {
    cat(
      "  Constraints on second-stage information:",
      paste(
        "[",
        x$minimumSecondStageInformation,
        ", ",
        x$maximumSecondStageInformation,
        "]",
        sep = ""
      ),
      "\n"
    )
  }

  cat("\n")

  cat("Conditional power specification: \n")
  if (is.na(x$conditionalPower)) {
    cat("  Data-dependent user-specified function \n")
  } else if (!is.na(x$conditionalPower)) {
    cat("  Target conditional power:", x$conditionalPower, "\n")
  }
  if (x$useInterimEstimate) {
    cat(
      "  Alternative: interim estimate restricted to",
      paste("[", x$delta1Min, ", ", x$delta1Max, "]", sep = ""),
      "\n"
    )
    cat(
      "  First-stage non-centrality parameter restricted to",
      paste("[", x$ncp1Min, ", ", x$ncp1Max, "]", sep = ""),
      "\n"
    )
  } else {
    cat("  Alternative:", x$delta1, "\n")
    cat("  First-stage non-centrality parameter:", x$ncp1, "\n")
  }
  cat("  First-stage information:", x$firstStageInformation, "\n")
  cat("\n")

  cat("Likelihood ratio specification: \n")
  switch(
    x$likelihoodRatioDistribution,
    fixed = {
      cat(
        "  Fixed parameter(s) in likelihood ratio: ",
        paste(format(x$deltaLR, trim = TRUE), collapse = ", "),
        "\n"
      )
      cat(
        "  Parameter weights: ",
        paste(format(x$weightsDeltaLR, trim = TRUE), collapse = ", "),
        "\n"
      )
    },
    normal = {
      cat(
        "  Normally distributed prior in likelihood ratio with mean ",
        x$deltaLR,
        " and standard deviation ",
        x$tauLR,
        "\n"
      )
    },
    unif = {
      cat(
        "  Uniformly distributed prior in likelihood ratio with maximum ",
        x$deltaMaxLR,
        "\n"
      )
    },
    exp = {
      cat(
        "  Exponentially distributed prior in likelihood ratio with mean ",
        x$kappaLR,
        "\n"
      )
    },
    maxlr = {
      cat("  Maximum likelihood ratio \n")
    }
  )

  cat("\n")
  cat("Level constant: \n")
  cat("  Constant:", x$levelConstant, "\n")
  cat(
    "  Searched on interval:",
    paste(
      "[",
      x$levelConstantMinimum,
      ", ",
      x$levelConstantMaximum,
      "]",
      sep = ""
    ),
    "\n"
  )

  if (!is.null(unlist(x$monotonisationConstants))) {
    cat("\n")
    cat("Monotonisation constants: \n")
    cat(
      "  Intervals (p-value scale):",
      paste(
        "[",
        apply(
          X = cbind(
            x$monotonisationConstants$dls,
            x$monotonisationConstants$dus
          ),
          FUN = paste,
          MARGIN = 1,
          collapse = ", "
        ),
        "]",
        sep = ""
      ),
      "\n"
    )
    cat("  Constant(s) (Q scale):", x$monotonisationConstants$qs, "\n")
  }
  if (!x$enforceMonotonicity) {
    cat("\n")
    cat("Monotonicity was not enforced \n")
  }
}


#' Print simulation results
#'
#' @description
#' Print an overview of simulation results.
#'
#' @param x Simulation results object of class \code{SimulationResultsOptimalConditionalError}
#' @param ... Additional arguments required for generic compatibility
#'
#' @return No return value, prints the simulation results.
#'
#' @export
print.SimulationResultsOptimalConditionalError <- function(x, ...) {
  cat("Simulation results \n \n")
  cat(
    " Alternative: ",
    paste(format(x$alternative, trim = TRUE), collapse = ", "),
    "\n"
  )
  cat(
    " First-stage futility: ",
    paste(format(x$firstStageFutility, trim = TRUE), collapse = ", "),
    "\n"
  )
  cat(
    " First-stage efficacy: ",
    paste(format(x$firstStageEfficacy, trim = TRUE), collapse = ", "),
    "\n"
  )
  cat(
    " Overall power: ",
    paste(format(x$overallPower, trim = TRUE), collapse = ", "),
    "\n"
  )
  cat(" Simulated trials per scenario: ", x$maxNumberOfIterations)
}


#' Print power results
#'
#' @description
#' Print an overview of exact power results.
#'
#' @param x Power results object of class \code{PowerResultsOptimalConditionalError}
#' @param ... Additional arguments required for generic compatibility
#'
#' @return No return value, prints the power calculation results.
#'
#' @export
print.PowerResultsOptimalConditionalError <- function(x, ...) {
  cat("Power calculation results \n \n")
  cat(
    " Alternative: ",
    paste(format(x$alternative, trim = TRUE), collapse = ", "),
    "\n"
  )
  cat(
    " First-stage futility: ",
    paste(format(x$firstStageFutility, trim = TRUE), collapse = ", "),
    "\n"
  )
  cat(
    " First-stage efficacy: ",
    paste(format(x$firstStageEfficacy, trim = TRUE), collapse = ", "),
    "\n"
  )
  cat(
    " Overall power: ",
    paste(format(x$overallPower, trim = TRUE), collapse = ", "),
    "\n"
  )
}


#' Plot the optimal conditional error function
#' @name plot.TrialDesignOptimalConditionalError
#'
#' @description
#' The returned plot is a \code{ggplot2} object and can be supplemented with additional layers using \code{ggplot2} commands.
#'
#' @param x Design object of class \code{TrialDesignOptimalConditionalError}.
#' @param range Numeric vector with two entries specifying the range of the x-axis of the plot.
#' @param type Type of plot to be created. Options are: \itemize{
#' \item \code{type = 1}: Plot the values of the optimal conditional error function against the first-stage p-value.
#' \item \code{type = 2}: Plot the second-stage information resulting from the optimal conditional error function against the first-stage p-value.
#' \item \code{type = 3}: Plot the likelihood ratio of the given specification of the optimal conditional error function against the first-stage p-value.
#' \item \code{type = 4}: Plot the function Q of the given specification of the optimal conditional error function against the first-stage p-value.
#' }
#' @param plotNonMonotoneFunction Logical. Should the non-monotone version of the plot be drawn? Not applicable for plot type 3. Default: \code{FALSE}.
#' @param ... Additional arguments required for generic compatibility
#'
#' @return No return value, plots the design.
#'
#' @export
plot.TrialDesignOptimalConditionalError <- function(
  x,
  range = c(0, 1),
  type = 1,
  plotNonMonotoneFunction = FALSE,
  ...
) {
  # Set a range of first-stage p-values
  firstStagePValues <- seq(from = range[1], to = range[2], length.out = 1e3)

  # Plot type 1: draw the optimal conditional error function
  if (type == 1) {
    # Calculate optimal conditional error for provided design
    optimalConditionalErrors <- getOptimalConditionalError(
      firstStagePValue = firstStagePValues,
      design = x
    )

    # Create base plot and save it for possible addition of the non-monotone function
    designPlot <- ggplot2::ggplot(data = NULL) +
      ggplot2::geom_line(
        mapping = ggplot2::aes(
          x = firstStagePValues,
          y = optimalConditionalErrors
        ),
        colour = "black",
        linetype = "solid",
        linewidth = 1.1
      ) +
      ggplot2::labs(
        x = "First-stage p-value",
        y = "Optimal Conditional Error"
      ) +
      ggplot2::theme_bw() +
      ggplot2::geom_vline(
        xintercept = x$alpha0,
        linetype = "dotted",
        col = "red",
        linewidth = 0.8
      ) +
      ggplot2::geom_vline(
        xintercept = x$alpha1,
        linetype = "dotted",
        col = "blue",
        linewidth = 0.8
      ) +
      ggplot2::xlim(c(range[1], range[2]))

    # In addition to the monotone function, the non-monotone one should be drawn
    if (plotNonMonotoneFunction) {
      # There are no monotonisation constants -> monotonisation is not required
      if (is.null(unlist(x$monotonisationConstants))) {
        warning(
          "No monotonisation required. Displaying monotone function only."
        )
        designPlot # Display base plot
      } else {
        # Plotting the non-monotone function only makes sense if the provided design is enforced to be monotone
        if (x$enforceMonotonicity) {
          # Here, a new design must be created, modifying the old one will not work as intended (Operation is not "copy-on-modify")
          # All fields apart from enforceMonotonicity are copied from the original design object.
          # (There may be a better way to implement this)
          # suppressWarnings() is used because this code is expected to always produce a warning
          secondDesign <- suppressWarnings(new(
            "TrialDesignOptimalConditionalError",
            alpha = x$alpha,
            alpha1 = x$alpha1,
            alpha0 = x$alpha0,
            conditionalPower = x$conditionalPower,
            conditionalPowerFunction = x$conditionalPowerFunction,
            delta1 = x$delta1,
            firstStageInformation = x$firstStageInformation,
            useInterimEstimate = x$useInterimEstimate,
            likelihoodRatioDistribution = x$likelihoodRatioDistribution,
            deltaLR = x$deltaLR,
            weightsDeltaLR = x$weightsDeltaLR,
            tauLR = x$tauLR,
            kappaLR = x$kappaLR,
            deltaMaxLR = x$deltaMaxLR,
            minimumConditionalError = x$minimumConditionalError,
            maximumConditionalError = x$maximumConditionalError,
            minimumSecondStageInformation = x$minimumSecondStageInformation,
            maximumSecondStageInformation = x$maximumSecondStageInformation,
            levelConstantMinimum = x$levelConstantMinimum,
            levelConstantMaximum = x$levelConstantMaximum,
            ncp1Min = x$ncp1Min,
            ncp1Max = x$ncp1Max,
            enforceMonotonicity = FALSE
          ))

          # Calculate optimal conditional error for the new design
          nonMonoOptimalConditionalErrors <- getOptimalConditionalError(
            firstStagePValue = firstStagePValues,
            design = secondDesign
          )

          # Add non-monotone optimal conditional error to base plot and display it
          designPlot +
            ggplot2::geom_line(
              mapping = ggplot2::aes(
                x = firstStagePValues,
                y = nonMonoOptimalConditionalErrors
              ),
              colour = "gray",
              linetype = "dashed",
              linewidth = 1.1
            )
        } else {
          warning(
            "When using plotNonMonotoneFunction=TRUE, x should provide a monotone function. Consider setting enforceMonotonicity=TRUE in design object."
          )
          designPlot # Display base plot
        }
      }
    } else {
      designPlot # Display base plot
    }
  } else if (type == 2) {
    # Plot type 2: draw second-stage information
    # Calculate second-stage information
    secondStageInformation <- getSecondStageInformation(
      firstStagePValue = firstStagePValues,
      design = x
    )

    # Create base plot and save it for possible addition of the non-monotone function
    informationPlot <- ggplot2::ggplot() +
      ggplot2::geom_line(
        mapping = ggplot2::aes(
          x = firstStagePValues,
          y = secondStageInformation
        ),
        colour = "black",
        linetype = "solid",
        linewidth = 1.1
      ) +
      ggplot2::labs(x = "First-stage p-value", y = "Second-stage information") +
      ggplot2::theme_bw() +
      ggplot2::geom_vline(
        xintercept = x$alpha0,
        linetype = "dotted",
        col = "red",
        linewidth = 0.8
      ) +
      ggplot2::geom_vline(
        xintercept = x$alpha1,
        linetype = "dotted",
        col = "blue",
        linewidth = 0.8
      ) +
      ggplot2::xlim(c(range[1], range[2]))

    if (plotNonMonotoneFunction) {
      # There are no monotonisation constants -> monotonisation is not required
      if (is.null(unlist(x$monotonisationConstants))) {
        warning(
          "No monotonisation required. Displaying monotone function only."
        )
        informationPlot # Display base plot
      } else {
        # Plotting the non-monotone function only makes sense if the provided design is enforced to be monotone
        if (x$enforceMonotonicity) {
          # Here, a new design must be created, modifying the old one will not work as intended (Operation is not "copy-on-modify")
          # All fields apart from enforceMonotonicity are copied from the original design object.
          # (There may be a better way to implement this)
          # suppressWarnings() is used because this code is expected to always produce a warning
          secondDesign <- suppressWarnings(new(
            "TrialDesignOptimalConditionalError",
            alpha = x$alpha,
            alpha1 = x$alpha1,
            alpha0 = x$alpha0,
            conditionalPower = x$conditionalPower,
            conditionalPowerFunction = x$conditionalPowerFunction,
            delta1 = x$delta1,
            firstStageInformation = x$firstStageInformation,
            useInterimEstimate = x$useInterimEstimate,
            likelihoodRatioDistribution = x$likelihoodRatioDistribution,
            deltaLR = x$deltaLR,
            weightsDeltaLR = x$weightsDeltaLR,
            tauLR = x$tauLR,
            kappaLR = x$kappaLR,
            deltaMaxLR = x$deltaMaxLR,
            minimumConditionalError = x$minimumConditionalError,
            maximumConditionalError = x$maximumConditionalError,
            minimumSecondStageInformation = x$minimumSecondStageInformation,
            maximumSecondStageInformation = x$maximumSecondStageInformation,
            levelConstantMinimum = x$levelConstantMinimum,
            levelConstantMaximum = x$levelConstantMaximum,
            ncp1Min = x$ncp1Min,
            ncp1Max = x$ncp1Max,
            enforceMonotonicity = FALSE
          ))

          # Calculate optimal conditional error for the new design
          nonMonoSecondStageInformation <- getSecondStageInformation(
            firstStagePValue = firstStagePValues,
            design = secondDesign
          )

          # Add non-monotone optimal conditional error to base plot and display it
          informationPlot +
            ggplot2::geom_line(
              mapping = ggplot2::aes(
                x = firstStagePValues,
                y = nonMonoSecondStageInformation
              ),
              colour = "gray",
              linetype = "dashed",
              linewidth = 1.1
            )
        } else {
          warning(
            "When using plotNonMonotoneFunction=TRUE, x should provide a monotone function. Consider setting enforceMonotonicity=TRUE in design object."
          )
          informationPlot # Display base plot
        }
      }
    } else {
      informationPlot # Display base plot
    }
  } else if (type == 3) {
    # Plot type 3: likelihood ratio
    # Calculate likelihood ratio values
    likelihoodRatios <- getLikelihoodRatio(
      firstStagePValue = firstStagePValues,
      design = x
    )

    likelihoodRatioPlot <- ggplot2::ggplot() +
      ggplot2::geom_line(
        mapping = ggplot2::aes(x = firstStagePValues, y = likelihoodRatios),
        colour = "black",
        linetype = "solid",
        linewidth = 1.1
      ) +
      ggplot2::labs(x = "First-stage p-value", y = "Likelihood ratio") +
      ggplot2::theme_bw() +
      ggplot2::geom_vline(
        xintercept = x$alpha0,
        linetype = "dotted",
        col = "red",
        linewidth = 0.8
      ) +
      ggplot2::geom_vline(
        xintercept = x$alpha1,
        linetype = "dotted",
        col = "blue",
        linewidth = 0.8
      ) +
      ggplot2::xlim(c(range[1], range[2]))

    likelihoodRatioPlot
  } else if (type == 4) {
    # Plot type 4: Q
    firstStagePValues <- seq(
      from = max(range[1], x$alpha1),
      to = min(range[2], x$alpha0),
      length.out = 1e3
    )
    # Calculate Q values
    Q <- getMonotoneFunction(
      x = firstStagePValues,
      fun = getQ,
      design = x
    )
    QPlot <- ggplot2::ggplot() +
      ggplot2::geom_line(
        mapping = ggplot2::aes(x = firstStagePValues, y = Q),
        colour = "black",
        linetype = "solid",
        linewidth = 1.1
      ) +
      ggplot2::labs(x = "First-stage p-value", y = "Q") +
      ggplot2::theme_bw() +
      ggplot2::geom_vline(
        xintercept = x$alpha0,
        linetype = "dotted",
        col = "red",
        linewidth = 0.8
      ) +
      ggplot2::geom_vline(
        xintercept = x$alpha1,
        linetype = "dotted",
        col = "blue",
        linewidth = 0.8
      ) +
      ggplot2::xlim(c(max(range[1], x$alpha1), min(x$alpha0, range[2])))

    if (
      plotNonMonotoneFunction &&
        x$enforceMonotonicity &&
        !is.null(unlist(x$monotonisationConstants))
    ) {
      nonMonoQ <- getQ(
        firstStagePValue = firstStagePValues,
        design = x
      )

      QPlot <- QPlot +
        ggplot2::geom_line(
          mapping = ggplot2::aes(
            x = firstStagePValues,
            y = nonMonoQ
          ),
          colour = "gray",
          linetype = "dashed",
          linewidth = 1.1
        )
    }
    QPlot
  }
}

#' Summary of the optimal conditional error trial design
#'
#' @description
#' Provide an overview of the operating characteristics of the optimal conditional error trial design.
#'
#' @param object Design object of class \code{TrialDesignOptimalConditionalError}
#' @param ... Additional arguments required for generic compatibility
#'
#' @return No return value, prints a summary of design performance.
#'
#' @export
summary.TrialDesignOptimalConditionalError <- function(object, ...) {
  cat("Summary of the Optimal Conditional Error Function Design: \n \n")
  cat("General design parameters: \n")
  cat("  Overall significance level:", object$alpha, "\n")
  cat("  First-stage efficacy boundary (p-value scale):", object$alpha1, "\n")
  cat(
    "  Binding first-stage futility boundary (p-value scale):",
    object$alpha0,
    "\n"
  )
  cat("\n")

  cat("Second-stage information: \n")
  cat(
    "  Expected second-stage information (delta=0):",
    getExpectedSecondStageInformation(
      design = object,
      likelihoodRatioDistribution = "fixed",
      deltaLR = 0
    ),
    "\n"
  )
  if (object$useInterimEstimate == FALSE & length(object$weightsDeltaLR) <= 1) {
    cat(
      "  Expected second-stage information (delta=delta1=",
      object$delta1,
      "): ",
      getExpectedSecondStageInformation(
        design = object,
        likelihoodRatioDistribution = "fixed",
        deltaLR = object$delta1
      ),
      "\n",
      sep = ""
    )
  }
  if (object$useInterimEstimate == TRUE) {
    cat(
      "  Expected second-stage information (delta=delta1Min=",
      object$delta1Min,
      "): ",
      getExpectedSecondStageInformation(
        design = object,
        likelihoodRatioDistribution = "fixed",
        deltaLR = object$delta1Min
      ),
      "\n",
      sep = ""
    )
  }
  cat(
    "  Expected second-stage information (Given likelihood ratio distr.):",
    getExpectedSecondStageInformation(design = object),
    "\n"
  )
  if (object$likelihoodRatioDistribution != "maxlr") {
    if (object$likelihoodRatioDistribution == "fixed") {
      if (length(object$weightsDeltaLR) <= 1) {
        delta <- object$deltaLR
      } else {
        delta <- object$deltaLR %*% object$weightsDeltaLR
      }
    }
    if (object$likelihoodRatioDistribution == "normal") {
      delta <- object$deltaLR
    }
    if (object$likelihoodRatioDistribution == "exp") {
      delta <- object$kappaLR
    }
    if (object$likelihoodRatioDistribution == "unif") {
      delta <- object$deltaMaxLR / 2
    }
    cat(
      "  Expected second-stage information (delta=Mean of given likelihood ratio distr.=",
      delta,
      "): ",
      getExpectedSecondStageInformation(
        design = object,
        likelihoodRatioDistribution = "fixed",
        deltaLR = delta
      ),
      "\n",
      sep = ""
    )
  }
  if (!is.na(object$conditionalPower)) {
    cat(
      "  Maximum second-stage information:",
      getSecondStageInformation(
        design = object,
        firstStagePValue = object$alpha0
      )
    )
  }
  cat("\n \n")

  cat("Power and stopping probabilities: \n")
  if (is.na(object$conditionalPower)) {
    cat("  Conditional power: Data-dependent user-specified function \n")
  } else if (!is.na(object$conditionalPower)) {
    cat("  Conditional power (fixed):", object$conditionalPower, "\n")
  }
  if (object$useInterimEstimate == FALSE & length(object$weightsDeltaLR) <= 1) {
    powerResults <- getOverallPower(
      design = object,
      alternative = object$delta1
    )
    cat(
      "  Overall power (delta1=",
      object$delta1,
      "): ",
      powerResults$overallPower,
      "\n",
      sep = ""
    )
    cat(
      "  Efficacy stopping probability (delta1=",
      object$delta1,
      "): ",
      powerResults$firstStageEfficacy,
      "\n",
      sep = ""
    )
    cat(
      "  Futility stopping probability (delta1=",
      object$delta1,
      "): ",
      powerResults$firstStageFutility,
      "\n",
      sep = ""
    )
  }
  if (object$useInterimEstimate == TRUE) {
    powerResults <- getOverallPower(
      design = object,
      alternative = object$delta1Min
    )
    cat(
      "  Overall power (delta1=delta1Min= ",
      object$delta1Min,
      "): ",
      powerResults$overallPower,
      "\n",
      sep = ""
    )
    cat(
      "  Efficacy stopping probability (delta1=delta1Min= ",
      object$delta1Min,
      "): ",
      powerResults$firstStageEfficacy,
      "\n",
      sep = ""
    )
    cat(
      "  Futility stopping probability (delta1=delta1Min= ",
      object$delta1Min,
      "): ",
      powerResults$firstStageFutility,
      "\n",
      sep = ""
    )
  }
  if (object$likelihoodRatioDistribution != "maxlr") {
    if (object$likelihoodRatioDistribution == "fixed") {
      if (length(object$weightsDeltaLR) <= 1) {
        delta1 <- object$deltaLR
      } else {
        delta1 <- as.numeric(object$deltaLR %*% object$weightsDeltaLR)
      }
    }
    if (object$likelihoodRatioDistribution == "normal") {
      delta1 <- object$deltaLR
    }
    if (object$likelihoodRatioDistribution == "exp") {
      delta1 <- object$kappaLR
    }
    if (object$likelihoodRatioDistribution == "unif") {
      delta1 <- object$deltaMaxLR / 2
    }
    powerResults <- getOverallPower(design = object, alternative = delta1)
    cat(
      "  Overall power (delta1=Mean of given likelihood ratio distr.= ",
      delta1,
      "): ",
      powerResults$overallPower,
      "\n",
      sep = ""
    )
    cat(
      "  Efficacy stopping probability (delta1=Mean of given likelihood ratio distr.= ",
      delta1,
      "): ",
      powerResults$firstStageEfficacy,
      "\n",
      sep = ""
    )
    cat(
      "  Futility stopping probability (delta1=Mean of given likelihood ratio distr.= ",
      delta1,
      "): ",
      powerResults$firstStageFutility,
      "\n",
      sep = ""
    )
  }
  cat("\n")
}


#' Simple range check for numeric variables
#' @description
#' This function performs very basic range checks for numeric variables and throws
#' an error if the range is violated. A custom hint may be added to the message.
#'
#' @param variable The (named) variable to be checked.
#' @param range A vector of length 2 giving the minimum and maximum of the allowed range.
#' @param allowedEqual Logical. Are the borders of range valid values?
#' @param hint Additional message that may be printed after the error.
#'
#' @return Invisibly returns \code{TRUE} if the check was successful.
.rangeCheck <- function(variable, range, allowedEqual, hint = "") {
  if (allowedEqual) {
    if (any(variable < range[1]) || any(variable > range[2])) {
      variableName <- deparse(substitute(variable))
      stop(paste0(
        variableName,
        " must lie in [",
        range[1],
        ", ",
        range[2],
        "]. ",
        hint
      ))
    }
  } else {
    if (any(variable <= range[1]) || any(variable >= range[2])) {
      variableName <- deparse(substitute(variable))
      stop(paste0(
        variableName,
        " must lie in ]",
        range[1],
        ", ",
        range[2],
        "[. ",
        hint
      ))
    }
  }
  return(base::invisible(TRUE))
}

#' Calculates the upper constraint.
#' @name getConstraintC_max
#'
#' @details Internal function that calculates the upper constraint.
#'
#' @template param_firstStagePValue
#' @template param_design
#'
#' @return Value of the upper constraint
#'
#'

getConstraintC_max <- function(firstStagePValue, design){

  C_max_Info <- NULL

  # Constraints on conditional error scale
  C_max_cond_error <- design$maximumConditionalError

  # Identify conditional power to be supplied to getPsi and to be used for (information) constraints
  # Check if conditional power function should be used
  if(!is.null(suppressWarnings(body(design$conditionalPowerFunction)))) {
    conditionalPower <- design$conditionalPowerFunction(firstStagePValue)

    #Calculate possibly data-dependent constraints
    #Identify effect for constraints
    if(design$useInterimEstimate) {
      delta1 <- pmin(pmax(qnorm(1-firstStagePValue)/sqrt(design$firstStageInformation), design$delta1Min), design$delta1Max)
    } else {
      delta1 <- design$delta1
    }

    #If minimumSecondStageInformation is given, use this
    if(design$minimumSecondStageInformation > 0){
      C_max_Info <- 1 - pnorm(delta1* sqrt(design$minimumSecondStageInformation)-qnorm(conditionalPower))
    }

  } else {
    conditionalPower <- design$conditionalPower

    #Check if interim estimate is used
    if(design$useInterimEstimate) {
      delta_C_max <- min(qnorm(1-design$alpha1)/sqrt(design$firstStageInformation),design$delta1Max)
    }
    # Otherwise use fixed effect
    else {
      delta_C_max <- design$delta1
    }

    #If minimumSecondStageInformation is given, use this
    if(design$minimumSecondStageInformation > 0){
      C_max_Info <- 1 - pnorm(delta_C_max* sqrt(design$minimumSecondStageInformation)-qnorm(conditionalPower))
    }
  }

  #Use the constraint that is the stronger restriction
  C_max <- min(C_max_Info, C_max_cond_error)

  return(C_max)
}

#' Calculate Likelihood Ratio
#'
#' @name getLikelihoodRatio
#'
#' @description Calculate the likelihood ratio of a p-value for a given distribution.
#'
#' @details The calculation of the likelihood ratio for a first-stage p-value \eqn{p_1} is done based on a distributional assumption, specified in the \code{design} object.
#' The different options require different parameters, elaborated in the following.
#' \itemize{
#'    \item \code{likelihoodRatioDistribution="fixed"}: calculates the likelihood ratio for a fixed \eqn{\Delta}. The non-centrality parameter of the likelihood ratio \eqn{\vartheta} is then computed as \code{deltaLR}*\code{sqrt(firstStageInformation)} and the likelihood ratio is calculated as:
#'          \deqn{l(p_1) = e^{\Phi^{-1}(1-p_1)\vartheta - \vartheta^2/2}.} \code{deltaLR} may also contain multiple elements, in which case a weighted likelihood ratio is calculated for the given values. Unless positive weights that sum to 1 are provided by the argument \code{weightsDeltaLR}, equal weights are assumed.
#'    \item \code{likelihoodRatioDistribution="normal"}: calculates the likelihood ratio for a normally distributed prior of \eqn{\vartheta} with mean \code{deltaLR}*\code{sqrt(firstStageInformation)} (\eqn{\mu}) and standard deviation \code{tauLR}*\code{sqrt(firstStageInformation)} (\eqn{\sigma}). The parameters \code{deltaLR} and \code{tauLR} must be specified on the mean difference scale.
#'          \deqn{l(p_1) = (1+\sigma^2)^{-\frac{1}{2}}\cdot e^{-(\mu/\sigma)^2/2 + (\sigma\Phi^{-1}(1-p_1) + \mu/\sigma)^2 / (2\cdot (1+\sigma^2))}}
#'    \item \code{likelihoodRatioDistribution="exp"}: calculates the likelihood ratio for an exponentially distributed prior of \eqn{\vartheta} with mean \code{kappaLR}*\code{sqrt(firstStageInformation)} (\eqn{\eta}). The likelihood ratio is then calculated as:
#'          \deqn{l(p_1) = \eta \cdot \sqrt{2\pi} \cdot e^{(\Phi^{-1}(1-p_1)-\eta)^2/2} \cdot \Phi(\Phi^{-1}(1-p_1)-\eta)}
#'    \item \code{likelihoodRatioDistribution="unif"}: calculates the likelihood ratio for a uniformly distributed prior of \eqn{\vartheta} on the support \eqn{[0, \Delta\cdot\sqrt{I_1}]}, where \eqn{\Delta} is specified as \code{deltaMaxLR} and \eqn{I_1} is the \code{firstStageInformation}.
#'          \deqn{l(p_1) = \frac{\sqrt{2\pi}}{\Delta\cdot\sqrt{I_1}} \cdot e^{\Phi^{-1}(1-p_1)^2/2} \cdot (\Phi(\Delta\cdot\sqrt{I_1} - \Phi^{-1}(1-p_1))-p_1)}
#'    \item \code{likelihoodRatioDistribution="maxlr"}: the non-centrality parameter \eqn{\vartheta} is estimated from the data and no additional parameters must be specified. The likelihood ratio is estimated from the data as:
#'          \deqn{l(p_1) = e^{max(0, \Phi^{-1}(1-p_1))^2/2}}
#'          The maximum likelihood ratio is always restricted to effect sizes \eqn{\vartheta \geq 0} (corresponding to \eqn{p_1 \leq 0.5}).
#' }
#'
#' @template param_firstStagePValue
#' @template param_design
#'
#' @return The value of the likelihood ratio for the given specification.
#' @export
#'
#' @examples
#' # Get a design
#' design <- getDesignOptimalConditionalErrorFunction(
#' alpha = 0.025, alpha1 = 0.001, alpha0 = 0.5, conditionalPower = 0.9,
#' delta1 = 0.25, likelihoodRatioDistribution = "fixed", deltaLR = 0.25,
#' firstStageInformation = 80, useInterimEstimate = FALSE,
#' )
#'
#' getLikelihoodRatio(firstStagePValue = c(0.05, 0.1, 0.2), design = design)
#'
#' @template reference_optimal
#' @references Hung, H. M. J., O’Neill, R. T., Bauer, P. & Kohne, K. (1997). The behavior of the p-value when the alternative hypothesis is true. Biometrics. https://www.jstor.org/stable/2533093

getLikelihoodRatio <- function(firstStagePValue, design) {
  # Initialise likelihood ratio
  likelihoodRatio <- NA
  # Fixed effect case
  if (design$likelihoodRatioDistribution == "fixed") {
    # Get ncp and weights

    # Ensure that ncp argument is provided
    # This is a fallback protection, as the design function should already ensure this
    if (is.null(design$deltaLR)) {
      stop(
        "Argument deltaLR required for fixed likelihood case, but not found in design object."
      )
    }

    ncp <- design$deltaLR * sqrt(design$firstStageInformation)
    weights <- design$weightsDeltaLR

    # If weights argument was not specified, automatically use equal weights
    if (is.null(weights)) {
      weights <- rep(1 / length(ncp), length(ncp))
    }

    # Ensure that weights are positive and sum up to 1
    # This is a fallback protection, as the design function should already ensure this
    if (sum(weights) != 1 || any(weights < 0)) {
      stop("weightsDeltaLR must be positive and sum up to 1")
    }

    # Calculate likelihood ratio
    likelihoodRatio <- exp(qnorm(1 - firstStagePValue) * ncp - ncp^2 / 2) %*%
      weights
  } else if (design$likelihoodRatioDistribution == "normal") {
    # Normal prior
    # Get ncp and tau

    # Ensure that arguments were specified
    # This is a fallback protection, as the design function should already ensure this
    if (is.null(design$deltaLR) || is.null(design$tauLR)) {
      stop(
        "Arguments deltaLR and tauLR required for normal likelihood case, but not found in design object."
      )
    }

    ncp <- design$deltaLR * sqrt(design$firstStageInformation)
    tau <- design$tauLR * sqrt(design$firstStageInformation)

    # Calculate likelihood ratio
    likelihoodRatio <- (1 / sqrt(1 + tau^2)) *
      exp(
        -(ncp / tau)^2 /
          2 +
          (tau * qnorm(1 - firstStagePValue) + (ncp / tau))^2 /
            (2 * (1 + tau^2))
      )
  } else if (design$likelihoodRatioDistribution == "exp") {
    # Exponential prior
    # Get ncp

    # Ensure that argument was specified
    # This is a fallback protection, as the design function should already ensure this
    if (is.null(design$kappaLR)) {
      stop(
        "Argument kappaLR required for exponential likelihood case, but not found in design object."
      )
    }

    ncp <- design$kappaLR * sqrt(design$firstStageInformation)

    # Calculate likelihood ratio
    likelihoodRatio <- sqrt(2 * pi) *
      ncp *
      exp((stats::qnorm(1 - firstStagePValue) - ncp)^2 / 2) *
      stats::pnorm(stats::qnorm(1 - firstStagePValue) - ncp)
  } else if (design$likelihoodRatioDistribution == "unif") {
    # Uniform prior
    # Get ncp

    # Ensure that argument was specified
    # This is a fallback protection, as the design function should already ensure this
    if (is.null(design$deltaMaxLR)) {
      stop(
        "Argument deltaMaxLR required for uniform likelihood case, but not found in design object."
      )
    }

    ncp <- design$deltaMaxLR * sqrt(design$firstStageInformation)

    # Calculate likelihood ratio
    likelihoodRatio <- sqrt(2 * pi) *
      exp(stats::qnorm(1 - firstStagePValue)^2 / 2) *
      (stats::pnorm(ncp - stats::qnorm(1 - firstStagePValue)) -
        firstStagePValue) /
      ncp
  } else if (design$likelihoodRatioDistribution == "maxlr") {
    # Maximum likelihood ratio case
    # Calculate likelihood ratio
    likelihoodRatio <- exp(max(0, stats::qnorm(1 - firstStagePValue))^2 / 2)
  } else {
    stop("Distribution not matched.")
  }

  # Return likelihood ratio
  return(unname(likelihoodRatio))
}

getLikelihoodRatio <- Vectorize(getLikelihoodRatio, "firstStagePValue")

testthat::test_that("Likelihood Ratio correctly calculated", {
  # Sequence of p-values
  pValues <- c(0.001, 0.002, 0.05, 0.1, 0.2, 0.3, 0.4, 0.5, 1)

  # Only specify required information to reduce total testing time

  # Fixed likelihood ratio
  design_fixed_list <- list(
    "likelihoodRatioDistribution" = "fixed",
    "deltaLR" = 0.25,
    "firstStageInformation" = 80
  )

  likelihoodRatio_fixed <- getLikelihoodRatio(
    firstStagePValue = pValues,
    design = design_fixed_list
  )

  testthat::expect_equal(
    object = likelihoodRatio_fixed,
    expected = c(
      82.2669546,
      51.2013193,
      3.2478866,
      1.4414311,
      0.5389795,
      0.2651645,
      0.1446413,
      0.0820850,
      0.0000000
    ),
    tolerance = 1e-5
  )

  # Normally distributed likelihood ratio
  design_normal_list <- list(
    "likelihoodRatioDistribution" = "normal",
    "deltaLR" = 0.25,
    "tauLR" = 0.5,
    "firstStageInformation" = 80
  )

  likelihoodRatio_normal <- getLikelihoodRatio(
    firstStagePValue = pValues,
    design = design_normal_list
  )

  testthat::expect_equal(
    object = likelihoodRatio_normal,
    expected = c(
      25.4098091,
      13.5967390,
      0.8371000,
      0.4854073,
      0.2968895,
      0.2335123,
      0.2052004,
      0.1937263,
      Inf
    ),
    tolerance = 1e-5
  )

  # Exponentially distributed likelihood ratio
  design_exp_list <- list(
    "likelihoodRatioDistribution" = "exp",
    "kappaLR" = 0.25,
    "firstStageInformation" = 80
  )

  likelihoodRatio_exp <- getLikelihoodRatio(
    firstStagePValue = pValues,
    design = design_exp_list
  )

  testthat::expect_equal(
    object = likelihoodRatio_exp,
    expected = c(
      6.4861496,
      5.0944188,
      1.8503353,
      1.5018965,
      1.2090883,
      1.0545168,
      0.9483223,
      0.8653926,
      NaN
    ),
    tolerance = 1e-5
  )

  # Uniformly distributed likelihood ratio
  design_uniform_list <- list(
    "likelihoodRatioDistribution" = "unif",
    "deltaMaxLR" = 0.5,
    "firstStageInformation" = 80
  )

  likelihoodRatio_uniform <- getLikelihoodRatio(
    firstStagePValue = pValues,
    design = design_uniform_list
  )

  testthat::expect_equal(
    object = likelihoodRatio_uniform,
    expected = c(
      60.7978625,
      33.2412443,
      2.0545914,
      1.1458079,
      0.6388505,
      0.4501561,
      0.3472601,
      0.2802474,
      NaN
    ),
    tolerance = 1e-5
  )

  # Maximum likelihood ratio
  design_maxlr_list <- list(
    "likelihoodRatioDistribution" = "maxlr",
    "firstStageInformation" = 80
  )

  likelihoodRatio_maxlr <- getLikelihoodRatio(
    firstStagePValue = pValues,
    design = design_maxlr_list
  )

  testthat::expect_equal(
    object = likelihoodRatio_maxlr,
    expected = c(
      118.482806,
      62.922732,
      3.868132,
      2.273197,
      1.424988,
      1.147399,
      1.032613,
      1.000000,
      1.000000
    ),
    tolerance = 1e-5
  )
})

#' Simulate trials
#'
#' @description
#' Simulate the rejection probability for a given design and alternative.
#'
#' @details
#' Simulates the probabilities of overall rejection as well as early futility and early efficacy for the provided scenario and design.
#' This is done by generating random normally distributed test statistics and calculating their p-values.
#'
#' @template param_design
#' @param maxNumberOfIterations Number of trials to be simulated.
#' @template param_alternative
#' @param seed An optional seed for reproducibility.
#'
#' @return An object of class \code{SimulationResultsOptimalConditionalError} containing the simulation results.
#'
#' @importFrom methods new
#' @importFrom stats pnorm
#' @importFrom stats rnorm
#'
#' @seealso [getDesignOptimalConditionalErrorFunction()], [getOverallPower()]
#'
#' @export
#' @examples
#' \donttest{
#' design <- getDesignOptimalConditionalErrorFunction(
#'  alpha = 0.025, alpha1 = 0.001, alpha0 = 0.5, delta1 = 0.25,
#'  useInterimEstimate = FALSE,
#'  conditionalPower = 0.9, likelihoodRatioDistribution = "maxlr",
#'  firstStageInformation = 10
#' )
#'
#' # Simulate under the null hypothesis and for a mean difference of 0.5
#' getSimulationResults(
#'  design = design, alternative = c(0, 0.5)
#' )
#'
#' }

getSimulationResults <- function(
  design,
  maxNumberOfIterations = 10000,
  alternative,
  seed = NULL
) {
  # If seed provided, use it
  if (!is.null(seed)) {
    set.seed(seed)
  }

  # Create empty vectors
  firstStageRejections <- numeric()
  firstStageFutility <- numeric()
  overallRejections <- numeric()

  # For each item of the alternative, perform simulations
  for (i in 1:length(alternative)) {
    # Generate first-stage p-values with desired effect size
    firstStagePValues <- 1 -
      stats::pnorm(stats::rnorm(
        n = maxNumberOfIterations,
        mean = alternative[i] * sqrt(design$firstStageInformation)
      ))

    # Proportion of early rejections
    firstStageRejections <- c(
      firstStageRejections,
      sum(firstStagePValues <= design$alpha1) / length(firstStagePValues)
    )
    # Proportion of early futility stops
    firstStageFutility <- c(
      firstStageFutility,
      sum(firstStagePValues > design$alpha0) / length(firstStagePValues)
    )

    optimalConditionalErrors <- getOptimalConditionalError(
      firstStagePValue = firstStagePValues,
      design = design
    )

    # Generate second-stage p-values with desired effect size
    secondStagePValues <- 1 -
      stats::pnorm(stats::rnorm(
        n = maxNumberOfIterations,
        mean = alternative[i] *
          sqrt(getSecondStageInformation(
            firstStagePValue = firstStagePValues,
            design = design
          ))
      ))

    # Proportion of overall rejections
    overallRejections <- c(
      overallRejections,
      sum(secondStagePValues <= optimalConditionalErrors) /
        length(secondStagePValues)
    )
  }

  # Create new object
  simulationResults <- new(
    "SimulationResultsOptimalConditionalError",
    alternative = alternative,
    firstStageFutility = firstStageFutility,
    firstStageEfficacy = firstStageRejections,
    overallPower = overallRejections,
    maxNumberOfIterations = maxNumberOfIterations
  )

  return(simulationResults)
}

#' Calculate the Optimal Conditional Error
#' @name getOptimalConditionalError
#'
#' @details The optimal conditional error \eqn{\alpha_2} given a first-stage p-value \eqn{p_1} is calculated as:
#' \deqn{\alpha_2(p_1)=\psi(-e^{c_0} \cdot \frac{\Delta_1^2}{l(p_1)}).}
#'
#' The level constant \eqn{c_0} as well as the specification of the effect size \eqn{\Delta_1} and the likelihood ratio \eqn{l(p_1)}
#' must be contained in the \code{design} object (see \code{?getDesignOptimalConditionalErrorFunction}).
#' Early stopping rules are supported, i.e., for \eqn{p_1 \leq \alpha_1}, the returned conditional error is 1 and for \eqn{p_1 > \alpha_0}, the returned conditional error is 0.
#'
#'
#' @template param_firstStagePValue
#' @template param_design
#'
#' @return Value of the optimal conditional error function.
#' @export
#'
#' @template reference_optimal
#'
#' @seealso [getDesignOptimalConditionalErrorFunction()]
#'
#' @examples
#' # Create a design
#' design <- getDesignOptimalConditionalErrorFunction(
#' alpha = 0.025, alpha1 = 0.001, alpha0 = 0.5, conditionalPower = 0.9,
#' delta1 = 0.5, firstStageInformation = 40, useInterimEstimate = FALSE,
#' likelihoodRatioDistribution = "fixed", deltaLR = 0.5)
#'
#' # Calculate optimal conditional error
#' getOptimalConditionalError(
#' firstStagePValue = c(0.1, 0.2, 0.3), design = design
#' )

getOptimalConditionalError <- function(firstStagePValue, design) {
  # Check if firstStagePValue lies outside early decision boundaries
  if (firstStagePValue <= design$alpha1 && design$alpha1 != 0) {
    return(1)
  } else if (firstStagePValue > design$alpha0) {
    return(0)
  }

  # If monotonisation constants specified and monotonisation enforced, perform non-increasing transformation
  if (
    design$enforceMonotonicity &&
      !is.null(unlist(design$monotonisationConstants))
  ) {
    Q <- getMonotoneFunction(
      x = firstStagePValue,
      fun = getQ,
      design = design
    )
  } else {
    Q <- getQ(firstStagePValue = firstStagePValue, design = design)
  }

  # Check if conditional power function should be used
  if (!is.null(suppressWarnings(body(design$conditionalPowerFunction)))) {
    conditionalPower <- design$conditionalPowerFunction(firstStagePValue)
  } else {
    conditionalPower <- design$conditionalPower
  }

  #Calculate the constraints
  C_max <- getConstraintC_max(firstStagePValue = firstStagePValue, design = design)
  C_min <- getConstraintC_min(firstStagePValue = firstStagePValue, design = design)

  #Handling of the special case firstStagePValue=0 and no early stopping
  if (firstStagePValue == 0 && design$alpha1 == 0) {
    # Calculate the specified conditional power for a firstStagePValue of 0
    if (!is.null(suppressWarnings(body(design$conditionalPowerFunction)))) {
      conditionalPower_0 <- design$conditionalPowerFunction(0)
    } else {
      conditionalPower_0 <- design$conditionalPower
    }
    return(min(C_max, conditionalPower_0))
  }

  return(max(
    C_min,
    min(
      C_max,
      getPsi(
        nuPrime = (-exp(design$levelConstant) / Q),
        conditionalPower = conditionalPower,
        design = design,
        firstStagePValue = firstStagePValue
      )
    )
  ))
}

getOptimalConditionalError <- Vectorize(
  FUN = getOptimalConditionalError,
  vectorize.args = c("firstStagePValue")
)

#' Integrate over information
#'
#' @description Internal function used by \code{getExpectedSecondStageInformation()} to calculate the integral over the information.
#'
#' @template param_firstStagePValue
#' @template param_likelihoodRatioDistribution
#' @template param_design
#' @param ... Additional arguments needed for \code{getOptimalConditionalError()} and \code{getLikelihoodRatio()}.
#'
#' @return Integral over the information of the second stage
#'
#' @keywords internal

integrateExpectedInformation <- function(firstStagePValue, design, likelihoodRatioDistribution, ...) {

  # Calculate optimal conditional error function
  optimalCondErr <- getOptimalConditionalError(
    firstStagePValue = firstStagePValue, design = design)

  # Identify how the likelihood ratio should be calculated
  likelihoodRatio <- NA
  args <- list(...)

  # Fixed effect
  # If NULL, use specification in design object
  if(is.null(likelihoodRatioDistribution)) {
    ghostDesign <- design
    likelihoodRatio <- getLikelihoodRatio(
      firstStagePValue = firstStagePValue, design = ghostDesign
    )
  }
  else if(likelihoodRatioDistribution == "fixed") {
    deltaLR <- unlist(args["deltaLR"])
    weights <- unlist(args["weightsDeltaLR"])

    # Ensure argument specified
    if(is.null(deltaLR)) {
      stop("Argument deltaLR must be provided for fixed likelihood ratio case.")
    }
    if(is.null(weights)) {
      weights <- rep(1/length(deltaLR), length(deltaLR))
    }

    # Create a list that acts as a design object to calculate true likelihood ratio
    ghostDesign <- list("likelihoodRatioDistribution" = likelihoodRatioDistribution,
                        "deltaLR" = deltaLR, "weightsDeltaLR" = weights,
                        "firstStageInformation" = design$firstStageInformation)

    # Calculate likelihood ratio
    likelihoodRatio <- getLikelihoodRatio(
      firstStagePValue = firstStagePValue, design = ghostDesign
    )
  }
  # Normal prior for effect
  else if(likelihoodRatioDistribution == "normal") {
    deltaLR <- unlist(args["deltaLR"])
    tauLR <- unlist(args["tauLR"])

    # Ensure arguments specified
    if(is.null(deltaLR) || is.null(tauLR)) {
      stop("Arguments deltaLR and tauLR must be provided for fixed likelihood ratio case.")
    }

    # Create a list that acts as a design object to calculate true likelihood ratio
    ghostDesign <- list("likelihoodRatioDistribution" = likelihoodRatioDistribution,
                        "deltaLR" = deltaLR, "tauLR" = tauLR,
                        "firstStageInformation" = design$firstStageInformation)

    # Calculate likelihood ratio
    likelihoodRatio <- getLikelihoodRatio(
      firstStagePValue = firstStagePValue, design = ghostDesign
    )
  }
  # Exponential prior for effect
  else if(likelihoodRatioDistribution == "exp") {

    kappaLR <- unlist(args["kappaLR"])

    # Ensure argument specified
    if(is.null(kappaLR)) {
      stop("Argument kappaLR must be specified for exponential likelihood case.")
    }

    # Create a list that acts as a design object to calculate true likelihood ratio
    ghostDesign <- list("likelihoodRatioDistribution" = likelihoodRatioDistribution,
                        "kappaLR" = kappaLR,
                        "firstStageInformation" = design$firstStageInformation)

    # Calculate likelihood ratio
    likelihoodRatio <- getLikelihoodRatio(
      firstStagePValue = firstStagePValue, design = ghostDesign
    )
  }
  # Uniform prior for effect
  else if(likelihoodRatioDistribution == "unif") {

    deltaMax <- unlist(args["deltaMaxLR"])

    # Ensure argument specified
    if(is.null(deltaMax)) {
      stop("Argument deltaMaxLR must be specified for uniform likelihood case.")
    }

    # Create a list that acts as a design object to calculate true likelihood ratio
    ghostDesign <- list("likelihoodRatioDistribution" = likelihoodRatioDistribution,
                        "deltaMaxLR" = deltaMax,
                        "firstStageInformation" = design$firstStageInformation)

    # Calculate likelihood ratio
    likelihoodRatio <- getLikelihoodRatio(
      firstStagePValue = firstStagePValue, design = ghostDesign
    )
  }
  # Maximum likelihood ratio
  else if(likelihoodRatioDistribution == "maxlr") {
    # Create a list that acts as a design object to calculate true likelihood ratio
    ghostDesign <- list("likelihoodRatioDistribution" = likelihoodRatioDistribution)

    # Calculate likelihood ratio
    likelihoodRatio <- getLikelihoodRatio(
      firstStagePValue = firstStagePValue, design = ghostDesign
    )
  }
  # Unknown distribution specified
  else {
    stop("Distribution not matched.")
  }

  # Identify effect size to calculate second-stage information
  # The effect is the same as the one specified in the design object.

  # Fixed effect case
  if(!design$useInterimEstimate) {
    delta1 <- design$delta1
  }
  # Interim estimate
  else {
    # Apply restrictions that are given in the design object
    delta1 <- pmin(pmax(design$delta1Min, qnorm(1-firstStagePValue)/sqrt(design$firstStageInformation)), design$delta1Max)
  }

  # Check if conditional power function should be used
  if(!is.null(suppressWarnings(body(design$conditionalPowerFunction)))) {
    conditionalPower <- design$conditionalPowerFunction(firstStagePValue)
  }
  else {
    conditionalPower <- design$conditionalPower
  }

  return((getNu(alpha=optimalCondErr,  conditionalPower = conditionalPower)*likelihoodRatio) / (delta1^2))

}

#' Calculate the Derivate of Nu
#' @name getNuPrime
#'
#' @description Calculates the derivative of nu for a given conditional error and conditional power.
#'
#' @details The function \eqn{\nu'} is defined as
#' \deqn{\nu'(p_1) = -2 \cdot (\Phi^{-1}(1-\alpha_2(p_1)) + \Phi^{-1}(CP))/\phi(\Phi^{-1}(1-\alpha_2(p_1))).}
#' Note that in this implementation, the the factor -2 is used instead of -4, which is used in by Brannath & Bauer (2004), who explicitly investigate the setting of a balanced two-group trial.
#' The argument \code{conditionalPower} is either the fixed target conditional power or the value of the conditional power function at the corresponding first-stage p-value.
#'
#' @template param_alpha_cerr
#' @template param_conditionalPower
#'
#' @return Value for nu prime.
#' @export
#'
#' @examples
#' getNuPrime(alpha = 0.05, conditionalPower = 0.9)
#'
#' @template reference_optimal
#'
getNuPrime <- function(alpha, conditionalPower) {

    return(-2*(qnorm(1-alpha)+qnorm(conditionalPower))/dnorm(qnorm(1-alpha)))
}

getNuPrime <- Vectorize(FUN = getNuPrime, vectorize.args = "alpha")

test_that("NuPrime correctly calculated", {
  values <- seq(0, 1, 0.001)
  mdnu <- function(u, cp) {
    -2 * (qnorm(cp) - qnorm(u)) / dnorm(qnorm(u))
  }
  expect_equal(mdnu(values, 0.8), getNuPrime(values, conditionalPower = 0.8), tolerance = 1e-8)
  expect_equal(mdnu(values, 0.9), getNuPrime(values, conditionalPower = 0.9), tolerance = 1e-8)
})

#' Calculate Psi, the Inverse of Nu Prime
#' @name getPsi
#'
#' @description Get point-wise values of psi (inverse of nu prime)
#'
#' @param nuPrime The function value to be inverted.
#' @template param_conditionalPower
#' @template param_design
#' @template param_firstStagePValue
#'
#' @return The value of alpha which corresponds to nuPrime and lies between 0 and \code{conditionalPower}.
#' @export
#'
#' @details
#' The function \eqn{\psi} is the inverse of:
#' \deqn{\nu'(\alpha) = -2 \cdot(\Phi^{-1}(1-\alpha) + \Phi^{-1}(1-CP)) / \phi(\Phi^{-1}(1-\alpha))}.
#' If the conditional power \eqn{CP} lies outside of the range \eqn{1-\Phi(2) \leq CP \leq \Phi(2)}, the calculation is slightly more complicated (design and firstStagePValue need to be provided).
#' The argument \code{conditionalPower} is either the fixed target conditional power or the value of the conditional power function at the corresponding first-stage p-value.
#'
#' @examples
#' # Returns 0.05
#' getPsi(getNuPrime(alpha = 0.05, conditionalPower = 0.9), conditionalPower = 0.9,
#'   design = NULL, firstStagePValue = NULL)

getPsi <- function(nuPrime, conditionalPower, design, firstStagePValue) {
  # If the conditional power is between 1-pnorm(2) and pnorm(2) nu prime is monotone and we can build the inverse directly
  if ((pnorm(-2) <= conditionalPower & conditionalPower <= pnorm(2))) {
    rootlist <- uniroot(
      f = function(alpha) {
        getNuPrime(alpha = alpha, conditionalPower = conditionalPower) - nuPrime
      },
      lower = 0,
      upper = conditionalPower,
      tol = 1e-16
    )
    return(rootlist$root)

    # If the conditional power is not between 1-pnorm(2) and pnorm(2) nu prime is not monotone and we need to build the inverse differently
  } else {
    # Calculate the minimum and the maximum of NuPrime(u)
    u_max <- 1 -
      pnorm(
        -qnorm(conditionalPower) / 2 + sqrt(qnorm(conditionalPower)^2 / 4 - 1)
      )
    u_min <- 1 -
      pnorm(
        -qnorm(conditionalPower) / 2 - sqrt(qnorm(conditionalPower)^2 / 4 - 1)
      )
    NuPrime_u_max <- getNuPrime(
      alpha = u_max,
      conditionalPower = conditionalPower
    )
    NuPrime_u_min <- getNuPrime(
      alpha = u_min,
      conditionalPower = conditionalPower
    )

    if (nuPrime > NuPrime_u_max) {
      rootlist <- uniroot(
        f = function(alpha) {
          getNuPrime(alpha = alpha, conditionalPower = conditionalPower) -
            nuPrime
        },
        lower = u_min,
        upper = conditionalPower,
        tol = 1e-16
      )
      return(rootlist$root)
    } else if (nuPrime < NuPrime_u_min) {
      rootlist <- uniroot(
        f = function(alpha) {
          getNuPrime(alpha = alpha, conditionalPower = conditionalPower) -
            nuPrime
        },
        lower = 0,
        upper = u_max,
        tol = 1e-16
      )
      return(rootlist$root)
    } else {
      #In this case the constraints already need to be considered when building the inverse
      C_max <- getConstraintC_max(
        firstStagePValue = firstStagePValue,
        design = design
      )
      C_min <- getConstraintC_min(
        firstStagePValue = firstStagePValue,
        design = design
      )

      # Calculate psi_lower_c and psi_upper_c
      rootlist1 <- uniroot(
        f = function(alpha) {
          getNuPrime(alpha = alpha, conditionalPower = conditionalPower) -
            nuPrime
        },
        lower = 0,
        upper = u_max,
        tol = 1e-16
      )
      psi_lower_c <- rootlist1$root
      psi_lower_c <- pmax(C_min, pmin(C_max, rootlist1$root))
      rootlist2 <- uniroot(
        f = function(alpha) {
          getNuPrime(alpha = alpha, conditionalPower = conditionalPower) -
            nuPrime
        },
        lower = u_min,
        upper = conditionalPower,
        tol = 1e-16
      )
      psi_upper_c <- pmax(C_min, pmin(C_max, rootlist2$root))
      psi_upper_c <- rootlist2$root
      # Calculate the quotient that is needed to decide if psi_lower_c or psi_upper_c is used
      quotient <- getNu(
        alpha = psi_upper_c,
        conditionalPower = conditionalPower
      ) -
        getNu(alpha = psi_lower_c, conditionalPower = conditionalPower) /
          (psi_upper_c - psi_lower_c)
      if (quotient <= nuPrime) {
        return(psi_upper_c)
      } else {
        return(psi_lower_c)
      }
    }
  }
}

getPsi <- Vectorize(
  FUN = getPsi,
  vectorize.args = c("nuPrime", "conditionalPower")
)

#' Calculate Q
#' @name getQ
#' @description Calculate the ratio of likelihood ratio and squared effect size.
#'
#' @template param_firstStagePValue
#' @template param_design
#'
#' @details For more information on how to specify the likelihood ratio, see \code{?getLikelihoodRatio()}.
#' In case the optimal conditional error function is ever increasing in the first-stage p-value \eqn{p_1}, a monotone transformation of \code{getQ()}
#' is needed for logical consistency and type I error rate control. \cr
#' The formula for \eqn{Q(p_1)} is:
#' \deqn{Q(p_1) = l(p_1) / \Delta_1^2,}
#' where \eqn{l(p_1)} is the likelihood ratio and \eqn{\Delta_1} is the effect size at which the conditional power should be achieved.
#' The effect size may also depend on the interim data (i.e., on \eqn{p_1}) in case \code{useInterimEstimate = TRUE} was specified for the design object.
#'
#' @importFrom stats qnorm
#'
#' @return Ratio of likelihood ratio and squared effect size.
#' @export
#'
#' @examples
#' # Get a design
#' design <- getDesignOptimalConditionalErrorFunction(
#' alpha = 0.025, alpha1 = 0.001, alpha0 = 0.5, conditionalPower = 0.9,
#' delta1 = 0.25, likelihoodRatioDistribution = "fixed", deltaLR = 0.25,
#' firstStageInformation = 80, useInterimEstimate = FALSE,
#' )
#'
#' getQ(firstStagePValue = c(0.05, 0.1, 0.2), design = design)
#'
#' @template reference_monotone

getQ <- function(firstStagePValue, design) {
  # Initialise effect and likelihood ratio
  effect <- NA
  likelihoodRatio <- NA

  # When using interim estimate, apply the restrictions given in the design
  if (design$useInterimEstimate) {
    effect <- min(
      max(design$ncp1Min, stats::qnorm(1 - firstStagePValue)),
      design$ncp1Max
    ) /
      sqrt(design$firstStageInformation)
  } else {
    # Fixed effect case
    effect <- design$delta1
  }

  likelihoodRatio <- getLikelihoodRatio(
    firstStagePValue = firstStagePValue,
    design = design
  )

  Q <- likelihoodRatio / (effect^2)

  return(Q)
}

getQ <- Vectorize(getQ, vectorize.args = "firstStagePValue")

#' Calculate the integral over a partially constant function
#'
#' @description This function aims to provide a more accurate integration routine than the \code{integrate()} function
#' for a partially constant function.
#'
#' @template param_constant_integrate
#' @template param_design
#'
#' @return Integral over the partially constant function.
#' @keywords internal
#'

getIntegralWithConstants <- function(constant, design) {

  conditionalPower <- design$conditionalPower

  # Number of non-decreasing intervals of the optimal conditional error function
  numberOfIntervals <- length(design$monotonisationConstants$qs)

  # Integrate over the first (non-constant) part of the function
  firstPart <- stats::integrate(
    f = getInnerPsi, lower = design$alpha1, upper = design$monotonisationConstants$dls[1],
    constant = constant, design = design)$value

  # Integrate over the final (non-constant) part of the function
  lastPart <- stats::integrate(
    f = getInnerPsi, lower = design$monotonisationConstants$dus[numberOfIntervals], upper = design$alpha0,
    constant = constant, design = design)$value

  # Calculate integral for all constant parts
  constantParts <- 0
  for (x in 1:numberOfIntervals){
    newPart <- getPsi(nuPrime = -exp(constant)/design$monotonisationConstants$qs[x],
                   conditionalPower = conditionalPower)*
      (design$monotonisationConstants$dus[x] - design$monotonisationConstants$dls[x])
    constantParts <- constantParts + newPart
  }

  # Calculate integral for all remaining non-constant parts
  nonConstantParts <- 0
  if (numberOfIntervals>1){
    for (x in 1:(numberOfIntervals-1)){
      newPart <- stats::integrate(
        f = getInnerPsi, lower = design$monotonisationConstants$dus[x], upper = design$monotonisationConstants$dls[x+1],
        constant = constant, design = design)$value
      nonConstantParts <- nonConstantParts + newPart
    }
  }

  # Return the sum of all parts
  return(sum(c(firstPart, lastPart, constantParts, nonConstantParts)))
}

#' @name TrialDesignOptimalConditionalError
#' @title Optimal Conditional Error Design
#' @description
#' A class for a trial design object using the optimal conditional error function.
#' @details
#' This object should not be created directly; use \code{getDesignOptimalConditionalErrorFunction()} with suitable arguments to create a design.
#'
TrialDesignOptimalConditionalError <- setRefClass(
  Class = "TrialDesignOptimalConditionalError",
  fields = list(
    alpha = "numeric",
    alpha1 = "numeric",
    alpha0 = "numeric",
    conditionalPower = "numeric",
    conditionalPowerFunction = "function",
    delta1 = "numeric",
    delta1Min = "numeric",
    delta1Max = "numeric",
    firstStageInformation = "numeric",
    useInterimEstimate = "logical",
    likelihoodRatioDistribution = "character",
    deltaLR = "numeric",
    weightsDeltaLR = "numeric",
    tauLR = "numeric",
    kappaLR = "numeric",
    deltaMaxLR = "numeric",
    levelConstant = "numeric",
    monotonisationConstants = "list",
    minimumSecondStageInformation = "numeric",
    maximumSecondStageInformation = "numeric",
    minimumConditionalError = "numeric",
    maximumConditionalError = "numeric",
    levelConstantMinimum = "numeric",
    levelConstantMaximum = "numeric",
    ncp1 = "numeric",
    ncp1Min = "numeric",
    ncp1Max = "numeric",
    enforceMonotonicity = "logical"
  ),
  methods = list(
    initialize = function(
    alpha = NA_real_,
    alpha1 = NA_real_,
    alpha0 = NA_real_,
    conditionalPower = NA_real_,
    conditionalPowerFunction = NA,
    delta1 = NA_real_,
    delta1Min = NA_real_,
    delta1Max = NA_real_,
    firstStageInformation = NA_real_,
    useInterimEstimate = TRUE,
    likelihoodRatioDistribution = "",
    deltaLR = NA_real_,
    weightsDeltaLR = NA_real_,
    tauLR = NA_real_,
    kappaLR = NA_real_,
    deltaMaxLR = NA_real_,
    levelConstant = NA_real_,
    monotonisationConstants,
    minimumSecondStageInformation = 0,
    maximumSecondStageInformation = Inf,
    minimumConditionalError = 0,
    maximumConditionalError = 1,
    levelConstantMinimum = 0,
    levelConstantMaximum = 10,
    ncp1 = NA_real_,
    ncp1Min = NA_real_,
    ncp1Max = NA_real_,
    enforceMonotonicity = TRUE
    ) {
      # Range checks for numeric variables
      # General range checks
      .rangeCheck(variable = alpha, range = c(0, 1), allowedEqual = FALSE)
      .rangeCheck(variable = alpha1, range = c(0, 1), allowedEqual = TRUE)
      .rangeCheck(variable = alpha0, range = c(0, 1), allowedEqual = TRUE)

      # Context-related range checks
      .rangeCheck(variable = alpha1, range = c(0, alpha), allowedEqual = TRUE)
      .rangeCheck(variable = alpha0, range = c(alpha1, 1), allowedEqual = TRUE)

      if (
        is.na(conditionalPower) &&
        is.null(suppressWarnings(body(conditionalPowerFunction)))
      ) {
        stop(
          "Must specify either conditionalPower or a valid conditionalPowerFunction."
        )
      } else {
        if (!is.na(conditionalPower)) {
          .rangeCheck(
            variable = conditionalPower,
            range = c(0, 1),
            allowedEqual = FALSE
          )

          if (!is.null(suppressWarnings(body(conditionalPowerFunction)))) {
            warning(
              "Both conditionalPower and conditionalPowerFunction are provided. Using conditionalPower and ignoring conditionalPowerFunction."
            )
          }
        } else if (!is.null(suppressWarnings(body(conditionalPowerFunction)))) {
          # Check if function is increasing
          # Grid of values
          pValueGrid <- seq(from = alpha1, to = alpha0, length.out = 50)
          condPowerValues <- conditionalPowerFunction(pValueGrid)

          # Any function value larger than previous?
          if (
            any(
              condPowerValues[2:(length(condPowerValues))] >
              condPowerValues[1:(length(condPowerValues) - 1)]
            )
          ) {
            warning(
              "Conditional power function should not be increasing in the first-stage p-value."
            )
          }

          if (
            useInterimEstimate &&
            (minimumSecondStageInformation > 0 ||
             maximumSecondStageInformation < Inf)
          ) {
            warning(
              "Use of conditional power function, interim estimate and information constraints may lead to non-monotone conditional error function."
            )
          }

          .self$conditionalPowerFunction <- conditionalPowerFunction
        }
      }

      .rangeCheck(
        variable = firstStageInformation,
        range = c(0, Inf),
        allowedEqual = FALSE
      )

      # Set initial parameters
      .self$alpha <- alpha
      .self$alpha1 <- alpha1
      .self$alpha0 <- alpha0
      .self$conditionalPower <- conditionalPower
      .self$firstStageInformation <- firstStageInformation
      .self$likelihoodRatioDistribution <- likelihoodRatioDistribution
      .self$useInterimEstimate <- useInterimEstimate

      if (levelConstantMinimum >= levelConstantMaximum) {
        stop("levelConstantMinimum must be smaller than levelConstantMaximum.")
      }

      .self$levelConstantMinimum <- levelConstantMinimum
      .self$levelConstantMaximum <- levelConstantMaximum

      # Derive effect sizes for conditional power
      # When using an interim estimate, derive minimal or maximal effects
      if (useInterimEstimate) {
        # Neither lower limit provided -> error
        if ((is.na(ncp1Min) && (is.na(delta1Min)))) {
          stop(
            "Must provide a lower limit for the interim estimate by using ncp1Min or delta1Min."
          )
        } else if (!is.na(delta1Min)) {
          .rangeCheck(
            variable = delta1Min,
            range = c(0, Inf),
            allowedEqual = FALSE
          )
          .self$delta1Min <- delta1Min
          .self$delta1Max <- delta1Max

          if (!is.na(ncp1Min)) {
            warning(
              "Both ncp1Min and delta1Min are provided. Using delta1Min and ignoring ncp1Min."
            )
          }

          .self$ncp1Min <- delta1Min * sqrt(firstStageInformation)
          .self$ncp1Max <- delta1Max * sqrt(firstStageInformation)
        } else if (!is.na(ncp1Min)) {
          .rangeCheck(
            variable = ncp1Min,
            range = c(0, Inf),
            allowedEqual = FALSE
          )
          .self$ncp1Min <- ncp1Min
          .self$ncp1Max <- ncp1Max

          .self$delta1Min <- ncp1Min / sqrt(firstStageInformation)
          .self$delta1Max <- ifelse(
            ncp1Max == Inf,
            Inf,
            ncp1Max / sqrt(firstStageInformation)
          )
        } else {
          stop(
            "Unexpected error occured during determination of restrictions for interim estimate."
          )
        }
      } else {
        # When not using an interim estimate, derive fixed effects
        # If non-centrality parameter was not specified, calculate it from delta1
        if (!is.na(delta1)) {
          .rangeCheck(
            variable = delta1,
            range = c(0, Inf),
            allowedEqual = FALSE
          )
          .self$delta1 <- delta1

          if (!is.na(ncp1)) {
            warning(
              "Both delta1 and ncp1 are provided. Using delta1 and ignoring ncp1."
            )
          }
          .self$ncp1 <- delta1 * sqrt(firstStageInformation)
        } else if (!is.na(ncp1)) {
          # If delta1 was not specified, calculate it from ncp1
          .rangeCheck(variable = ncp1, range = c(0, Inf), allowedEqual = FALSE)
          .self$ncp1 <- ncp1
          .self$delta1 <- ncp1 / sqrt(firstStageInformation)
        } else {
          # Else, none of ncp1 and delta1 were specified
          stop(
            "Must specify delta1 or ncp1 when using a fixed effect for conditional power."
          )
        }
      }

      # Range checks for constraints
      # General range checks

      .rangeCheck(
        variable = minimumConditionalError,
        range = c(0, 1),
        allowedEqual = TRUE
      )
      .rangeCheck(
        variable = maximumConditionalError,
        range = c(0, 1),
        allowedEqual = TRUE
      )
      .rangeCheck(
        variable = minimumSecondStageInformation,
        range = c(0, Inf),
        allowedEqual = TRUE
      )
      .rangeCheck(
        variable = maximumSecondStageInformation,
        range = c(0, Inf),
        allowedEqual = TRUE
      )

      if (maximumSecondStageInformation == 0) {
        stop("Maximum second-stage information must be larger than 0.")
      }

      # Context-related range checks
      .rangeCheck(
        variable = minimumConditionalError,
        range = c(0, maximumConditionalError),
        allowedEqual = TRUE,
        hint = "It must not exceed maximumConditionalError."
      )

      .rangeCheck(
        variable = maximumConditionalError,
        range = c(minimumConditionalError, 1),
        allowedEqual = TRUE,
        hint = "It must not be smaller than minimumConditionalError."
      )

      .rangeCheck(
        variable = minimumSecondStageInformation,
        range = c(0, maximumSecondStageInformation),
        allowedEqual = TRUE,
        hint = "It must not exceed maximumSecondStageInformation."
      )
      .rangeCheck(
        variable = maximumSecondStageInformation,
        range = c(minimumSecondStageInformation, Inf),
        allowedEqual = TRUE,
        hint = "It must not be smaller than minimumSecondStageInformation."
      )

      # Check if the maximumConditionalError exceeds the conditionalPower
      if (!is.na(conditionalPower)){
        if (maximumConditionalError > conditionalPower && 1 > maximumConditionalError){
          message("Note that the restriction of not exceeding the conditionalPower is stricter than the specified maximumConditionalError.")
        }
      }

      # Check if the maximumConditionalError exceeds the conditionalPowerFunction
      if (!is.null(suppressWarnings(body(conditionalPowerFunction)))){
        if (!is.na(maximumConditionalError) && maximumConditionalError > conditionalPowerFunction(alpha0) && 1 > maximumConditionalError){
          message("Note that there exist first stage p-values for which the restriction of not exceeding the conditionalPowerFunction is stricter than the specified maximumConditionalError.")
        }
      }

      # Identify constraints for minimum conditional error / maximum second-stage information
      .self$minimumConditionalError <- minimumConditionalError
      .self$maximumSecondStageInformation <- maximumSecondStageInformation

      # Identify constraints for maximum conditional error / minimum second-stage information
      .self$maximumConditionalError <- maximumConditionalError
      .self$minimumSecondStageInformation <- minimumSecondStageInformation

      # Check if the constraints are too strict
      getConstraintC_max_int <- Vectorize(getConstraintC_max, vectorize.args = "firstStagePValue")
      getConstraintC_min_int <- Vectorize(getConstraintC_min, vectorize.args = "firstStagePValue")

      if(alpha1 + integrate(f = function(firstStagePValue) {getConstraintC_max_int(firstStagePValue, design = .self)},
                            lower = alpha1, upper = alpha0)$value < alpha){
        stop("The upper constraint (maximumConditionalError and/or minimumSecondStageInformation) is too strict.")
      }

      if(alpha1 + integrate(f = function(firstStagePValue) {getConstraintC_min_int(firstStagePValue, design = .self)},
                         lower = alpha1, upper = alpha0)$value > alpha){
        stop("The lower constraint (minimumConditionalError and/or maximumSecondStageInformation) is too strict.")
      }

      .self$enforceMonotonicity <- enforceMonotonicity

      # Identify specific distribution parameters
      if (likelihoodRatioDistribution == "fixed") {
        if (any(is.na(deltaLR))) {
          stop("Must provide deltaLR for fixed effect in likelihood ratio.")
        } else {
          .self$deltaLR <- deltaLR
          # If any of the weights are NA, use equal weights
          if (any(is.na(weightsDeltaLR))) {
            .self$weightsDeltaLR <- rep(1 / length(deltaLR), length(deltaLR))
            # For multiple effects, tell the user that equal weights are used.
            if (length(deltaLR) > 1) {
              message(
                "At least one entry in weightsDeltaLR is NA. Using equal weights for effects in fixed likelihood ratio."
              )
            }
          } else {
            .rangeCheck(
              variable = weightsDeltaLR,
              range = c(0, 1),
              allowedEqual = TRUE
            )
            # Check if weightsDeltaLR and deltaLR are of equal length
            if (length(weightsDeltaLR) != length(deltaLR)) {
              stop(
                "Must provide exactly one weight in weightsDeltaLR per entry of deltaLR."
              )
            }
            # Verify that weightsDeltaLR sums to 1
            if (sum(weightsDeltaLR) != 1) {
              stop("Weights in weightsDeltaLR must sum to 1.")
            }
            .self$weightsDeltaLR <- weightsDeltaLR
          }
        }
      } else if (likelihoodRatioDistribution == "normal") {
        if (is.na(deltaLR) || is.na(tauLR)) {
          stop(
            "Must provide deltaLR and tauLR for normal prior in likelihood ratio."
          )
        } else {
          .rangeCheck(variable = tauLR, range = c(0, Inf), allowedEqual = FALSE)

          .self$deltaLR <- deltaLR
          .self$tauLR <- tauLR
        }
      } else if (likelihoodRatioDistribution == "exp") {
        if (is.na(kappaLR)) {
          stop(
            "Must provide kappaLR for exponential prior in likelihood ratio."
          )
        } else {
          .rangeCheck(
            variable = kappaLR,
            range = c(0, Inf),
            allowedEqual = FALSE
          )
          .self$kappaLR <- kappaLR
        }
      } else if (likelihoodRatioDistribution == "unif") {
        if (is.na(deltaMaxLR)) {
          stop("Must provide deltaMaxLR for uniform prior in likelihood ratio.")
        } else {
          .rangeCheck(
            variable = deltaMaxLR,
            range = c(0, Inf),
            allowedEqual = FALSE
          )
          .self$deltaMaxLR <- deltaMaxLR
        }
      } else if (likelihoodRatioDistribution == "maxlr") {} else {
        stop(
          "Distribution not matched. likelihoodRatioDistribution should be one of 'fixed', 'normal', 'exp', 'unif' or 'maxlr'."
        )
      }

      # Produce warning for specific case in which the interim estimate is unrestricted (delta1Max=Inf and alpha1=0),
      # but a minimum constraint on the second-stage information, which may not be achievable, is specified.
      if (
        useInterimEstimate &&
        alpha1 == 0 &&
        delta1Max == Inf &&
        minimumSecondStageInformation > 0
      ) {
        warning(
          "When using an interim estimate with no upper restriction (delta1Max = Inf) and alpha1=0, the second-stage information may become arbitrarily small.
          Lower constraints on the second-stage information may lead to issues when determining the level constant in this case."
        )
      }

      # Calculate monotonisation constants
      .self$monotonisationConstants <- getMonotonisationConstants(
        fun = "getQ",
        lower = alpha1,
        upper = alpha0,
        argument = "firstStagePValue",
        design = .self
      )

      # Calculate level constant
      .self$levelConstant <- getLevelConstant(
        design = .self
      )$root
    },
    show = function() {
      print.TrialDesignOptimalConditionalError(.self)
    }
  )
)

#' @name SimulationResultsOptimalConditionalError
#' @title Simulation results for optimal conditional error design
#' @description
#' A class for simulation results of the optimal conditional error function.
#'
SimulationResultsOptimalConditionalError <- setRefClass(
  Class = "SimulationResultsOptimalConditionalError",
  fields = list(
    alternative = "numeric",
    firstStageFutility = "numeric",
    firstStageEfficacy = "numeric",
    overallPower = "numeric",
    maxNumberOfIterations = "numeric"
  ),
  methods = list(
    show = function() {
      print.SimulationResultsOptimalConditionalError(.self)
    }
  )
)

#' @name PowerResultsOptimalConditionalError
#' @title Power results for optimal conditional error design
#' @description
#' A class for power results of the optimal conditional error function.
#'
PowerResultsOptimalConditionalError <- setRefClass(
  Class = "PowerResultsOptimalConditionalError",
  fields = list(
    alternative = "numeric",
    firstStageFutility = "numeric",
    firstStageEfficacy = "numeric",
    overallPower = "numeric"
  ),
  methods = list(
    show = function() {
      print.PowerResultsOptimalConditionalError(.self)
    }
  )
)

---
title: "Introduction to the 'optconerrf' package"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{introduction}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.dim = c(9, 6)
)
options(rmarkdown.html_vignette.check_title = FALSE)
```

```{r setup}
library(optconerrf)
```

# Purpose of the package

The R package `optconerrf` can be used to derive the optimal conditional error function proposed by Brannath & Bauer (2004), which minimises the expected second-stage sample size of a two-stage adaptive clinical trial design while controlling for the conditional power.
The package implements a more general approach by using the second-stage *information* rather than the second-stage *sample-size* and introduces extensions to the original methodology.
In case the specification of the optimal conditional error function leads to a non-monotone function, the package can be used to implement the monotone transformation detailed in Brannath et al. (2024).

This document provides a brief introduction to the central functions of the package.
These are:

* `getDesignOptimalConditionalErrorFunction()` to create a design which facilitates the use of the remaining functions 
* `getOptimalConditionalErrorFunction()` to calculate the optimal conditional error for a given first-stage p-value
* `getSecondStageInformation()` to calculate the second-stage information required to reach the target conditional power for a given first-stage p-value
* `getExpectedSecondStageInformation()` to calculate the expected second-stage information for a specification of the optimal conditional error function under an effect assumption

# Prerequisites to computing the optimal conditional error function
\label{section_ocef}

In `optconerrf`, the tested set of hypotheses is always assumed to be:

$$
H_0: \Delta \leq 0 \enspace \text{vs.} \enspace H_1: \Delta>0,
$$
where $\Delta$ is the effect size of interest.

The optimal conditional error function is defined as:

$$
\alpha_2(p_1) = \psi \left( \frac{-e^{c_0}}{Q(p_1)}\right).
$$

The individual parameters and functions are the following:

* $p_1$ is the first-stage p-value.
* $\psi$ is the inverse of the function $\nu'(u) = -2\cdot(\Phi^{-1}(1-u)+\Phi^{-1}(CP))/\phi(\Phi^{-1}(1-u))$. For $\psi$, no closed form is available and its solution is found through the `uniroot()` function in `optconerrf`.
* $c_0$ is the level constant, required for the optimal conditional error function to preserve the overall type I error rate.
* $Q(p_1)=l(p_1)/\Delta_1^2$ is the ratio of the likelihood ratio $l(p_1)$ of $p_1$ and an effect assumption $\Delta_1$ at which the conditional power $CP$ should be reached. The likelihood ratio requires an assumption about the true treatment effect, denoted $\vartheta$. This is the effect assumption under which the second-stage information is minimised.

Note that $\vartheta$ denotes a non-centrality parameter rather than the corresponding mean difference $\Delta$.
The non-centrality parameter $\vartheta$ is proportional to $\Delta$ and additionally takes into account the first-stage information $I_1$ of the trial.
It is calculated as:

$$
\vartheta = \Delta\cdot \sqrt{I_1}.
$$

In `optconerrf`, the treatment effect for the conditional power can be specified either as the mean difference `delta1` or the non-centrality parameter `ncp1`, corresponding to $\vartheta_1 = \Delta_1 \cdot \sqrt{I_1}$.

The `optconerrf` package is based on a design object which contains the required specifications of the optimal conditional error functions.
This object is created by the function `getDesignOptimalConditionalErrorFunction()` and can be simply passed to other functions instead of specifying each argument separately.
The following section explains how to derive the design object.

# Creating the design object

For the creation of a design object, the following parameters are required and discussed in more detail below:

* Overall type I error rate and early stopping boundaries
* Effect for likelihood ratio
* Conditional power
* Effect for conditional power
* First-stage information
* Optional: Constraints providing minimum and maximum conditional error (or maximum and minimum second-stage information, respectively)

In addition to these arguments, which must be specified by the user, the following fields are automatically calculated and added to the design object:

* Level constant
* Monotonisation constants to ensure a non-increasing conditional error function (if necessary and not explicitly inhibited by the user)

## Overall type I error rate and early stopping boundaries

The overall type I error rate of the design, denoted $\alpha$, is specified to `getDesignOptimalConditionalErrorFunction()` by the argument `alpha`.
Early stopping boundaries $\alpha_1$ and $\alpha_0$ are specified via the respective arguments `alpha1` and `alpha0`, with the stopping rules:

* Stop and reject $H_0$ after the first stage if $p_1 \leq \alpha_1$.
* Stop and fail to reject $H_0$ after the first stage if $p_1 > \alpha_0$.

Note that $\alpha_0$ is a binding futility boundary, i.e., it is not permitted to continue the trial in case $p_1>\alpha_0$.
Designs with $\alpha_1 = 0$ and/or $\alpha_0 = 1$, i.e., designs with no early efficacy and/or futility stopping, are permitted.

## Effect for likelihood ratio

The likelihood ratio of the first-stage p-value $p_1$ for a normally distributed test statistic with true non-centrality parameter $\vartheta=\Delta \cdot \sqrt{I_1}$ according to Hung et al. (1997) is calculated as:

$$
l(p_1) = e^{\Phi^{-1}(1-p_1)\cdot \vartheta -\frac{\vartheta^2}{2}}.
$$

To use a fixed effect for the likelihood ratio in the specification of the optimal conditional error function, `likelihoodRatioDistribution="fixed"` must be specified to `getDesignOptimalConditionalErrorFunction()`. In addition, the effect size $\Delta$ must be provided on the **mean difference scale** via the argument `deltaLR.`
How to specify the first-stage information $I_1$ is elaborated in more detail below.

Selecting a single value for `deltaLR` (and consequently, $\vartheta$) may however provide an inefficient design under misspecification and instead, one may wish to use weighted alternatives for this parameter.

A discrete weighting of the parameter $\vartheta$ for the likelihood ratio is achieved by specifying multiple values for $\Delta$ via `deltaLR`.
The respective weights can be provided by `weightsDeltaLR` and are assumed to be equal if not explicitly specified.
An exemplary specification could be `deltaLR = c(0, 0.5, 1)` and `weightsDeltaLR = c(0.25, 0.25, 0.5)`.
Note that `likelihoodRatioDistribution = "fixed"` must still be specified in this case.

The weighting of values for $\vartheta$ may also be done via a distributional assumption, i.e., a prior density $u(\vartheta)$.
The general form of the likelihood ratio is then:

$$
l(p_1) = \int_{-\infty}^{\infty}e^{\Phi^{-1}(1-p_1)\vartheta - \vartheta ^2/2}u(\vartheta)d\vartheta.
$$

Currently, a normal, exponential and uniform prior density $u(\vartheta)$ for $\vartheta$ are implemented.

A normal prior is specified via `likelihoodRatioDistribution = "normal"` and the mean `deltaLR` and standard deviation `tauLR`.
The use of an exponential prior is possible by `likelihoodRatioDistribution = "exp"` and requires the specification of its mean `kappaLR` (corresponding to the inverse of the rate parameter $\lambda$).
A uniform prior can be used with `likelihoodRatioDistribution = "unif"` and `deltaMaxLR` specifies the maximum of its support [0, `deltaMaxLR`]. 
It is also possible to use the data-driven maximum likelihood via `likelihoodRatioDistribution = "maxlr"`.
In this approach, the effect size $\vartheta$ is estimated from the data and used in the likelihood ratio calculation, provided it is not smaller than 0.
In this case, no additional parameters must be specified.

The arguments `deltaLR`, `tauLR`, `kappaLR` and `deltaMaxLR` must be specified on the mean difference scale, i.e., without accounting for the first-stage information.

## Conditional power

The conditional power to be achieved at a certain planning alternative is provided to `getDesignOptimalConditionalErrorFunction()` by the argument `conditionalPower`.
Instead of a fixed value, a user-defined conditional power function may also be provided by the argument `conditionalPowerFunction`.
This function should be specified in a way that it calculates the target conditional power from the first-stage p-value.
Internally, the conditional power function is provided with the first-stage p-value as its unnamed first and only argument.
Furthermore, it should generally be non-increasing in the first-stage p-value.
This is checked when initialising the design object (by using a grid of values) and a warning is displayed if the conditional power function is increasing.
For this purpose, the conditional power function should be able to handle vectors in its argument.

## Effect for conditional power

The optimal conditional error function controls for the conditional power of a design at a fixed point alternative or at an interim estimate of the treatment effect.
To use a fixed point alternative, `useInterimEstimate=FALSE` must be specified as well as a value for the point alternative via `delta1` (mean difference scale) or `ncp1` (non-centrality parameter scale).
Apart from a fixed effect, the treatment effect at which the conditional power should be achieved can also be estimated from the first-stage data.
In this case, it is necessary that a lower limit for this interim estimate is specified, as treatment effects $\Delta_1 \leq 0$ (respectively, $\vartheta_1 \leq 0$) belong to the null hypothesis and cannot be powered for. If `useInterimEstimate=TRUE` is specified, this lower limit must be provided via `delta1Min` or `ncp1Min`.
Furthermore, one may wish to deselect overly large interim treatment effect estimates by imposing an upper limit on the estimator.
This can be done by using the arguments `delta1Max` or `ncp1Max`, but is not required.

## First-stage information

The first-stage information of the trial design must be specified to allow for calculations between the mean difference and non-centrality parameter scale.
It is provided to the design object via `firstStageInformation`.

Listed below are some examples for the calculation between information ($I_1$) and sample size:

One-sample z-test with $n$ total patients: $I_1 = \frac{n}{\sigma^2}$, where $\sigma^2$ is the variance of an individual observation

Balanced two-sample z-test with $n_1$ patients per group: $I_1 = \frac{1}{2}\cdot\frac{n_1}{\sigma^2}$, where $\sigma^2$ is the common variance

General two-sample z-test with $n_1$, $n_2$ patients per group: $I_1 = 1/(\frac{\sigma_1^2}{n_1}+\frac{\sigma_2^2}{n_2})$, where $\sigma_1^2$, $\sigma_2^2$ are the group-wise variances 

## Optional: Minimum and maximum conditional error

In certain scenarios, it may be desirable to restrict the values that the optimal conditional error function can take by a minimum and/or maximum.
For the function `getDesignOptimalConditionalErrorFunction()`, this can be achieved by specifying values for the arguments `minimumConditionalError` and `maximumConditionalError`.
The default values for the arguments are `minimumConditionalError = 0` and `maximumConditionalError = 1`, corresponding to no restrictions.
Alternatively, the second-stage information may be restricted by using the arguments `minimumSecondStageInformation` and `maximumSecondStageInformation`.
If constraints are specified on both scales, the more restrictive constraint is applied.

## Calculating the level constant

The level constant, required for the optimal conditional error function to meet the level condition, is automatically calculated whenever a design is created using `getDesignOptimalConditionalErrorFunction()`.
The level constant ensures that the optimal conditional error function meets the overall type I error rate of the design and requires that:
$$
\alpha = \alpha_1 + \int_{\alpha_1}^{\alpha_0}\alpha_2(p_1)dp_1.
$$

This is done via the function `getLevelConstant()`, which extracts the required parameters from the specified design object.
This function uses the `uniroot()` function to search for the level constant, by default on an interval from 0 to 10.
In certain cases, in particular for design specifications with very large or very small non-centrality parameters, the level constant may lie outside of this interval.
For such designs, `uniroot()` produces an error. The search interval can be manually changed by tempering with the arguments `levelConstantMinimum` and `levelConstantMaximum`.
Typically, designs with very large non-centrality parameters will require a smaller level constant (i.e., a reduction of `levelConstantMinimum`) and designs with very small non-centrality parameters will require a larger level constant (i.e., an increase of `levelConstantMaximum`).

Note that imposing inappropriate restrictions on the optimal conditional error function via `minimumConditionalError` and `maximumConditionalError` or `maximumSecondStageInformation` and `minimumSecondStageInformation` may make it impossible to calculate a level constant which fulfills the level condition.

## Calculating monotonisation constants

The use of a conditional error function which is increasing for any first-stage p-values is discouraged due to a type I error inflation in the case of a conservatively distributed $p_1$ and because the resulting power function of the trial may be decreasing in the true effect size (Koyama et al., 2005).
Because the framework of the optimal conditional error function may however lead to an increasing conditional error function for certain specifications, a non-increasing transformation may be required.
To retain optimality, the non-monotone optimal conditional error function must be set to constant values on certain intervals.
How to identify the constants and intervals is described in detail in Brannath et al. (2024).

Whenever a design object is created using `getDesignOptimalCondtionalErrorFunction()`, the function `getMonotonisationConstants()` is internally called to identify the intervals and constants.
If the specified design produces an optimal conditional error function which already is non-increasing in the first-stage p-value, no monotonisation is necessary.
For illustration purposes, `optconerrf` also allows the user to skip the monotonisation process by setting `enforceMonotonicity=FALSE`. 
In this case, the resulting conditional error function may be increasing for some p-values. 
Note that generally, such a design should not be used for practical purposes.

# Using the design object

As an example, we consider an adaptive design with overall type I error rate `alpha=0.025`, early rejection and futility boundaries `alpha1=0.0154` and `alpha0=0.5` with a target conditional power `conditionalPower=0.9` at a fixed alternative `delta1=0.25` (note that `useInterimEstimate=FALSE`).
We assume a fixed effect of `deltaLR=0.25` for the calculation of the likelihood ratio (`likelihoodRatioDistribution="fixed"`) and `firstStageInformation=50`.
Note that this design approximates the original trial design used in Brannath & Bauer (2004) and achieves an overall power of 90%.

```{r, message = FALSE, results = "markup"}
design <- getDesignOptimalConditionalErrorFunction(
  alpha = 0.025, alpha1 = 0.0154, alpha0 = 0.5, conditionalPower = 0.9,
  delta1 = 0.25, likelihoodRatioDistribution = "fixed", deltaLR = 0.25,
  firstStageInformation = 50, useInterimEstimate = FALSE
)
```

Once the design object is created using `getDesignOptimalCondtionalErrorFunction()`, it can be easily passed to other functions of `optconerrf`.
The design object is an instance of the class `TrialDesignOptimalConditionalError` and offers implementations of the generic functions `print()`, `plot()` and `summary()`.
The `print()` function provides a comprehensive overview of user-specified parameters (such as early stopping boundaries and effect sizes) as well as internally calculated variables (such as the level constant):

```{r, message = FALSE, results = "markup"}
# Comprehensive design output
print(design)
```

The `plot()` function has 4 different modes of operation, which are specified via the argument `type`:
Passing `design` and `type = 1` to the `plot()` function produces a plot visualising the optimal conditional error function resulting from the specification in `design` in relation to the first-stage p-value.
As `type = 1` is also the default value, the argument can be omitted to achieve this function behaviour:

```{r, fig.dim = c(6,4)}
plot(design) # Plot of the optimal conditional error function
```

Using `type = 2` produces a plot of the second-stage information resulting from the design specification in relation to the first-stage p-value.
The second-stage information $I_2(p_1)$ of a first-stage p-value $p_1$ is calculated as:

$$
I_2(p_1) = \frac{\bigl( \Phi^{-1}(1-\alpha_2(p_1)) + \Phi^{-1}(CP)  \bigr)^2}{\Delta_1^2}.
$$

```{r, fig.dim = c(6,4)}
plot(design, type = 2) # Plot of the second-stage information
```

The remaining plot types can be used to visualise the likelihood ratio $l(p_1)$ (`type = 3`) and the function $Q(p_1)$ (`type = 4`) resulting from the design specification and will not be shown here.

To access individual fields of the object, e.g., the overall type I error rate (`alpha`) or the level constant (`levelConstant`) the following code can be used:

```{r} 
design$alpha
design$levelConstant
print(design$levelConstant, digits = 12) # For more precision
```

# Computing the optimal conditional error function and second-stage information

Values for the optimal conditional error function are calculated using the function `getOptimalConditionalError()`.
For this function, the p-value of the first stage (argument `firstStagePValue`) and a design object, created using `getDesignOptimalConditionalErrorFunction()`, must be provided.

For the specification above, the optimal conditional error for an exemplary first-stage p-value of `0.1` can be calculated as:

```{r}
getOptimalConditionalError(
  firstStagePValue = 0.1, design = design
)
```

Multiple first-stage p-values may also be provided as a vector.
For first-stage p-values with $p_1 \leq \alpha_1$ or $p_1 > \alpha_0$, a conditional error of 1 or 0 is returned, respectively.

```{r}
getOptimalConditionalError(
  firstStagePValue = c(0.0005, 0.1, 0.05, 0.5, 0.8),
  design = design
)
```

The calculation of the second-stage information is achieved in a similar fashion, using `getSecondStageInformation()`:

```{r}
getSecondStageInformation(
  firstStagePValue = 0.1, design = design
)

getSecondStageInformation(
  firstStagePValue = c(0.0005, 0.1, 0.05, 0.5, 0.8),
  design = design
)
```

Note that the second-stage information for first-stage p-values with $p_1 \leq \alpha_1$ or $p_1 > \alpha_0$ is always 0, as the trial is stopped early for such results.

# Calculation of expected second-stage information
\label{section_exp}

With the function `getExpectedSecondStageInformation()`, a calculation of the expected second-stage information that a specification of the optimal conditional error function yields can be calculated.
The calculation of the expected second-stage information $\mathbb{E}(I_2)$ is performed as:

$$
\mathbb{E}(I_2) = \int_{\alpha_1}^{\alpha_0} \frac{ (\Phi^{-1}(1-\alpha_2(p_1)) + \Phi^{-1}(CP_{\Delta_1}))^2\cdot l(p_1)}{\Delta_1^2}dp_1.
$$
The effect $\Delta_1$ provides the effect for which the conditional power $CP_{\Delta_1}$ is targeted.
$\Delta_1$ is extracted from the design object and may depend on the interim data.

The likelihood ratio $l(p_1)$ in this case represents the scenario under which the performance of a specific design is to be evaluated.
For example, one may have specified a design assuming $\Delta=0.25$ as a fixed effect for the likelihood ratio (as above), but now wishes to investigate the performance of this design in case $\Delta=0$, i.e., if the null hypothesis is true.
This can be achieved by the following code:

```{r}
getExpectedSecondStageInformation(
  design = design, likelihoodRatioDistribution = "fixed", deltaLR = 0
)
```

The specification of the likelihood ratio under which to calculate the expected second-stage information is given by `likelihoodRatioDistribution = "fixed"` and the corresponding treatment effect is specified by `deltaLR = 0`.

Alternatively, one may calculate the expected second-stage information of the design if it was correctly specified, i.e., for $\Delta=0.25$:

```{r}
getExpectedSecondStageInformation(
  design = design, likelihoodRatioDistribution = "fixed", deltaLR = 0.25
)
```

A shortcut for the calculation of the expected second-stage information under correct specification is also available by omitting the arguments `likelihoodRatioDistribution` and `deltaLR`, in which case the likelihood ratio specification given in the design is extracted and used for the calculation.

```{r}
getExpectedSecondStageInformation(
  design = design
)
```

Multiplying the respective information values by a factor of 2 (approximately) yields the original results of Brannath & Bauer (2004).

In addition to calculating the expected second-stage information for a single fixed effect size (by specifying `likelihoodRatioDistribution = "fixed"` and a single value for `deltaLR`), it is also possible to provide multiple weighted fixed effect sizes (by specifying `likelihoodRatioDistribution = "fixed"`, multiple values for `deltaLR` and possibly respective weights with `weightsDeltaLR`) or to use a distributional assumption for the effect size in the likelihood ratio (by specifying one of `"normal"`, `"exp"` or `"unif"` for `likelihoodRatioDistribution` and the respective parameters).

# Complementary functions

In addition to the core functions described above, the `optconerrf` package provides additional functions which relate to the operating characteristics of the design.
An exact calculation of the overall power of a given design can be achieved by using the function `getOverallPower()`:

```{r}
getOverallPower(
  design, alternative = c(0, 0.25))
```

The argument `alternative` may consist of multiple elements and provides the effect sizes for which the overall power should be calculated.

In addition to an exact calculation via `getOverallPower()`, the same operating characteristics may be simulated using the function `getSimulationResults()`.

The `summary()` function is also implemented for objects of class `TrialDesignOptimalConditionalError` and combines selected results from `getExpectedSecondStageInformation()` and `getOverallPower()` to provide a general overview of the design performance:

```{r}
summary(design)
```


# References

Brannath, W. & Bauer, P. (2004). Optimal conditional error functions for the control of conditional power. Biometrics, 60 (3), 715–723. https://www.jstor.org/stable/3695393

Brannath, W., Dreher, M., zur Verth, J., Scharpenberg, M. (2024). Optimal monotone conditional error functions. https://arxiv.org/abs/2402.00814

Hung, H. M. J., O’Neill, R. T., Bauer, P. & Kohne, K. (1997). The behavior of the p-value when the alternative hypothesis is true. Biometrics. https://www.jstor.org/stable/2533093

---
title: "Constraints, Interim Estimates and Conditional Power Functions for the Optimal Conditional Error Function"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Using Constraints and Interim Estimates for the Optimal Conditional Error Function}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.dim = c(6,4)
)
```

```{r setup}
library(optconerrf)
```

# Summary

This vignette provides examples on some slightly more advanced methods which can be applied in the framework of the optimal conditional error function.
Firstly, the use of constraints for the optimal conditional error function is illustrated and afterwards, the use of an interim estimate for the effect at which the conditional power should be achieved as well as the implementation of a data-dependent conditional power function are discussed.

# Constraints

In the framework of two-stage adaptive designs, it may not be feasible to allow the second-stage information to fall below a certain minimum $I_{2,\min}$, e.g., if a certain amount of data on safety or secondary endpoints are required.
Conversely, budgeting considerations may impose an upper limit $I_{2, \max}$ on the information that can be collected in the second-stage.
Instead of ignoring these constraints in the planning phase and applying them "post-hoc", they can be accounted for in the minimisation procedure of the optimal conditional error function.
For illustration purposes, we use a trial design with overall type I error rate $\alpha=0.025$, early decision boundaries $\alpha_1 = 0.000158$ and $\alpha_0 = 0.5$ with conditional power $CP=0.9$ at the fixed alternative $\Delta_1 = 0.25$.
After collecting a first-stage information of $I_1=85$, the information of the second-stage of the trial should lie between $I_{2, \min} = 85/2$ and $I_{2, \max} = 85\cdot 2$.
To implement this design using the optimal conditional error function with the maximum likelihood ratio approach, the following code is required:

```{r}
trialDesignConstraints <- getDesignOptimalConditionalErrorFunction(
  alpha = 0.025,
  alpha1 = 0.000158,
  alpha0 = 0.5,
  conditionalPower = 0.9,
  delta1 = 0.25,
  useInterimEstimate = FALSE,
  likelihoodRatioDistribution = "maxlr",
  firstStageInformation = 85,
  minimumSecondStageInformation = 85/2,
  maximumSecondStageInformation = 85*2
)
```

The constraints are supplied by the arguments `minimumSecondStageInformation` and `maximumSecondStageInformation`.
Plotting the second-stage information of this design highlights the application of the constraints (represented by the gray dashed lines):

```{r, fig.dim = c(6,4)}
# type = 2 for second-stage information
plot(trialDesignConstraints, type = 2) +
  ggplot2::geom_hline(yintercept = c(85/2, 85*2), linetype = "dashed", colour = "gray")
```

Alternatively, the constraints may be directly specified on the conditional error function as $\alpha_{2,\min}$ and $\alpha_{2,\max}$ due to

$$
\alpha_{2,\min} = \Phi(\Phi^{-1}(CP)- \Delta_1\sqrt{I_{2,\max}})
$$
and
$$
\alpha_{2,\max} = \Phi(\Phi^{-1}(CP)- \Delta_1\sqrt{I_{2,\min}}).
$$
Note that constraining the *maximum* second-stage information corresponds to imposing a constraint on the *minimum* conditional error and that constraining the *minimum* second-stage information corresponds to imposing a constraint on the *maximum* conditional error.
Plugging in $I_{2, \max} = 85\cdot 2$ and $I_{2,\min} = 85/2$, we get $\alpha_{2,\min} \approx 0.02396$ and $\alpha_{2,\max} \approx 0.36382$ and can create the same design, now using `minimumConditionalError` and `maximumConditionalError` for the constraints:

```{r}
trialDesignConstraintsConditionalError <- getDesignOptimalConditionalErrorFunction(
  alpha = 0.025,
  alpha1 = 0.000158,
  alpha0 = 0.5,
  conditionalPower = 0.9,
  delta1 = 0.25,
  useInterimEstimate = FALSE,
  likelihoodRatioDistribution = "maxlr",
  firstStageInformation = 85,
  minimumConditionalError =  0.02396,
  maximumConditionalError = 0.36382
)
```

Comparing the expected second-stage information of the designs for $\Delta=0.25$ yields essentially the same values, with a minor difference resulting from rounding of $\alpha_{2,\min}$ and $\alpha_{2,\max}$. 

```{r}
getExpectedSecondStageInformation(
  trialDesignConstraints, 
  likelihoodRatioDistribution = "fixed", 
  deltaLR = 0.25
)
getExpectedSecondStageInformation(
  trialDesignConstraintsConditionalError, 
  likelihoodRatioDistribution = "fixed",
  deltaLR = 0.25
)
```

It is also possible to specify constraints on *both* scales simultaneously, i.e., on the second-stage information and on the conditional error function at the same time.
If this is done, the more restrictive constraint is applied. 
However, when using a fixed effect size $\Delta_1$ and constant conditional power $CP$, the scales have a one-to-one correspondence (as seen above in the formulas for $\alpha_{2,\min}$ and $\alpha_{2,\max}$) and supplying one of them is sufficient. 

Note that the specification of inappropriate constraints can make it impossible to fully exhaust (and not exceed) the $\alpha$ level, i.e., find a level constant for the design.
(As an example, consider a design with $\alpha=0.025$, $\alpha_1=0$ and $\alpha_0=1$. Since the full $\alpha$ level is to be spent in the second stage, it follows from the level condition (see `?getLevelConstant`) that $0.025 = \int_0^1 \alpha_2(p_1)dp_1$. The exemplary constraint $\alpha_{2,\max}=0.01$ is inappropriate since when applying it, $\int_0^1 \min(0.01, \alpha_2(p_1))dp_1 \leq \int_0^1 0.01 dp_1 = 0.01 \neq 0.025$ and the level condition cannot be fulfilled.)

# Interim Estimate for Treatment Effect

In certain applications, using a fixed effect $\Delta_1$ at which the conditional power (be it constant or a function, more details below) should be achieved may not be appropriate.
For example, we may wish to estimate this effect from the first-stage data if only little or unreliable evidence regarding the treatment effect is available during initial trial planning.
A simple estimate for the interim effect is given by:

$$
\hat\Delta_1(p_1) = \Phi^{-1}(1-p_1)/ \sqrt{I_1}.
$$
An issue with this estimate is the fact that it may become negative for $p_1 > 0.5$.
However, negative effects are part of the null hypothesis $H_0: \Delta \leq 0$ and cannot reasonably be powered for.
Furthermore, a clinical trial typically operates with a minimally clinically relevant effect size $\Delta_{1, \min} > 0$, which marks the lower boundary of effects considered worthwhile detecting.
For this reason, it is generally advisable to cut off interim effects that fall below $\Delta_{1,\min}$ (and instead use $\Delta_{1, \min}$).
An upper limit $\Delta_{1, \max}$ may also be put in place to deselect overly optimistic interim effects (and instead use $\Delta_{1, \max}$).
Implementing both cutoffs leads to the truncated interim estimate:

$$
\tilde\Delta_1(p_1) = \min(\max(\hat\Delta_1(p_1), \, \Delta_{1,\min}),\, \Delta_{1,\max}).
$$
When creating a design object with `getDesignOptimalConditionalErrorFunction()`, the use of an interim estimate for the treatment effect at which the conditional power should be achieved is specified by setting `useInterimEstimate = TRUE`.
When an interim estimate is used, the lower limit $\Delta_{1,\min}$ must be specified by the argument `delta1Min`.
An upper limit is not required, but can optionally be specified via `delta1Max`.
Both arguments may also be specified on the non-centrality parameter scale with $\vartheta_{1,\min} = \Delta_{1, \min}\cdot\sqrt{I_1}$ as `ncp1Min` and $\vartheta_{1,\max} = \Delta_{1, \max}\cdot\sqrt{I_1}$ as `ncp1Max`. 

As an example, consider the same trial setting as above with $\alpha=0.025$, $\alpha_1=0.000158$, $\alpha_0=0.5$, $CP=0.9$, $I_1 = 85$ and the maximum likelihood ratio.
Now, instead of using a fixed effect $\Delta_1$, we wish to estimate this effect from the data with a minimally clinically relevant effect size of $\Delta_{1, \min}=0.15$.
Additionally, we wish to deselect interim estimate effects above $\Delta_{1,\max} = 0.35$.

The design object is then created as:

```{r}
trialDesignInterimEstimate <- 
  getDesignOptimalConditionalErrorFunction(
    alpha = 0.025,
    alpha1 = 0.000158,
    alpha0 = 0.5,
    conditionalPower = 0.9,
    useInterimEstimate = TRUE,
    delta1Min = 0.15,
    delta1Max = 0.35,
    firstStageInformation = 85,
    likelihoodRatioDistribution = "maxlr"
  )
```

Plotting the optimal conditional error function and second-stage information from this design yields:

```{r}
plot(trialDesignInterimEstimate, type = 1)
plot(trialDesignInterimEstimate, type = 2)
```

Note that for a design with an early efficacy boundary $\alpha_1>0$, an upper limit for the effect is always implicitly given by $\Delta_{1,\max} = \Phi^{-1}(1-\alpha_1)/\sqrt{I_1}$, which is $\Phi^{-1}(1-0.000158)/\sqrt{85}=0.390672$ in our example.
While this always applies for the upper limit, a futility boundary $\alpha_0 < 1$ may not always correspond to a lower limit $\Delta_{1, \min}>0$ (more precisely, the implicit boundary $\Delta_{1,\min} \leq 0$ would occur if $0.5 \leq \alpha_0 < 1$).
For this reason, it is mandatory that $\Delta_{1,\min}>0$ (or, alternatively, $\vartheta_{1,\min}>0$) are explicitly specified, regardless of the futility boundary $\alpha_0$.

It is also possible to specify constraints on either or both the optimal conditional error scale or the second-stage information scale for designs implementing an interim estimate.
Rescaling the constraints, as opposed to the above case using a fixed effect size $\Delta_1$, must now take into account the interim estimate $\tilde{\Delta}_1(p_1)$ and requires the use of the constraint functions:


$$
\alpha_{2,\min}(p_1) = \Phi(\Phi^{-1}(CP)- \tilde{\Delta}_1(p_1)\sqrt{I_{2,\max}})
$$
and

$$
\alpha_{2,\max}(p_1) = \Phi(\Phi^{-1}(CP)- \tilde{\Delta}_1(p_1)\sqrt{I_{2,\min}}).
$$

Since $I_{2, \max} \geq I_{2,\min}$, it follows that $\alpha_{2,\min}(p_1) \leq \alpha_{2,\max}(p_1)$.
Note that both constraint functions are *non-decreasing* in $p_1$, since $\tilde{\Delta}_1(p_1)$ is non-increasing in $p_1$.
To ensure that the resulting conditional error function is nevertheless non-increasing, we may exploit a property of $\tilde{\Delta}_1(p_1)$:
The smallest effect size of the interim estimate is always given by $\Delta_{1,\min}$ (or may occur for $p_1 = \alpha_0$).
As the smallest effect size corresponds to the largest second-stage information, we can plug in the smallest effect in the formula for $\alpha_{2,\min}(p_1)$ and achieve a constant constraint.
If an upper limit for the effect size is provided, either via $\Delta_{1,\max}$ or at $p_1 = \alpha_1$, the same shortcut is applicable for $\alpha_{2,\max}(p_1)$.

# Using a Conditional Power Function

The conditional power describes the probability of a successful trial outcome at the final analysis (i.e., rejection of $H_0$) given the interim data and an effect assumption, which can either be pre-defined or derived from the interim data (see the section above).
Aiming to achieve the same value for the conditional power, e.g., 80%, regardless of the interim results of the trial may not be intuitive.
Instead, one may aim for a higher conditional power if the interim results are promising and settle for a lower conditional power for poor interim results.
Such an approach can be realised through the use of a conditional power function which depends on the interim data.
Generally, a small first-stage p-value may warrant a large conditional power, whereas for weaker first-stage evidence, a lower conditional power may be more appropriate.
Thus, the conditional power function should be decreasing (or at least non-increasing) in the first-stage p-value.

Importantly, the use of a (decreasing) conditional power function may lead to a non-monotone second-stage information function.
For a fixed conditional power, the second-stage information required to achieve that conditional power will always be increasing in $p_1$ (provided the conditional error function is non-increasing in $p_1$).
A decreasing conditional power function may impact the second-stage information function in a way that it increases for small first-stage p-values, reaches a maximum and decreases for larger first-stage p-values.
In `optconerrf`, the user may freely specify a conditional power function to `getDesignOptimalConditionalErrorFunction()` via the argument `conditionalPowerFunction`.
In the current implementation, the first-stage p-value is passed as the first and only argument to the conditional power function.
For illustration purposes, we consider the exemplary conditional power function $CP(p_1) = \Phi(1-p_1)$, which is decreasing in the first-stage p-value $p_1$:

```{r}
myConditionalPowerFunction <- function(firstStagePValue) {
  return(pnorm(1 - firstStagePValue))
}
```

A simple plot shows the properties of this function:

```{r}
p1 <- seq(0, 1, 0.01)
plot(p1, myConditionalPowerFunction(p1), type = "l")
abline(h = myConditionalPowerFunction(c(0, 1)), col = "blue")
```

For small first-stage p-values, the conditional power function leads to values just above 0.8 and approaches 0.5 as the first-stage p-value becomes larger.
As above, we will use $\alpha = 0.025$, $\alpha_1 = 0.000158$ and $\alpha_0 = 0.5$.
Implementing this in a design object is straightforward:

```{r}
trialDesignConditionalPowerFunction <- 
  getDesignOptimalConditionalErrorFunction(
    alpha = 0.025,
    alpha1 = 0.000158,
    alpha0 = 0.5,
    conditionalPowerFunction = myConditionalPowerFunction,
    delta1 = 0.25,
    useInterimEstimate = FALSE,
    firstStageInformation = 100,
    likelihoodRatioDistribution = "maxlr"
  )
```

Plotting the optimal conditional error function resulting from this design yields a seemingly normal function:

```{r}
plot(trialDesignConditionalPowerFunction, type = 1)
```

The merits of using a conditional power function become more apparent when plotting the second-stage information resulting from the design:

```{r}
plot(trialDesignConditionalPowerFunction, type = 2)
```

This second plot shows that the second-stage information required to achieve the conditional power (function) first increases in the first-stage p-value, reaches a maximum and then decreases as the first-stage evidence becomes weaker.
This is different to a design with a constant conditional power, for which the second-stage information function is always non-decreasing (and typically, increasing) in the first-stage p-value.
Using a decreasing conditional power function thus allows for potential savings in first-stage information in case of weak first-stage evidence.

The application of *either* constraints *or* an interim estimate for the treatment effect is possible in combination with a conditional power function, however the combination of all three methods may lead to complications further elaborated in the next section.

# Limitations

The `optconerrf` package provides support for each of the three methods detailed above and for each combination of up to two methods.
However, using all three of them simultaneously may lead to issues.

To illustrate this, presume we wish to implement a design with:

 * $\alpha = 0.025$, $\alpha_1 = 0.000158$, $\alpha_0 = 0.5$
 * maximum likelihood ratio
 * first-stage information $I_1 = 100$
 * second-stage information constraint $I_{2,\min} = 50$
 * conditional power function $CP(p_1)=\Phi(1-p_1)$
 * an interim estimate with $\Delta_{1,\min}=0.25$
 
 
```{r}
combinedDesign <- 
  getDesignOptimalConditionalErrorFunction(
    alpha = 0.025,
    alpha1 = 0.000158,
    alpha0 = 0.5,
    conditionalPowerFunction = myConditionalPowerFunction,
    delta1Min = 0.25,
    useInterimEstimate = TRUE,
    firstStageInformation = 100,
    likelihoodRatioDistribution = "maxlr",
    minimumSecondStageInformation = 50
  )
```

Plotting the optimal conditional error function resulting from this design shows a non-monotone function:

```{r}
plot(combinedDesign)
```

This comes from the fact that now, the constraint functions are more difficult to handle than before.

We see that:

$$
\alpha_{2,\min}(p_1) = \Phi(\Phi^{-1}(CP(p_1)) - \tilde{\Delta}_1(p_1)\sqrt{I_{2,\max}})
$$
and

$$
\alpha_{2,\max}(p_1) = \Phi(\Phi^{-1}(CP(p_1)) - \tilde{\Delta}_1(p_1)\sqrt{I_{2,\min}})
$$

may not be monotone in $p_1$, because both $CP(p_1)$ and $\tilde{\Delta}_1(p_1)$ are non-increasing in $p_1$.
We can no longer use the workaround of plugging in $\Delta_{1,\min}$ (respectively, $\Delta_{1, \max}$) for the interim estimate, because it is no longer guaranteed that the maximum (respectively, minimum) second-stage information is achieved for this effect size (due to the conditional power function).

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\docType{class}
\name{PowerResultsOptimalConditionalError}
\alias{PowerResultsOptimalConditionalError}
\title{Power results for optimal conditional error design}
\description{
A class for power results of the optimal conditional error function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getNu.R
\name{getNu}
\alias{getNu}
\title{Calculate Nu}
\usage{
getNu(alpha, conditionalPower)
}
\arguments{
\item{alpha}{The (conditional) type I error rate of the design. Must be a numeric vector with values between 0 and 1.}

\item{conditionalPower}{The target conditional power \eqn{CP} of the design. Must be a numeric value.}
}
\value{
Factor linking information and \eqn{\alpha_2}.
}
\description{
Calculate the factor which relates \eqn{\alpha_2} to the second-stage information for given conditional power.
}
\details{
Note that this function uses factor 1 instead of factor 2 (Brannath & Bauer 2004). This has no impact on the optimal conditional error function, as constant factors are absorbed by the level constant \eqn{c_0}. \cr
The calculation is:
\deqn{\nu(\alpha_2(p_1)) = (\Phi^{-1}(1-\alpha_2(p_1)) + \Phi^{-1}(CP))^2.}
}
\examples{
getNu(alpha = 0.05, conditionalPower = 0.9)

# Returns 0 if alpha exceeds conditionalPower
getNu(alpha = 0.8, conditionalPower = 0.7)

}
\references{
Brannath, W. & Bauer, P. (2004). Optimal conditional error functions for the control of conditional power. Biometrics. https://www.jstor.org/stable/3695393
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{summary.TrialDesignOptimalConditionalError}
\alias{summary.TrialDesignOptimalConditionalError}
\title{Summary of the optimal conditional error trial design}
\usage{
\method{summary}{TrialDesignOptimalConditionalError}(object, ...)
}
\arguments{
\item{object}{Design object of class \code{TrialDesignOptimalConditionalError}}

\item{...}{Additional arguments required for generic compatibility}
}
\value{
No return value, prints a summary of design performance.
}
\description{
Provide an overview of the operating characteristics of the optimal conditional error trial design.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getOptimalConditionalError.R
\name{getOptimalConditionalError}
\alias{getOptimalConditionalError}
\title{Calculate the Optimal Conditional Error}
\usage{
getOptimalConditionalError(firstStagePValue, design)
}
\arguments{
\item{firstStagePValue}{First-stage p-value or p-values. Must be a numeric vector between 0 and 1.}

\item{design}{An object of class \code{TrialDesignOptimalConditionalError} created by \code{getDesignOptimalConditionalErrorFunction()}. Contains all necessary arguments to calculate the optimal conditional error function for the specified case.}
}
\value{
Value of the optimal conditional error function.
}
\description{
Calculate the Optimal Conditional Error
}
\details{
The optimal conditional error \eqn{\alpha_2} given a first-stage p-value \eqn{p_1} is calculated as:
\deqn{\alpha_2(p_1)=\psi(-e^{c_0} \cdot \frac{\Delta_1^2}{l(p_1)}).}

The level constant \eqn{c_0} as well as the specification of the effect size \eqn{\Delta_1} and the likelihood ratio \eqn{l(p_1)}
must be contained in the \code{design} object (see \code{?getDesignOptimalConditionalErrorFunction}).
Early stopping rules are supported, i.e., for \eqn{p_1 \leq \alpha_1}, the returned conditional error is 1 and for \eqn{p_1 > \alpha_0}, the returned conditional error is 0.
}
\examples{
# Create a design
design <- getDesignOptimalConditionalErrorFunction(
alpha = 0.025, alpha1 = 0.001, alpha0 = 0.5, conditionalPower = 0.9,
delta1 = 0.5, firstStageInformation = 40, useInterimEstimate = FALSE,
likelihoodRatioDistribution = "fixed", deltaLR = 0.5)

# Calculate optimal conditional error
getOptimalConditionalError(
firstStagePValue = c(0.1, 0.2, 0.3), design = design
)
}
\references{
Brannath, W. & Bauer, P. (2004). Optimal conditional error functions for the control of conditional power. Biometrics. https://www.jstor.org/stable/3695393
}
\seealso{
\code{\link[=getDesignOptimalConditionalErrorFunction]{getDesignOptimalConditionalErrorFunction()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getExpectedSecondStageInformation.R
\name{getExpectedSecondStageInformation}
\alias{getExpectedSecondStageInformation}
\title{Calculate Expected Second-stage Information}
\usage{
getExpectedSecondStageInformation(
  design,
  likelihoodRatioDistribution = NULL,
  ...
)
}
\arguments{
\item{design}{An object of class \code{TrialDesignOptimalConditionalError} created by \code{getDesignOptimalConditionalErrorFunction()}. Contains all necessary arguments to calculate the optimal conditional error function for the specified case.}

\item{likelihoodRatioDistribution}{The distribution to be used for the effect size of the likelihood ratio in the calculation of the expected second-stage information. Options are \code{"fixed", "normal", "exp", "unif", "maxlr"} for fixed effect size, normally distributed, exponentially distributed, uniformly distributed prior of the effect size and maximum likelihood ratio, respectively.
Each case requires different additional specifications: \cr
\itemize{
\item \code{likelihoodRatioDistribution="fixed"} uses one (or more) fixed effect sizes for the likelihood ratio and requires the parameter \code{deltaLR} which provides the mean difference under which to calculate the likelihood ratio. If \code{deltaLR} contains multiple values, they may be weighted using an additional argument \code{weightsDeltaLR}. Omitting \code{weightsDeltaLR} automatically leads to equal weighting.
\item \code{likelihoodRatioDistribution="normal"} uses a normal prior for the effect size and requires parameters \code{deltaLR} and \code{tauLR} for the mean and standard deviation of the normal distribution (both on mean difference scale).
\item \code{likelihoodRatioDistribution="exp"} uses an exponential prior for the effect size and requires the parameter \code{kappaLR} which is the mean of the exponential distribution (on the mean difference scale).
\item \code{likelihoodRatioDistribution="unif"} uses a uniform prior for the effect size and requires the specification of \code{deltaMaxLR}, which is the maximum of the support for the uniform likelihood ratio distribution (on the mean difference scale).
\item \code{likelihoodRatioDistribution="maxlr"} estimates the non-centrality parameter to be used for the likelihood ratio from the data. No additional parameters must be specified.
}
The default is \code{likelihoodRatioDistribution=NULL}.
In this case, the likelihood ratio distribution under which the expected second-stage information is calculated is taken directly from the design object.}

\item{...}{{Additional parameters required for the specification of \code{likelihoodRatioDistribution}}.}
}
\value{
Expected second-stage information.
}
\description{
Calculate the expected second-stage information using the optimal conditional error function with specific assumptions.
}
\details{
{The expected second-stage information is calculated as:
\deqn{\mathbb{E}(I_{2})=\int_{\alpha_1}^{\alpha_0}\frac{\nu(\alpha_2(p_1)) \cdot l(p_1)}{\Delta_1^2} dp_1,}
where
\itemize{
\item \eqn{\alpha_1, \alpha_0} are the first-stage efficacy and futility boundaries
\item \eqn{\alpha_2(p_1)} is the optimal conditional error calculated for \eqn{p_1}
\item \eqn{l(p_1)} is the "true" likelihood ratio under which to calculate the expected sample size. This can be different from the likelihood ratio used to calibrate the optimal conditional error function.
\item \eqn{\Delta_1} is the assumed treatment effect to power for, expressed as a mean difference. It may depend on the interim data (i.e., \eqn{p_1}) in case \code{useInterimEstimate = TRUE} was specified for the design object.
\item \eqn{\nu(\alpha_2(p_1)) = (\Phi^{-1}(1-\alpha_2(p_1))+\Phi^{-1}(CP))^2} is a factor calculated for the specific assumptions about the optimal conditional error function and the target conditional power \eqn{CP}.
}}
}
\examples{
# Get a design
design <- getDesignOptimalConditionalErrorFunction(
alpha = 0.025, alpha1 = 0.001, alpha0 = 0.5, conditionalPower = 0.9,
delta1 = 0.25, likelihoodRatioDistribution = "fixed", deltaLR = 0.25,
firstStageInformation = 80, useInterimEstimate = FALSE,
)
# Calculate expected information under correct specification
getExpectedSecondStageInformation(design)

# Calculate expected information under the null hypothesis
getExpectedSecondStageInformation(
 design = design, likelihoodRatioDistribution = "fixed", deltaLR = 0
)

}
\references{
Brannath, W. & Bauer, P. (2004). Optimal conditional error functions for the control of conditional power. Biometrics. https://www.jstor.org/stable/3695393
}
\seealso{
\code{\link[=getDesignOptimalConditionalErrorFunction]{getDesignOptimalConditionalErrorFunction()}}, \code{\link[=getSecondStageInformation]{getSecondStageInformation()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getConstraintC_max.R
\name{getConstraintC_max}
\alias{getConstraintC_max}
\title{Calculates the upper constraint.}
\usage{
getConstraintC_max(firstStagePValue, design)
}
\arguments{
\item{firstStagePValue}{First-stage p-value or p-values. Must be a numeric vector between 0 and 1.}

\item{design}{An object of class \code{TrialDesignOptimalConditionalError} created by \code{getDesignOptimalConditionalErrorFunction()}. Contains all necessary arguments to calculate the optimal conditional error function for the specified case.}
}
\value{
Value of the upper constraint
}
\description{
Calculates the upper constraint.
}
\details{
Internal function that calculates the upper constraint.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integrateExpectedInformation.R
\name{integrateExpectedInformation}
\alias{integrateExpectedInformation}
\title{Integrate over information}
\usage{
integrateExpectedInformation(
  firstStagePValue,
  design,
  likelihoodRatioDistribution,
  ...
)
}
\arguments{
\item{firstStagePValue}{First-stage p-value or p-values. Must be a numeric vector between 0 and 1.}

\item{design}{An object of class \code{TrialDesignOptimalConditionalError} created by \code{getDesignOptimalConditionalErrorFunction()}. Contains all necessary arguments to calculate the optimal conditional error function for the specified case.}

\item{likelihoodRatioDistribution}{The distribution to be used for the effect size of the likelihood ratio in the optimal conditional error function. Options are \code{"fixed", "normal", "exp", "unif", "maxlr"} for fixed effect size, normally distributed, exponentially distributed, uniformly distributed prior of the effect size and maximum likelihood ratio, respectively.
Each case requires different additional specifications: \cr
\itemize{
\item \code{likelihoodRatioDistribution="fixed"} uses one (or more) fixed effect sizes for the likelihood ratio and requires the parameter \code{deltaLR} which provides the mean difference under which to calculate the likelihood ratio. If \code{deltaLR} contains multiple values, they may be weighted using an additional argument \code{weightsDeltaLR}. Omitting \code{weightsDeltaLR} automatically leads to equal weighting.
\item \code{likelihoodRatioDistribution="normal"} uses a normal prior for the effect size and requires parameters \code{deltaLR} and \code{tauLR} for the mean and standard deviation of the normal distribution (both on mean difference scale).
\item \code{likelihoodRatioDistribution="exp"} uses an exponential prior for the effect size and requires the parameter \code{kappaLR} which is the mean of the exponential distribution (on the mean difference scale).
\item \code{likelihoodRatioDistribution="unif"} uses a uniform prior for the effect size and requires the specification of \code{deltaMaxLR}, which is the maximum of the support for the uniform likelihood ratio distribution (on the mean difference scale).
\item \code{likelihoodRatioDistribution="maxlr"} estimates the non-centrality parameter to be used for the likelihood ratio from the data. No additional parameters must be specified.
}}

\item{...}{Additional arguments needed for \code{getOptimalConditionalError()} and \code{getLikelihoodRatio()}.}
}
\value{
Integral over the information of the second stage
}
\description{
Internal function used by \code{getExpectedSecondStageInformation()} to calculate the integral over the information.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\docType{class}
\name{TrialDesignOptimalConditionalError}
\alias{TrialDesignOptimalConditionalError}
\title{Optimal Conditional Error Design}
\description{
A class for a trial design object using the optimal conditional error function.
}
\details{
This object should not be created directly; use \code{getDesignOptimalConditionalErrorFunction()} with suitable arguments to create a design.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{print.TrialDesignOptimalConditionalError}
\alias{print.TrialDesignOptimalConditionalError}
\title{Print optimal conditional error trial design}
\usage{
\method{print}{TrialDesignOptimalConditionalError}(x, ...)
}
\arguments{
\item{x}{Design object of class \code{TrialDesignOptimalConditionalError}}

\item{...}{Additional arguments required for generic compatibility}
}
\value{
No return value, prints the design.
}
\description{
Print an overview of the specified design parameters.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{print.SimulationResultsOptimalConditionalError}
\alias{print.SimulationResultsOptimalConditionalError}
\title{Print simulation results}
\usage{
\method{print}{SimulationResultsOptimalConditionalError}(x, ...)
}
\arguments{
\item{x}{Simulation results object of class \code{SimulationResultsOptimalConditionalError}}

\item{...}{Additional arguments required for generic compatibility}
}
\value{
No return value, prints the simulation results.
}
\description{
Print an overview of simulation results.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\docType{class}
\name{SimulationResultsOptimalConditionalError}
\alias{SimulationResultsOptimalConditionalError}
\title{Simulation results for optimal conditional error design}
\description{
A class for simulation results of the optimal conditional error function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getMonotoneFunction.R
\name{getMonotoneFunction}
\alias{getMonotoneFunction}
\title{Return Monotone Function Values}
\usage{
getMonotoneFunction(
  x,
  fun,
  lower = NULL,
  upper = NULL,
  argument = NULL,
  nSteps = 10^4,
  epsilon = 10^(-5),
  numberOfIterationsQ = 10^4,
  design
)
}
\arguments{
\item{x}{Argument values.}

\item{fun}{The function to be made monotone.}

\item{lower}{The lower limit of the interval on which the function should be monotonised. Must be a numeric value.}

\item{upper}{The upper limit of the interval on which the function should be monotonised.}

\item{argument}{The argument in which the function should be monotonised, given as a character.}

\item{nSteps}{The number of steps to be taken when checking the function for monotonicity. Must be a numeric value. Default 10^4.}

\item{epsilon}{Maximum allowed difference between the initial and monotone integral. Must be a numeric value. Default 10^-5.}

\item{numberOfIterationsQ}{Maximum number of iterations allowed to determine each value of q. Must be a numeric value. Default 10^4.}

\item{design}{An object of class \code{TrialDesignOptimalConditionalError} created by \code{getDesignOptimalConditionalErrorFunction()}. Contains all necessary arguments to calculate the optimal conditional error function for the specified case.}
}
\value{
Monotone function values.
}
\description{
Applies the provided monotonisation constants to a specified, possibly non-monotone function. The returned function values are non-increasing.
}
\details{
The exact monotonisation process is outlined in Brannath et al. (2024), but specified in terms of the first-stage test statistic \eqn{z_1} rather than the first-stage p-value \eqn{p_1}. \cr
The algorithm can easily be translated to the use of p-values by switching the maximum and minimum functions, i.e., replacing \eqn{\min\{q, Q(z_1)\}} by \eqn{\max\{q, Q(p_1)\}} and \eqn{\min\{q, Q(z_1)\}} by \eqn{\max\{q, Q(p_1\}}.
}
\references{
Brannath, W., Dreher, M., zur Verth, J., Scharpenberg, M. (2024). Optimal monotone conditional error functions. https://arxiv.org/abs/2402.00814
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getOverallPower.R
\name{getOverallPower}
\alias{getOverallPower}
\title{Calculate the overall power}
\usage{
getOverallPower(design, alternative)
}
\arguments{
\item{design}{An object of class \code{TrialDesignOptimalConditionalError} created by \code{getDesignOptimalConditionalErrorFunction()}. Contains all necessary arguments to calculate the optimal conditional error function for the specified case.}

\item{alternative}{Assumed relative effect size.}
}
\value{
The overall power of the design at the provided effect size.
}
\description{
Calculate the overall power and other operating characteristics of a design.
}
\details{
This function is used to evaluate the overall performance of a design.
The probabilities for first-stage futility, first-stage efficacy and overall efficacy (i.e., overall power) are saved in an object of class \code{PowerResultsOptimalConditionalError}.
}
\examples{
# Get a design
design <- getDesignOptimalConditionalErrorFunction(
alpha = 0.025, alpha1 = 0.001, alpha0 = 0.5, conditionalPower = 0.9,
delta1 = 0.25, likelihoodRatioDistribution = "fixed", deltaLR = 0.25,
firstStageInformation = 80, useInterimEstimate = FALSE,
)

getOverallPower(design, alternative = 0.25)

}
\seealso{
\code{\link[=getDesignOptimalConditionalErrorFunction]{getDesignOptimalConditionalErrorFunction()}}, \code{\link[=getSimulationResults]{getSimulationResults()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getMonotonisationConstants.R
\name{getMonotonisationConstants}
\alias{getMonotonisationConstants}
\title{Calculate the Constants for Monotonisation}
\usage{
getMonotonisationConstants(
  fun,
  lower = 0,
  upper = 1,
  argument,
  nSteps = 10^4,
  epsilon = 10^(-5),
  numberOfIterationsQ = 10^4,
  design
)
}
\arguments{
\item{fun}{The function to be made monotone.}

\item{lower}{The lower limit of the interval on which the function should be monotonised. Must be a numeric value.}

\item{upper}{The upper limit of the interval on which the function should be monotonised.}

\item{argument}{The argument in which the function should be monotonised, given as a character.}

\item{nSteps}{The number of steps to be taken when checking the function for monotonicity. Must be a numeric value. Default 10^4.}

\item{epsilon}{Maximum allowed difference between the initial and monotone integral. Must be a numeric value. Default 10^-5.}

\item{numberOfIterationsQ}{Maximum number of iterations allowed to determine each value of q. Must be a numeric value. Default 10^4.}

\item{design}{An object of class \code{TrialDesignOptimalConditionalError} created by \code{getDesignOptimalConditionalErrorFunction()}. Contains all necessary arguments to calculate the optimal conditional error function for the specified case.}
}
\value{
A list containing the monotonisation constants (element \code{$qs}) and the intervals on which they must be applied, specified via minimum (element \code{qls}) and maximum (element \code{qus}).
}
\description{
Computes the constants required to make a function non-increasing on the specified interval. The output of this function is necessary to calculate the monotone optimal conditional error function.
The output object is a list that contains the intervals on which constant values are required, specified by the minimum \code{dls} and maximum \code{dus} of the interval and the respective constants, \code{qs}.
}
\references{
Brannath, W., Dreher, M., zur Verth, J., Scharpenberg, M. (2024). Optimal monotone conditional error functions. https://arxiv.org/abs/2402.00814
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDesignOptimalConditionalErrorFunction.R
\name{getDesignOptimalConditionalErrorFunction}
\alias{getDesignOptimalConditionalErrorFunction}
\title{Create a design object for the optimal conditional error function.}
\usage{
getDesignOptimalConditionalErrorFunction(
  alpha,
  alpha1,
  alpha0,
  conditionalPower = NA_real_,
  delta1 = NA_real_,
  delta1Min = NA_real_,
  delta1Max = Inf,
  ncp1 = NA_real_,
  ncp1Min = NA_real_,
  ncp1Max = Inf,
  useInterimEstimate = TRUE,
  firstStageInformation,
  likelihoodRatioDistribution,
  minimumSecondStageInformation = 0,
  maximumSecondStageInformation = Inf,
  minimumConditionalError = 0,
  maximumConditionalError = 1,
  conditionalPowerFunction = NA,
  levelConstantMinimum = 0,
  levelConstantMaximum = 10,
  enforceMonotonicity = TRUE,
  ...
)
}
\arguments{
\item{alpha}{The overall type I error rate \eqn{\alpha} of the design. Must be a numeric value between 0 and 1.}

\item{alpha1}{Stage 1 efficacy boundary \eqn{\alpha_1} (p-value scale). Must be a numeric value between 0 and 1. Should be smaller than \code{alpha0}.}

\item{alpha0}{Binding stage 1 futility boundary \eqn{\alpha_0} (p-value scale). Must be a numeric value between 0 and 1. Should be greater than \code{alpha1}. For use of a non-binding futility boundary, specify \code{alpha0=1}.}

\item{conditionalPower}{The target conditional power \eqn{CP} of the design. Must be a numeric value.}

\item{delta1}{Fixed effect assumption at which the conditional power should be achieved, expressed on the mean difference scale. Is only used if \code{useInterimEstimate=FALSE}. Alternatively, \code{ncp1} can be specified. Must be a numeric value greater than 0.}

\item{delta1Min}{The minimum for an interim estimate of the treatment effect, specified on the mean difference scale. If the interim estimate (on the mean difference scale) yields a value smaller than \code{delta1Min}, \code{delta1Min} is used for it. Is only used if \code{useInterimEstimate=TRUE}. Alternatively, \code{ncp1Min} can be specified. Must be a numeric value.}

\item{delta1Max}{The maximum for an interim estimate of the treatment effect, specified on the mean difference scale. If the interim estimate (on the mean difference scale) yields a value larger than \code{delta1Max}, \code{delta1Max} is used for it. Is only used if \code{useInterimEstimate=TRUE}. Alternatively, \code{ncp1Max} can be specified. Must be a numeric value. Default value is \code{Inf}, i.e., no upper restriction.}

\item{ncp1}{Fixed effect assumption at which the conditional power should be achieved, expressed on the non-centrality parameter scale. Is only used if \code{useInterimEstimate=FALSE}. Alternatively, \code{delta1} can be specified. Must be a numeric value greater than 0.}

\item{ncp1Min}{The minimum for an interim estimate of the treatment effect, specified on the non-centrality parameter scale. If the interim estimate (on the non-centrality parameter scale) yields a value smaller than \code{ncp1Min}, \code{ncp1Min} is used for it. Is only used if \code{useInterimEstimate=TRUE}. Alternatively, \code{delta1Min} can be specified. Must be a numeric value.}

\item{ncp1Max}{The maximum for an interim estimate of the treatment effect, specified on the non-centrality parameter scale. If the interim estimate (on the non-centrality parameter scale) yields a value larger than \code{ncp1Max}, \code{ncp1Max} is used for it. Is only used if \code{useInterimEstimate=TRUE}. Alternatively, \code{delta1Max} can be specified. Must be a numeric value. Default value is \code{Inf}, i.e., no upper restriction.}

\item{useInterimEstimate}{Logical. Defines whether or not an interim estimate should be used for conditional power.
If \code{TRUE}, a lower cut-off for the interim estimate must be specified by \code{delta1Min} or \code{ncp1Min}. An upper cut-off may also
be specified by \code{delta1Max} or \code{ncp1Max}. If \code{FALSE}, the fixed effect size must be specified by \code{delta1} or \code{ncp1}.}

\item{firstStageInformation}{Information of the first stage of the trial. Must be a positive numeric value.}

\item{likelihoodRatioDistribution}{The distribution to be used for the effect size of the likelihood ratio in the optimal conditional error function. Options are \code{"fixed", "normal", "exp", "unif", "maxlr"} for fixed effect size, normally distributed, exponentially distributed, uniformly distributed prior of the effect size and maximum likelihood ratio, respectively.
Each case requires different additional specifications: \cr
\itemize{
\item \code{likelihoodRatioDistribution="fixed"} uses one (or more) fixed effect sizes for the likelihood ratio and requires the parameter \code{deltaLR} which provides the mean difference under which to calculate the likelihood ratio. If \code{deltaLR} contains multiple values, they may be weighted using an additional argument \code{weightsDeltaLR}. Omitting \code{weightsDeltaLR} automatically leads to equal weighting.
\item \code{likelihoodRatioDistribution="normal"} uses a normal prior for the effect size and requires parameters \code{deltaLR} and \code{tauLR} for the mean and standard deviation of the normal distribution (both on mean difference scale).
\item \code{likelihoodRatioDistribution="exp"} uses an exponential prior for the effect size and requires the parameter \code{kappaLR} which is the mean of the exponential distribution (on the mean difference scale).
\item \code{likelihoodRatioDistribution="unif"} uses a uniform prior for the effect size and requires the specification of \code{deltaMaxLR}, which is the maximum of the support for the uniform likelihood ratio distribution (on the mean difference scale).
\item \code{likelihoodRatioDistribution="maxlr"} estimates the non-centrality parameter to be used for the likelihood ratio from the data. No additional parameters must be specified.
}}

\item{minimumSecondStageInformation}{The minimum information allowed in the second stage of the trial. Must be a numeric value. Default value is \code{0}, i.e., no restriction.}

\item{maximumSecondStageInformation}{The maximum information allowed in the second stage of the trial. Must be a numeric value. Default value is \code{Inf}, i.e., no restriction.}

\item{minimumConditionalError}{Lower boundary for the optimal conditional error function. Default 0 (no restriction).}

\item{maximumConditionalError}{Upper boundary for the optimal conditional error function. Default value is 1, however, the optimal conditional error function is inherently bounded by the conditional power.}

\item{conditionalPowerFunction}{A user-specified function which calculates the conditional power from the first-stage p-value. This function should not be increasing in the first-stage p-value or monotonicity issues may occur.}

\item{levelConstantMinimum}{The minimum of the interval on which the value for the level constant should be searched. Default value is 0.}

\item{levelConstantMaximum}{The maximum of the interval on which the value for the level constant should be searched. Default value is 10.}

\item{enforceMonotonicity}{Logical. Determines whether or not the optimal conditional error function should automatically be modified to be non-increasing. Default is \code{TRUE}.}

\item{...}{Additional arguments required for the specification of the likelihood ratio.}
}
\value{
An object of class \code{TrialDesignOptimalConditionalError}, which can be passed to other package functions.
}
\description{
This function returns a design object which contains all important parameters for the specification of the optimal conditional error function.
The returned object is of class \code{TrialDesignOptimalConditionalError} and can be passed to other package functions.
}
\details{
The design object contains the information required to determine the specific setting of the optimal conditional error function and can be passed to other package functions.
From the given user specifications, the constant to achieve level condition for control of the overall type I error rate as well as the constants to ensure a non-increasing optimal CEF (if required) are automatically calculated.
}
\section{Likelihood ratio distribution}{

To calculate the optimal conditional error function, an assumption about the true parameter under which the second-stage information is to be minimised is required.
Various options are available and can be specified via the argument \code{likelihoodRatioDistribution}:
\itemize{
\item \code{likelihoodRatioDistribution="fixed"}: calculates the likelihood ratio for a fixed \eqn{\Delta}. The non-centrality parameter of the likelihood ratio \eqn{\vartheta} is then computed as \code{deltaLR}*\code{sqrt(firstStageInformation)} and the likelihood ratio is calculated as:
\deqn{l(p_1) = e^{\Phi^{-1}(1-p_1)\vartheta - \vartheta^2/2}.} \code{deltaLR} may also contain multiple elements, in which case a weighted likelihood ratio is calculated for the given values. Unless positive weights that sum to 1 are provided by the argument \code{weightsDeltaLR}, equal weights are assumed.
\item \code{likelihoodRatioDistribution="normal"}: calculates the likelihood ratio for a normally distributed prior of \eqn{\vartheta} with mean \code{deltaLR}*\code{sqrt(firstStageInformation)} (\eqn{\mu}) and standard deviation \code{tauLR}*\code{sqrt(firstStageInformation)} (\eqn{\sigma}). The parameters \code{deltaLR} and \code{tauLR} must be specified on the mean difference scale.
\deqn{l(p_1) = (1+\sigma^2)^{-\frac{1}{2}}\cdot e^{-(\mu/\sigma)^2/2 + (\sigma\Phi^{-1}(1-p_1) + \mu/\sigma)^2 / (2\cdot (1+\sigma^2))}}
\item \code{likelihoodRatioDistribution="exp"}: calculates the likelihood ratio for an exponentially distributed prior of \eqn{\vartheta} with mean \code{kappaLR}*\code{sqrt(firstStageInformation)} (\eqn{\eta}). The likelihood ratio is then calculated as:
\deqn{l(p_1) = \kappa \cdot \sqrt{2\pi} \cdot e^{(\Phi^{-1}(1-p_1)-\eta)^2/2} \cdot \Phi(\Phi^{-1}(1-p_1)-\eta)}
\item \code{likelihoodRatioDistribution="unif"}: calculates the likelihood ratio for a uniformly distributed prior of \eqn{\vartheta} on the support \eqn{[0, \Delta\cdot\sqrt{I_1}]}, where \eqn{\Delta} is specified as \code{deltaMaxLR} and \eqn{I_1} is the \code{firstStageInformation}.
\deqn{l(p_1) = \frac{\sqrt{2\pi}}{\Delta\cdot\sqrt{I_1}} \cdot e^{\Phi^{-1}(1-p_1)^2/2} \cdot (\Phi(\Delta\cdot\sqrt{I_1} - \Phi^{-1}(1-p_1))-p_1)}
\item \code{likelihoodRatioDistribution="maxlr"}: the non-centrality parameter \eqn{\vartheta} is estimated from the data and no additional parameters must be specified. The likelihood ratio is estimated from the data as:
\deqn{l(p_1) = e^{max(0, \Phi^{-1}(1-p_1))^2/2}}
The maximum likelihood ratio is always restricted to effect sizes \eqn{\vartheta \geq 0} (corresponding to \eqn{p_1 \leq 0.5}).
}
}

\section{Effect for conditional power}{

For the treatment effect at which the target conditional power should be achieved, either a fixed effect or an interim estimate can be used.
The usage of a fixed effect is indicated by setting \code{useInterimEstimate=FALSE}, in which case the fixed effect is provided by \code{delta1} on the mean difference scale or by \code{ncp1} on the non-centrality parameter scale (i.e., \code{delta1*sqrt(firstStageInformation)}).
For an interim estimate, specified by \code{useInterimEstimate=TRUE}, a lower cut-off for the interim estimate must be provided, either by \code{delta1Min} on the mean difference scale, or \code{ncp1Min} on the non-centrality parameter scale.
In addition, an upper limit of the estimate may be analogously provided by \code{delta1Max} or \code{ncp1Max}.
}

\section{Sample size and information}{

The first-stage information of the trial design must be specified to allow for calculations between the mean difference and non-centrality parameter scale.
It is provided to the design object via \code{firstStageInformation}. \cr
Listed below are some examples for the calculation between information (\eqn{I_1}) and sample size:
\itemize{
\item One-sample z-test with \eqn{n} total patients: \eqn{I_1 = \frac{n}{\sigma^2}}, where \eqn{\sigma^2} is the variance of an individual observation

\item Balanced two-sample z-test with \eqn{n_1} patients per group: \eqn{I_1 = \frac{1}{2}\cdot\frac{n_1}{\sigma^2}}, where \eqn{\sigma^2} is the common variance

\item General two-sample z-test with \eqn{n_1}, \eqn{n_2} patients per group: \eqn{I_1 = 1/(\frac{\sigma_1^2}{n_1}+\frac{\sigma_2^2}{n_2})}, where \eqn{\sigma_1^2}, \eqn{\sigma_2^2} are the group-wise variances
}
}

\section{Monotonicity}{

By default, the optimal conditional error function returned by \code{getDesignOptimalConditionalErrorFunction()} is transformed to be non-increasing in the first-stage p-value \eqn{p_1} if found to be increasing on any interval.
The necessary intervals and constants for the transformation are calculated by \code{getMonotonisationConstants()}.
Although not recommended for the operating characteristics of the design, the transformation may be omitted by setting \code{enforceMonotonicity=FALSE}.
}

\section{Constraints}{

In some applications, it may be feasible to restrict the optimal conditional error function by a lower and/or upper limit.
These constraints can be directly implemented on the function by using the arguments \code{minimumConditionalError} and \code{maximumConditionalError}.
By default, \code{minimumConditionalError=0} and \code{maximumConditionalError=1}, i.e., no constraints are applied.
The constraints may also be specified on the second-stage information via \code{minimumSecondStageInformation} and \code{maximumSecondStageInformation}.
If both \code{minimumConditionalError} and \code{maximumSecondStageInformation} respectively \code{maximumConditionalError} and \code{minimumSecondStageInformation}
are provided, both constraints will be applied.
}

\section{Level constant}{

The level constant is determined by the helper function \code{getLevelConstant()}. It is identified using the \code{uniroot()} function and by default, the interval between 0 and 10 is searched for the level constant.
In specific settings, the level constant may lie outside of this interval. In such cases, the search interval can be changed by altering the parameters \code{levelConstantMinimum} and \code{levelConstantMaximum}. \cr
If inappropriate constraints to the optimal conditional error function are provided via \code{minimumConditionalError} and \code{maximumConditionalError}
or \code{minimumSecondStageInformation} and \code{maximumSecondStageInformation}, it may be impossible to find a level constant which exhausts the full alpha level.
}

\section{Generic functions}{

The \code{print()} and \code{plot()} functions are available for objects of class \code{TrialDesignOptimalConditionalError}.
For details, see \code{?print.TrialDesignOptimalConditionalError} and \code{?plot.TrialDesignOptimalConditionalError}.
}

\examples{
\donttest{
# Create a single-arm design with fixed parameter for the likelihood ratio
# and a fixed effect for conditional power. 80 patients are observed in the
# first-stage (firstStageInformation = 80 in the one-sample test, variance 1).
# The second-stage information is restricted to be between 40 and 160.
getDesignOptimalConditionalErrorFunction(
  alpha = 0.025, alpha1 = 0.001, alpha0 = 0.5, conditionalPower = 0.9,
  delta1 = 0.25, likelihoodRatioDistribution = "fixed", deltaLR = 0.25,
  firstStageInformation = 80, useInterimEstimate = FALSE,
  minimumSecondStageInformation = 40, maximumSecondStageInformation = 160
)

# Create a design comparing two groups using the maximum likelihood ratio
# and an interim estimate for the effect for conditional power.
# 160 patients per arm are observed in the first stage
# (firstStageInformation = 80 in the balanced two-sample test, variance 1).
getDesignOptimalConditionalErrorFunction(
  alpha = 0.025, alpha1 = 0.001, alpha0 = 0.5, conditionalPower = 0.9,
  delta1Min = 0.25, likelihoodRatioDistribution = "maxlr",
  firstStageInformation = 80, useInterimEstimate = TRUE
)
}

}
\references{
Brannath, W. & Bauer, P. (2004). Optimal conditional error functions for the control of conditional power. Biometrics. https://www.jstor.org/stable/3695393

Brannath, W., Dreher, M., zur Verth, J., Scharpenberg, M. (2024). Optimal monotone conditional error functions. https://arxiv.org/abs/2402.00814
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getIntegral.R
\name{getIntegral}
\alias{getIntegral}
\title{Internal calculation of the integral over psi}
\usage{
getIntegral(constant, design)
}
\arguments{
\item{constant}{Current value of the level constant for the optimal conditional error function, internally used as integral argument.}

\item{design}{An object of class \code{TrialDesignOptimalConditionalError} created by \code{getDesignOptimalConditionalErrorFunction()}. Contains all necessary arguments to calculate the optimal conditional error function for the specified case.}
}
\value{
Distance of integral over psi to (alpha-alpha1).
}
\description{
Helper function that integrates over the optimal conditional error function. Used to find appropriate level constant.
}
\details{
Internal function called by \code{findLevelConstant()} that should be solved (i.e., root should be found).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getInnerPsi.R
\name{getInnerPsi}
\alias{getInnerPsi}
\title{Internal variation of \code{getPsi()} used by \code{findLevelConstant()}}
\usage{
getInnerPsi(firstStagePValue, constant, design)
}
\arguments{
\item{firstStagePValue}{First-stage p-value, internally used as integral argument.}

\item{constant}{Current value of the level constant for the optimal conditional error function, internally used as integral argument.}

\item{design}{An object of class \code{TrialDesignOptimalConditionalError} created by \code{getDesignOptimalConditionalErrorFunction()}. Contains all necessary arguments to calculate the optimal conditional error function for the specified case.}
}
\value{
Point value of \code{getPsi()}.
}
\description{
Calculate psi for the given scenario, incl. constant
}
\details{
Internal function called by \code{getIntegral()} that calculates a point value of \code{getPsi()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getSimulationResults.R
\name{getSimulationResults}
\alias{getSimulationResults}
\title{Simulate trials}
\usage{
getSimulationResults(
  design,
  maxNumberOfIterations = 10000,
  alternative,
  seed = NULL
)
}
\arguments{
\item{design}{An object of class \code{TrialDesignOptimalConditionalError} created by \code{getDesignOptimalConditionalErrorFunction()}. Contains all necessary arguments to calculate the optimal conditional error function for the specified case.}

\item{maxNumberOfIterations}{Number of trials to be simulated.}

\item{alternative}{Assumed relative effect size.}

\item{seed}{An optional seed for reproducibility.}
}
\value{
An object of class \code{SimulationResultsOptimalConditionalError} containing the simulation results.
}
\description{
Simulate the rejection probability for a given design and alternative.
}
\details{
Simulates the probabilities of overall rejection as well as early futility and early efficacy for the provided scenario and design.
This is done by generating random normally distributed test statistics and calculating their p-values.
}
\examples{
\donttest{
design <- getDesignOptimalConditionalErrorFunction(
 alpha = 0.025, alpha1 = 0.001, alpha0 = 0.5, delta1 = 0.25,
 useInterimEstimate = FALSE,
 conditionalPower = 0.9, likelihoodRatioDistribution = "maxlr",
 firstStageInformation = 10
)

# Simulate under the null hypothesis and for a mean difference of 0.5
getSimulationResults(
 design = design, alternative = c(0, 0.5)
)

}
}
\seealso{
\code{\link[=getDesignOptimalConditionalErrorFunction]{getDesignOptimalConditionalErrorFunction()}}, \code{\link[=getOverallPower]{getOverallPower()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{plot.TrialDesignOptimalConditionalError}
\alias{plot.TrialDesignOptimalConditionalError}
\title{Plot the optimal conditional error function}
\usage{
\method{plot}{TrialDesignOptimalConditionalError}(x, range = c(0, 1), type = 1, plotNonMonotoneFunction = FALSE, ...)
}
\arguments{
\item{x}{Design object of class \code{TrialDesignOptimalConditionalError}.}

\item{range}{Numeric vector with two entries specifying the range of the x-axis of the plot.}

\item{type}{Type of plot to be created. Options are: \itemize{
\item \code{type = 1}: Plot the values of the optimal conditional error function against the first-stage p-value.
\item \code{type = 2}: Plot the second-stage information resulting from the optimal conditional error function against the first-stage p-value.
\item \code{type = 3}: Plot the likelihood ratio of the given specification of the optimal conditional error function against the first-stage p-value.
\item \code{type = 4}: Plot the function Q of the given specification of the optimal conditional error function against the first-stage p-value.
}}

\item{plotNonMonotoneFunction}{Logical. Should the non-monotone version of the plot be drawn? Not applicable for plot type 3. Default: \code{FALSE}.}

\item{...}{Additional arguments required for generic compatibility}
}
\value{
No return value, plots the design.
}
\description{
The returned plot is a \code{ggplot2} object and can be supplemented with additional layers using \code{ggplot2} commands.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getQ.R
\name{getQ}
\alias{getQ}
\title{Calculate Q}
\usage{
getQ(firstStagePValue, design)
}
\arguments{
\item{firstStagePValue}{First-stage p-value or p-values. Must be a numeric vector between 0 and 1.}

\item{design}{An object of class \code{TrialDesignOptimalConditionalError} created by \code{getDesignOptimalConditionalErrorFunction()}. Contains all necessary arguments to calculate the optimal conditional error function for the specified case.}
}
\value{
Ratio of likelihood ratio and squared effect size.
}
\description{
Calculate the ratio of likelihood ratio and squared effect size.
}
\details{
For more information on how to specify the likelihood ratio, see \code{?getLikelihoodRatio()}.
In case the optimal conditional error function is ever increasing in the first-stage p-value \eqn{p_1}, a monotone transformation of \code{getQ()}
is needed for logical consistency and type I error rate control. \cr
The formula for \eqn{Q(p_1)} is:
\deqn{Q(p_1) = l(p_1) / \Delta_1^2,}
where \eqn{l(p_1)} is the likelihood ratio and \eqn{\Delta_1} is the effect size at which the conditional power should be achieved.
The effect size may also depend on the interim data (i.e., on \eqn{p_1}) in case \code{useInterimEstimate = TRUE} was specified for the design object.
}
\examples{
# Get a design
design <- getDesignOptimalConditionalErrorFunction(
alpha = 0.025, alpha1 = 0.001, alpha0 = 0.5, conditionalPower = 0.9,
delta1 = 0.25, likelihoodRatioDistribution = "fixed", deltaLR = 0.25,
firstStageInformation = 80, useInterimEstimate = FALSE,
)

getQ(firstStagePValue = c(0.05, 0.1, 0.2), design = design)

}
\references{
Brannath, W., Dreher, M., zur Verth, J., Scharpenberg, M. (2024). Optimal monotone conditional error functions. https://arxiv.org/abs/2402.00814
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getNuPrime.R
\name{getNuPrime}
\alias{getNuPrime}
\title{Calculate the Derivate of Nu}
\usage{
getNuPrime(alpha, conditionalPower)
}
\arguments{
\item{alpha}{The (conditional) type I error rate of the design. Must be a numeric vector with values between 0 and 1.}

\item{conditionalPower}{The target conditional power \eqn{CP} of the design. Must be a numeric value.}
}
\value{
Value for nu prime.
}
\description{
Calculates the derivative of nu for a given conditional error and conditional power.
}
\details{
The function \eqn{\nu'} is defined as
\deqn{\nu'(p_1) = -2 \cdot (\Phi^{-1}(1-\alpha_2(p_1)) + \Phi^{-1}(CP))/\phi(\Phi^{-1}(1-\alpha_2(p_1))).}
Note that in this implementation, the the factor -2 is used instead of -4, which is used in by Brannath & Bauer (2004), who explicitly investigate the setting of a balanced two-group trial.
The argument \code{conditionalPower} is either the fixed target conditional power or the value of the conditional power function at the corresponding first-stage p-value.
}
\examples{
getNuPrime(alpha = 0.05, conditionalPower = 0.9)

}
\references{
Brannath, W. & Bauer, P. (2004). Optimal conditional error functions for the control of conditional power. Biometrics. https://www.jstor.org/stable/3695393
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getLikelihoodRatio.R
\name{getLikelihoodRatio}
\alias{getLikelihoodRatio}
\title{Calculate Likelihood Ratio}
\usage{
getLikelihoodRatio(firstStagePValue, design)
}
\arguments{
\item{firstStagePValue}{First-stage p-value or p-values. Must be a numeric vector between 0 and 1.}

\item{design}{An object of class \code{TrialDesignOptimalConditionalError} created by \code{getDesignOptimalConditionalErrorFunction()}. Contains all necessary arguments to calculate the optimal conditional error function for the specified case.}
}
\value{
The value of the likelihood ratio for the given specification.
}
\description{
Calculate the likelihood ratio of a p-value for a given distribution.
}
\details{
The calculation of the likelihood ratio for a first-stage p-value \eqn{p_1} is done based on a distributional assumption, specified in the \code{design} object.
The different options require different parameters, elaborated in the following.
\itemize{
\item \code{likelihoodRatioDistribution="fixed"}: calculates the likelihood ratio for a fixed \eqn{\Delta}. The non-centrality parameter of the likelihood ratio \eqn{\vartheta} is then computed as \code{deltaLR}*\code{sqrt(firstStageInformation)} and the likelihood ratio is calculated as:
\deqn{l(p_1) = e^{\Phi^{-1}(1-p_1)\vartheta - \vartheta^2/2}.} \code{deltaLR} may also contain multiple elements, in which case a weighted likelihood ratio is calculated for the given values. Unless positive weights that sum to 1 are provided by the argument \code{weightsDeltaLR}, equal weights are assumed.
\item \code{likelihoodRatioDistribution="normal"}: calculates the likelihood ratio for a normally distributed prior of \eqn{\vartheta} with mean \code{deltaLR}*\code{sqrt(firstStageInformation)} (\eqn{\mu}) and standard deviation \code{tauLR}*\code{sqrt(firstStageInformation)} (\eqn{\sigma}). The parameters \code{deltaLR} and \code{tauLR} must be specified on the mean difference scale.
\deqn{l(p_1) = (1+\sigma^2)^{-\frac{1}{2}}\cdot e^{-(\mu/\sigma)^2/2 + (\sigma\Phi^{-1}(1-p_1) + \mu/\sigma)^2 / (2\cdot (1+\sigma^2))}}
\item \code{likelihoodRatioDistribution="exp"}: calculates the likelihood ratio for an exponentially distributed prior of \eqn{\vartheta} with mean \code{kappaLR}*\code{sqrt(firstStageInformation)} (\eqn{\eta}). The likelihood ratio is then calculated as:
\deqn{l(p_1) = \eta \cdot \sqrt{2\pi} \cdot e^{(\Phi^{-1}(1-p_1)-\eta)^2/2} \cdot \Phi(\Phi^{-1}(1-p_1)-\eta)}
\item \code{likelihoodRatioDistribution="unif"}: calculates the likelihood ratio for a uniformly distributed prior of \eqn{\vartheta} on the support \eqn{[0, \Delta\cdot\sqrt{I_1}]}, where \eqn{\Delta} is specified as \code{deltaMaxLR} and \eqn{I_1} is the \code{firstStageInformation}.
\deqn{l(p_1) = \frac{\sqrt{2\pi}}{\Delta\cdot\sqrt{I_1}} \cdot e^{\Phi^{-1}(1-p_1)^2/2} \cdot (\Phi(\Delta\cdot\sqrt{I_1} - \Phi^{-1}(1-p_1))-p_1)}
\item \code{likelihoodRatioDistribution="maxlr"}: the non-centrality parameter \eqn{\vartheta} is estimated from the data and no additional parameters must be specified. The likelihood ratio is estimated from the data as:
\deqn{l(p_1) = e^{max(0, \Phi^{-1}(1-p_1))^2/2}}
The maximum likelihood ratio is always restricted to effect sizes \eqn{\vartheta \geq 0} (corresponding to \eqn{p_1 \leq 0.5}).
}
}
\examples{
# Get a design
design <- getDesignOptimalConditionalErrorFunction(
alpha = 0.025, alpha1 = 0.001, alpha0 = 0.5, conditionalPower = 0.9,
delta1 = 0.25, likelihoodRatioDistribution = "fixed", deltaLR = 0.25,
firstStageInformation = 80, useInterimEstimate = FALSE,
)

getLikelihoodRatio(firstStagePValue = c(0.05, 0.1, 0.2), design = design)

}
\references{
Brannath, W. & Bauer, P. (2004). Optimal conditional error functions for the control of conditional power. Biometrics. https://www.jstor.org/stable/3695393

Hung, H. M. J., O’Neill, R. T., Bauer, P. & Kohne, K. (1997). The behavior of the p-value when the alternative hypothesis is true. Biometrics. https://www.jstor.org/stable/2533093
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getSecondStageInformation.R
\name{getSecondStageInformation}
\alias{getSecondStageInformation}
\title{Calculate the Second-stage Information}
\usage{
getSecondStageInformation(firstStagePValue, design)
}
\arguments{
\item{firstStagePValue}{First-stage p-value or p-values. Must be a numeric vector between 0 and 1.}

\item{design}{An object of class \code{TrialDesignOptimalConditionalError} created by \code{getDesignOptimalConditionalErrorFunction()}. Contains all necessary arguments to calculate the optimal conditional error function for the specified case.}
}
\value{
The second-stage information.
}
\description{
Calculate second-stage information for given first-stage p-value and design.
}
\details{
The second-stage information \eqn{I_{2}} is calculated given a first-stage p-value \eqn{p_1} as:
\deqn{I_{2}(p_1) = \frac{(\Phi^{-1}(1-\alpha_2(p_1)) + \Phi^{-1}(CP))^2}{\Delta_1^2} = \frac{\nu(\alpha_2(p_1))}{\Delta_1^2},}
where
\itemize{
\item \eqn{\alpha_2(p_1)} is the conditional error function
\item \eqn{CP} is the target conditional power
\item \eqn{\Delta_1} is the assumed treatment effect (expressed as a mean difference).
}
The conditional error is calculated according to the specification provided in the \code{design} argument.
For p-values smaller or equal to the first-stage efficacy boundary as well as p-values greater than the first-stage futility boundary,
the returned information is 0 (since the trial is ended early in both cases).
}
\examples{
design <- getDesignOptimalConditionalErrorFunction(
  alpha = 0.025, alpha1 = 0.001, alpha0 = 0.5,
  conditionalPower = 0.9, delta1 = 0.25, useInterimEstimate = FALSE,
  firstStageInformation = 40, likelihoodRatioDistribution = "maxlr"
)

getSecondStageInformation(
  firstStagePValue = c(0.05, 0.1, 0.2), design = design
)

}
\references{
Brannath, W. & Bauer, P. (2004). Optimal conditional error functions for the control of conditional power. Biometrics. https://www.jstor.org/stable/3695393
}
\seealso{
\code{\link[=getDesignOptimalConditionalErrorFunction]{getDesignOptimalConditionalErrorFunction()}}, \code{\link[=getExpectedSecondStageInformation]{getExpectedSecondStageInformation()}}, \code{\link[=getOptimalConditionalError]{getOptimalConditionalError()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getPsi.R
\name{getPsi}
\alias{getPsi}
\title{Calculate Psi, the Inverse of Nu Prime}
\usage{
getPsi(nuPrime, conditionalPower, design, firstStagePValue)
}
\arguments{
\item{nuPrime}{The function value to be inverted.}

\item{conditionalPower}{The target conditional power \eqn{CP} of the design. Must be a numeric value.}

\item{design}{An object of class \code{TrialDesignOptimalConditionalError} created by \code{getDesignOptimalConditionalErrorFunction()}. Contains all necessary arguments to calculate the optimal conditional error function for the specified case.}

\item{firstStagePValue}{First-stage p-value or p-values. Must be a numeric vector between 0 and 1.}
}
\value{
The value of alpha which corresponds to nuPrime and lies between 0 and \code{conditionalPower}.
}
\description{
Get point-wise values of psi (inverse of nu prime)
}
\details{
The function \eqn{\psi} is the inverse of:
\deqn{\nu'(\alpha) = -2 \cdot(\Phi^{-1}(1-\alpha) + \Phi^{-1}(1-CP)) / \phi(\Phi^{-1}(1-\alpha))}.
If the conditional power \eqn{CP} lies outside of the range \eqn{1-\Phi(2) \leq CP \leq \Phi(2)}, the calculation is slightly more complicated (design and firstStagePValue need to be provided).
The argument \code{conditionalPower} is either the fixed target conditional power or the value of the conditional power function at the corresponding first-stage p-value.
}
\examples{
# Returns 0.05
getPsi(getNuPrime(alpha = 0.05, conditionalPower = 0.9), conditionalPower = 0.9,
  design = NULL, firstStagePValue = NULL)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{.rangeCheck}
\alias{.rangeCheck}
\title{Simple range check for numeric variables}
\usage{
.rangeCheck(variable, range, allowedEqual, hint = "")
}
\arguments{
\item{variable}{The (named) variable to be checked.}

\item{range}{A vector of length 2 giving the minimum and maximum of the allowed range.}

\item{allowedEqual}{Logical. Are the borders of range valid values?}

\item{hint}{Additional message that may be printed after the error.}
}
\value{
Invisibly returns \code{TRUE} if the check was successful.
}
\description{
This function performs very basic range checks for numeric variables and throws
an error if the range is violated. A custom hint may be added to the message.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getIntegralWithConstants.R
\name{getIntegralWithConstants}
\alias{getIntegralWithConstants}
\title{Calculate the integral over a partially constant function}
\usage{
getIntegralWithConstants(constant, design)
}
\arguments{
\item{constant}{Current value of the level constant for the optimal conditional error function, internally used as integral argument.}

\item{design}{An object of class \code{TrialDesignOptimalConditionalError} created by \code{getDesignOptimalConditionalErrorFunction()}. Contains all necessary arguments to calculate the optimal conditional error function for the specified case.}
}
\value{
Integral over the partially constant function.
}
\description{
This function aims to provide a more accurate integration routine than the \code{integrate()} function
for a partially constant function.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{print.PowerResultsOptimalConditionalError}
\alias{print.PowerResultsOptimalConditionalError}
\title{Print power results}
\usage{
\method{print}{PowerResultsOptimalConditionalError}(x, ...)
}
\arguments{
\item{x}{Power results object of class \code{PowerResultsOptimalConditionalError}}

\item{...}{Additional arguments required for generic compatibility}
}
\value{
No return value, prints the power calculation results.
}
\description{
Print an overview of exact power results.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getLevelConstant.R
\name{getLevelConstant}
\alias{getLevelConstant}
\title{Get Level Constant for Optimal Conditional Error Function}
\usage{
getLevelConstant(design)
}
\arguments{
\item{design}{An object of class \code{TrialDesignOptimalConditionalError} created by \code{getDesignOptimalConditionalErrorFunction()}. Contains all necessary arguments to calculate the optimal conditional error function for the specified case.}
}
\value{
A list that contains the constant (element \code{$root}) and other components provided by \code{uniroot()}.
The level constant is calculated corresponding to the mean difference scale.
}
\description{
Find the constant required such that the conditional error function meets the overall level condition.
}
\details{
The level condition is defined as:
\deqn{\alpha = \alpha_1 + \int_{\alpha_1}^{\alpha_0} \alpha_2(p_1)dp_1.}
The constant \eqn{c_0} of the optimal conditional error function is calibrated such that it meets the level condition.
For a valid design, the additional following condition must be met to be able to exhaust the level \eqn{\alpha}:
\deqn{\alpha_1 + CP(\alpha_0-\alpha_1)>\alpha.}
This condition is checked by \code{getLevelConstant()} and the execution is terminated if it is not met. \cr
In case a conditional power function is used, the condition is instead:
\deqn{\alpha_1 + \int_{\alpha_1}^{\alpha_0} CP(p_1)dp_1>\alpha.}
}
\references{
Brannath, W. & Bauer, P. (2004). Optimal conditional error functions for the control of conditional power. Biometrics. https://www.jstor.org/stable/3695393

Brannath, W., Dreher, M., zur Verth, J., Scharpenberg, M. (2024). Optimal monotone conditional error functions. https://arxiv.org/abs/2402.00814
}
